# -*- encoding: utf-8 -*-
##############################################################################
#
#    Avanzosc - Avanced Open Source Consulting
#    Copyright (C) 2011 - 2012 Avanzosc <http://www.avanzosc.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import netsvc
from datetime import datetime
from tools.translate import _

from osv import osv
from osv import fields

class mrp_production(osv.osv):

    _inherit = 'mrp.production'
    
    _columns = {
        'qty_per_lot': fields.float('Qty per Lot', states={'draft':[('readonly',False)]}),
    }
    
    def onchange_product_qty(self, cr, uid, ids, qty, context=None):
        res = {}
        lot_list = {}
        if qty:
            res = {
                'qty_per_lot': qty
            }
        return {'value': res}
    
    def create_lot(self, cr, uid, ids, product_id, production_id, context=None):
        product_obj = self.pool.get('product.product')
        product = product_obj.browse(cr, uid, product_id)
        name = self.pool.get('ir.sequence').get(cr, uid, product.lot_sequence.code)
        if not name:
            raise osv.except_osv(_('Lot error'), _('%s needs production lot ! \
                            \nNo lot found for this product!') % (product.name))
        data = {
            'name': name,
            'product_id': product.id,
        }
        return self.pool.get('stock.production.lot').create(cr, uid, data)
    
    def get_lotlist(self, cr, uid, args, context=None):
        lot_list = {}
        ids = False
        
        inventory_obj = self.pool.get('report.stock.inventory')
        if args:
            ids = inventory_obj.search(cr, uid, args)
        else:
            return {}
        for inventory in inventory_obj.browse(cr, uid, ids):
            if inventory.prodlot_id and inventory.state == 'done':
                if str(inventory.prodlot_id.id) in lot_list:
                    lot_list[str(inventory.prodlot_id.id)] += inventory.product_qty
                else:
                    lot_list.update({str(inventory.prodlot_id.id): inventory.product_qty})
                    
        if lot_list:   
            for lot_id in lot_list.keys():
                if lot_list[lot_id] == 0:
                    del lot_list[lot_id]
                
        return lot_list
 
    def get_lot_auto(self, cr, uid, product_id, location_id, qty, context=None):
        product = self.pool.get('product.product').browse(cr, uid, product_id)
        lot_list = {}
        res = []
        
        lot_list = self.get_lotlist(cr, uid, [('product_id', '=', product_id), ('location_id', '=', location_id)], context)
       
        if not lot_list:
            if product.track_production:
                raise osv.except_osv(_('Stock error'), _('%s needs production lots ! \
                            \nNo lot founded for this product!') % (product.name))
            return False
        
        while qty > 0:
            cur_lot = 0
            if not lot_list and qty > 0:
                raise osv.except_osv(_('Stock error'), _('There is no enough stock for %s ! \
                            \n%s %s(s) missing!') % (product.name, round(qty,3), product.uom_id.name))
            elif product.lot_type_in == 'lifo':
                cur_lot = self.lifo_lot(cr, uid, lot_list, qty, context)
            elif product.lot_type_in == 'fifo':
                cur_lot = self.fifo_lot(cr, uid, lot_list, qty, context)
                
            if lot_list[cur_lot] <= qty:
                qty -= lot_list[cur_lot]
                res.append({cur_lot: lot_list[cur_lot]})
                del lot_list[cur_lot]
            else:
                res.append({cur_lot: qty})
                qty = 0
        
        return res
    
    def fifo_lot(self, cr, uid, lot_list, qty, context):
        lot_obj = self.pool.get('stock.production.lot')
        fifo_lot = False
        for lot_id in lot_list.keys():
            cur_lot = lot_obj.browse(cr, uid, int(lot_id))
            if not fifo_lot and (lot_list[lot_id] > 0):
                fifo_lot = cur_lot
            elif fifo_lot and cur_lot.date < fifo_lot.date and (lot_list[lot_id] > 0):
                fifo_lot = cur_lot
        return str(fifo_lot.id)
    
    def lifo_lot(self, cr, uid, lot_list, qty, context):
        lot_obj = self.pool.get('stock.production.lot')
        lifo_lot = False
        for lot_id in lot_list.keys():
            cur_lot = lot_obj.browse(cr, uid, int(lot_id))
            if not lifo_lot and (lot_list[lot_id] > 0):
                lifo_lot = cur_lot
            elif lifo_lot and cur_lot.date > lifo_lot.date and (lot_list[lot_id] > 0):
                lifo_lot = cur_lot
        return str(lifo_lot.id)
    
    def action_confirm(self, cr, uid, ids):
        res = super(mrp_production, self).action_confirm(cr, uid, ids)
        return res
    
mrp_production()