# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import fields
from osv import osv
import netsvc
import time
from datetime import datetime
import decimal_precision as dp
from tools.translate import _

class mrp_production_operation_real(osv.osv):
    
    def _get_date_date(self, cr, uid, ids, field_name, arg, context=None):
        """ Finds starting date.
        @return: Dictionary of values.
        """
        res={}
        for op in self.browse(cr, uid, ids, context=context):
            if op.date_start:
                res[op.id] = op.date_start[:10]
            else:
                res[op.id]=False
        return res

    def _get_date_end(self, cr, uid, ids, field_name, arg, context=None):
        """ Finds ending date.
        @return: Dictionary of values.
        """
        ops = self.browse(cr, uid, ids, context=context)
        date_and_hours_by_cal = [(op.date_planned, op.hour, op.workcenter_id.calendar_id.id) for op in ops if op.date_planned]

        intervals = self.pool.get('resource.calendar').interval_get_multi(cr, uid, date_and_hours_by_cal)

        res = {}
        for op in ops:
            res[op.id] = False
            if op.date_planned:
                i = intervals.get((op.date_planned, op.hour, op.workcenter_id.calendar_id.id))
                if i:
                    res[op.id] = i[-1][1].strftime('%Y-%m-%d %H:%M:%S')
                else:
                    res[op.id] = op.date_planned
        return res
    
    def _operation_done_rate(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for real_operation in self.browse(cr, uid, ids, context=context):
            res[real_operation.id] = 0
            qty = 0
            prop = 1
            for finish_move in real_operation.move_created_ids2:
                prop = finish_move.bom_prop
                if finish_move.product_id and finish_move.operation_id:
                    if finish_move.operation_id.product_id:
                        if finish_move.product_id and real_operation.product:
                            if ((finish_move.product_id.id == finish_move.operation_id.product_id.id) or (finish_move.product_id.id == real_operation.product.id)):
                                qty += finish_move.product_qty
                        else:
                            if finish_move.product_id.id == finish_move.operation_id.product_id.id:
                                qty += finish_move.product_qty
                        
                    else:
                        if finish_move.product_id and real_operation.product:
                            if finish_move.product_id.id == real_operation.product.id:
                                qty += finish_move.product_qty
                else:
                    if finish_move.product_id and real_operation.product:
                        if finish_move.product_id.id == real_operation.product.id:
                            qty += finish_move.product_qty
            
            if prop > 0 and real_operation.qty>0:
                res[real_operation.id] = round((float(qty) * 100) / (float(real_operation.qty)/prop), 2)
            if real_operation.state == 'sent':
                res[real_operation.id] = 50
            elif real_operation.state == 'received':
                res[real_operation.id] = 100
        return res
    
    _name = "mrp.production.operation.real"
    _order = "sequence, date_planned"

    _columns = {
       'name': fields.char('Name', size = 64, required= True, readonly=True),
       'code':fields.char('Code',56, readonly=True),
       'sequence': fields.integer('Sequence', size = 64, required= True),
       'descript': fields.text('Description', size= 64),    
       'operation_id': fields.many2one('mrp.operation', 'Operation', required=True, readonly=True),
       'location_id': fields.many2one('stock.location', 'Operation Location', required= True, domain=[('usage','in',('internal','supplier'))], readonly=True),
       'workcenter_id': fields.many2one('mrp.workcenter', 'Work Center', required=True, readonly=True),
       'production_id': fields.many2one('mrp.production', 'Work Order', required=True, readonly=True),
       'capacity_per_cycle': fields.float('Capacity per Cycle'),
       'time_efficiency' : fields.float('Efficiency factor', required=True),
       'time_cycle': fields.float('Time for 1 cycle (hour)', help="Time in hours for doing one cycle of the material."),
       'time_start': fields.float('Time before prod.', help="Time in hours for the setup for the material."),
       'time_stop': fields.float('Time after prod.', help="Time in hours for the cleaning."),
       'costs_hour': fields.float('Cost per hour', help="Specify Cost of Workcenter per hour when using this material."),
       'costs_cycle': fields.float('Cost per cycle', help="Specify Cost of Workcenter per cycle when using this material."),
       'cycle': fields.float('Nº Cycles', readonly=True),
       'hour': fields.float('Nº Hours', readonly=True),
       'move_lines': fields.many2many('stock.move', 'operation_move_prod', 'real_operation_id', 'move_id', 'Products to Consume', domain=[('state','not in', ('done', 'cancel'))], readonly=True, states={'startworking':[('readonly',False)]}),
       'move_lines2': fields.many2many('stock.move', 'operation_move_prod', 'real_operation_id', 'move_id', 'Consumed Products', domain=[('state','in', ('done', 'cancel'))], readonly=True, states={'startworking':[('readonly',False)]}),
       'move_created_ids': fields.one2many('stock.move', 'real_operation_id', 'Moves Created', domain=[('state','not in', ('done', 'cancel')),('picking_id','=',False)], readonly=True, states={'startworking':[('readonly',False)]}),
       'move_created_ids2': fields.one2many('stock.move', 'real_operation_id', 'Moves Created', domain=[('state','in', ('done', 'cancel')),('picking_id','=',False)], readonly=True, states={'startworking':[('readonly',False)]}),
       'state': fields.selection([('draft','Waiting goods'),('ready','Ready'),('startworking', 'In Progress'),('pause','Pause'),('cancel','Cancelled'), ('aborted','Aborted'),('done','Finished'),('received','Received'),('deliverpick','Waiting to Deliver'),('sent','Picking sent')],'State', readonly=True,
                                 help="* When a work order is created it is set in 'Waiting goods' state.\n" \
                                       "* When all goods are available is set in 'Ready' state.\n" \
                                       "* When user sets work order in start mode that time it will be set in 'In Progress' state.\n" \
                                       "* When work order is in running mode, during that time if user wants to stop or to make changes in order then can set in 'Pause' state.\n" \
                                       "* When the user cancels the work order it will be set in 'Cancelled' state.\n" \
                                       "* When order is completely processed that time it is set in 'Finished' state."),
       'date_start_date': fields.function(_get_date_date, method=True, string='Start Date', type='date'),
       'date_planned': fields.datetime('Scheduled Date'),
       'date_planned_end': fields.function(_get_date_end, method=True, string='End Date', type='datetime'),
       'date_start': fields.datetime('Start Date'),
       'date_pause': fields.datetime('Pause Date'),
       'date_restart': fields.datetime('Restart Date'),
       'date_finished': fields.datetime('End Date'),
       'delay': fields.float('Working Hours',help="This is lead time between operation start and stop in this workcenter",readonly=True),
       'production_state':fields.related('production_id','state',
            type='selection',
            selection=[('draft','Draft'),('picking_except', 'Picking Exception'),('confirmed','Waiting Goods'),('ready','Ready to Produce'),('in_production','In Production'),('cancel','Cancelled'),('done','Done')],
            string='Production State', readonly=True),
       'product':fields.related('production_id','product_id',type='many2one',relation='product.product',string='Product',
            readonly=True),
       'qty': fields.float('Qty', digits=(16,3),readonly=True),
       'uom':fields.related('production_id','product_uom',type='many2one',relation='product.uom',string='UOM',readonly=True),
       'send_picking_ids':fields.many2many('stock.picking', 'mrp_real_op_pick_out_rel','real_out_op_id', 'pick_id_out', domain=[('type', '=', 'out')]),
       'receive_picking_ids':fields.many2many('stock.picking', 'mrp_real_op_pick_in_rel','real_in_op_id', 'pick_id_in', domain=[('type', '=', 'in')]),       
       'mrp_manu_op':fields.many2one('mrp.production.operation','Manufacturing operation'),
       'paused_time':fields.float('Paused time',help="This is paused time between operation pause and restart in this workcenter",readonly=True),
       'progress_rate': fields.function(_operation_done_rate, method=True, string='Progress (%)', type='float'),
    }
    
    _defaults = {
        'state': lambda *a: 'draft',
        'delay': lambda *a: 0.0,
        'time_efficiency': lambda *a: 0,
        'cycle': lambda *a: 1.0,
        'hour': lambda *a: 0.0,
        'paused_time':lambda *a: 0.0,
        'progress_rate': lambda *a: 0.0,
    }

    def modify_production_order_state(self, cr, uid, ids, action):
        """ Modifies production order state if work order state is changed.
        @param action: Action to perform.
        @return: Nothing
        """
        wf_service = netsvc.LocalService("workflow")
        oper_obj = self.browse(cr, uid, ids)[0]
        prod_obj = oper_obj.production_id
        if action == 'start': 
            if prod_obj.state =='confirmed':
                self.pool.get('mrp.production').force_production(cr, uid, [prod_obj.id])
                wf_service.trg_validate(uid, 'mrp.production', prod_obj.id, 'button_produce', cr)
            elif prod_obj.state =='ready':
                wf_service.trg_validate(uid, 'mrp.production', prod_obj.id, 'button_produce', cr)
            elif prod_obj.state =='in_production':
                return
            else:
                raise osv.except_osv(_('Error!'),_('Production Order Cannot start in [%s] state') % (prod_obj.state,))
        elif action == 'ready':
            wf_service.trg_validate(uid, 'mrp.production', prod_obj.id, 'is_ready', cr)
        else:
            oper_ids = self.search(cr,uid,[('production_id','=',prod_obj.id)])
            obj = self.browse(cr,uid,oper_ids)
            flag = True
            for line in obj:
                if line.state not in ('done', 'received'):
                     flag = False
            if flag:
                wf_service.trg_validate(uid, 'mrp.production', oper_obj.production_id.id, 'button_produce_done', cr)
        return

    def write(self, cr, uid, ids, vals, context=None, update=True):
        result = super(mrp_production_operation_real, self).write(cr, uid, ids, vals, context=context)
        prod_obj = self.pool.get('mrp.production')
        if vals.get('date_planned', False) and update:
            for prod in self.browse(cr, uid, ids, context=context):
                if prod.production_id.workcenter_lines:
                    dstart = min(vals['date_planned'], prod.production_id.workcenter_lines[0]['date_planned'])
                    prod_obj.write(cr, uid, [prod.production_id.id], {'date_start':dstart}, context=context, mini=False)
        return result

    def check_components(self, cr, uid, ids):
        op = self.browse(cr, uid, ids)[0]
        available = True
        if op.move_lines:
            for move in op.move_lines:
                if move.state != 'assigned':
                    available = False
                    for semi_move in op.move_lines:
                        if (semi_move.product_id == move.product_id) and (semi_move.state == 'assigned'):
                            available = True
        return available
    
    def action_check_goods(self, cr, uid, ids, context=None):
        wf_service = netsvc.LocalService("workflow")
        for op in self.browse(cr, uid, ids):
            if op.state not in ('deliverpick', 'sent'):
                wf_service.trg_validate(uid, 'mrp.production.operation.real', op.id, 'check_signal', cr)
        return True

    def action_draft(self, cr, uid, ids):
        """ Sets state to draft.
        @return: True
        """
        self.write(cr, uid, ids, {'state':'draft'})
        return True
    
    def action_ready(self, cr, uid, ids):
        """ Sets state to ready.
        @return: True
        """
        op = self.browse(cr, uid, ids)[0]
        if op.production_id.state != 'ready':
            self.modify_production_order_state(cr, uid, [op.id], 'ready')
        self.write(cr, uid, ids, {'state':'ready'})
        return True

    def action_start_working(self, cr, uid, ids, context={}):
        """ Sets state to start working and writes starting date.
        @return: True
        """
        mrp_production_operation_obj = self.pool.get('mrp.production.operation')
                                   
        start_ids = []
        for op in self.browse(cr, uid, ids):
            for consumed in op.move_lines:
                if consumed.state == 'assigned':
                    start_ids.append(op.id)
                    break
        if start_ids == [] and not op.move_lines:
            start_ids.append(op.id)
        if start_ids == []:
            raise osv.except_osv(_('Error!'),_('Product to be consumed with RESERVED state NOT FOUND'))
        else:
            self.modify_production_order_state(cr, uid, ids, 'start')
            self.write(cr, uid, start_ids, {'state':'startworking', 'date_start': time.strftime('%Y-%m-%d %H:%M:%S')})
        return True

    def action_done(self, cr, uid, ids):
        """ Sets state to done, writes finish date and calculates delay.
        @return: True
        """
        
        mrp_production_operation_obj = self.pool.get('mrp.production.operation')
        mrp_production_obj = self.pool.get('mrp.production')
        move_obj = self.pool.get('stock.move')
        
        obj_line = self.browse(cr, uid, ids[0])
        
        if not obj_line.move_lines2:   
            raise osv.except_osv(_('Error'), _('You can not finsh before produce'))
        
        #
        ### Creo el albaran de salida
        last_operation = False
        real_operation_ids = self.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                    ('sequence', '>',obj_line.sequence)])
        if not real_operation_ids:
            last_operation = True
            production = mrp_production_obj.browse(cr,uid,obj_line.production_id.id)
            
            production_operation_ids = mrp_production_operation_obj.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                                                     ('sequence', '=',obj_line.sequence)])
            operation = mrp_production_operation_obj.browse(cr,uid,production_operation_ids[0])
            picking_id = mrp_production_obj.create_picking(cr, uid, 'internal', 'direct', production, operation, 'output')
            self.pool.get('stock.picking').write(cr, uid, [picking_id], {'production_id': production.id,
                                                                         'auto_picking': True})
            
            move_new_id = move_obj.create(cr, uid, {
                    'name': 'PROD:' + production.name + '/' + operation.name,
                    'date': production.date_planned,
                    'picking_id': picking_id,
                    'product_id': production.product_id.id,
                    'product_qty': production.real_product_qty,
                    'product_uom': production.product_uom.id,
                    'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                    'product_uos': production.product_uos and production.product_uos.id or False,
                    'location_id': operation.location_id.id,
                    'location_dest_id': production.product_id.default_location.id,
                    'state': 'confirmed',
                    'company_id': production.company_id.id,
                    'operation_id': operation.id,
                    'real_operation_id': obj_line.id,
                    'production_id': production.id,

                })
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)            

        tot_time = 0.0
        date_now = time.strftime('%Y-%m-%d %H:%M:%S')
               
        date_start = datetime.strptime(obj_line.date_start,'%Y-%m-%d %H:%M:%S')
        
        date_finished = datetime.strptime(date_now,'%Y-%m-%d %H:%M:%S')
        tot_time += (date_finished-date_start).days * 24
        tot_time += (date_finished-date_start).seconds / float(60*60)
        
        tot_time -=     obj_line.paused_time               
        
        self.write(cr, uid, ids, {'state':'done', 'date_finished': date_now,'delay':tot_time})
        self.modify_production_order_state(cr,uid,ids,'done')
        
        if last_operation:
            real_operation = self.browse(cr, uid, ids[0])
            self._endoperation_generate_products_to_return(cr, uid, real_operation.production_id.id)
        
        return True
    
    def _endoperation_generate_products_to_return(self, cr, uid, production_id):
        move_obj = self.pool.get('stock.move')
        production_obj = self.pool.get('mrp.production')
        production = production_obj.browse(cr,uid,production_id)
        if production.products_to_return_ids:
            for move in production.products_to_return_ids:
                move_obj.write(cr, uid, [move.id], {'production_id2': False})  
                
        if production.real_operation_ids:
            for operation in production.real_operation_ids:
                if operation.move_lines:
                    for move in operation.move_lines:
                        if move.state == 'assigned':
                            if move.product_id.default_location:
                                move_obj.write(cr, uid, [move.id], {'production_id2': production.id,
                                                                    'location_dest_id': move.product_id.default_location.id})  
                            else:
                                raise osv.except_osv(_('Product ERROR'), _('You must define a default location for the product: %s') %(move.product_id.name))
        return True

    def action_cancel(self, cr, uid, ids):
        """ Sets state to cancel.
        @return: True
        """
        self.write(cr, uid, ids, {'state':'cancel'})
        return True

    def action_pause(self, cr, uid, ids):
        """ Sets state to pause.
        @return: True
        """

        date_now = time.strftime('%Y-%m-%d %H:%M:%S')
        self.write(cr, uid, ids, {'state':'pause', 'date_pause': date_now})
        
        return True

    def action_resume(self, cr, uid, ids):
        """ Sets state to startworking.
        @return: True
        """
        tot_time = 0.0
        date_now = time.strftime('%Y-%m-%d %H:%M:%S')
        obj_line = self.browse(cr, uid, ids[0])
        
        date_pause = datetime.strptime(obj_line.date_pause,'%Y-%m-%d %H:%M:%S')
        
        delay = 0.0       
               
        date_finished = datetime.strptime(date_now,'%Y-%m-%d %H:%M:%S')
        delay += (date_finished-date_pause).days * 24
        delay += (date_finished-date_pause).seconds / float(60*60)              
        
        pause_time = obj_line.paused_time +delay
        
        
        self.modify_production_order_state(cr,uid,ids,'done')
      
        self.write(cr, uid, ids, {'state':'startworking', 'date_restart':date_now, 'paused_time':pause_time})
        return True
    
    def action_deliver_pick(self, cr, uid, ids):
        """ Sets state to deliverpick.
        @return: True
        """
        self.write(cr, uid, ids, {'state':'deliverpick'})
        return True
    
    def action_sent(self, cr, uid, ids):
        """ Sets state to sent.
        @return: True
        """
        self.write(cr, uid, ids, {'state':'sent'})
        return True
    
    def action_aborted(self, cr, uid, ids):
        """ Sets state to aborted.
        @return: True
        """
        self.write(cr, uid, ids, {'state':'aborted'})
        return True
    
    def action_received(self, cr, uid, ids):
        """ Sets state to received.
        @return: True
        """
        self.write(cr, uid, ids, {'state':'received'})
        self.modify_production_order_state(cr,uid,ids,'done')
        return True

    def button_cancel(self, cr, uid, ids, *args):
        wf_service = netsvc.LocalService("workflow")
        for op in self.browse(cr, uid, ids):
            if op.state in ('deliverpick', 'sent'):
                wf_service.trg_validate(uid, 'mrp.production.operation.real', op.id, 'button_abort', cr)
            else:
                wf_service.trg_validate(uid, 'mrp.production.operation.real', op.id, 'button_cancel', cr)
        return True
    
    def check_sent_pick(self, cr, uid, ids):
        op = self.browse(cr, uid, ids)[0]
        for pick in op.send_picking_ids:
            if pick.state == 'done':
                return True
        return False
    
    def check_received_pick(self, cr, uid, ids):
        op = self.browse(cr, uid, ids)[0]
        for pick in op.receive_picking_ids:
            if pick.state == 'done':
                return True
        return False
    
mrp_production_operation_real()

class mrp_production(osv.osv):
    _inherit = 'mrp.production'
    
    def _production_rate(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for production in self.browse(cr, uid, ids, context=context):
            res[production.id] = 0
            progress = 0
            for operation in production.real_operation_ids:
                progress += operation.progress_rate
            if progress:
                res[production.id] = progress / float(len(production.real_operation_ids))
        return res
    
    _columns = {
        'real_operation_ids': fields.one2many('mrp.production.operation.real', 'production_id', 'Operation'),
        'progress_rate': fields.function(_production_rate, method=True, string='Progress (%)', type='float'),
        'allow_reorder': fields.boolean('Free Serialisation', help="Check this to be able to move independently all production orders, without moving dependent ones."),
    }
    
    def create(self, cr, uid, data, context=None):
        data.update({'with_advanced_operations':True})
        return super(mrp_production, self).create(cr,uid,data,context)
    
    def copy(self, cr, uid, id, default=None, context=None):
        default['real_operation_ids'] = []
        return super(mrp_production, self).copy(cr, uid, id, default, context=context)
    
    def action_check_goods(self, cr, uid, ids, context=None):
        wf_service = netsvc.LocalService("workflow")
        for order in self.browse(cr, uid, ids):
            for op in order.real_operation_ids:
                if op.state not in ('deliverpick', 'sent'):
                    wf_service.trg_validate(uid, 'mrp.production.operation.real', op.id, 'check_signal', cr)
                elif op.state == 'deliverpick':
                    wf_service.trg_validate(uid, 'mrp.production.operation.real', op.id, 'send_sig', cr)
                else:
                    wf_service.trg_validate(uid, 'mrp.production.operation.real', op.id, 'receive_sig', cr)
        return True

    def _production_date_end(self, cr, uid, ids, prop, unknow_none, context=None):
        """ Calculates planned end date of production order.
        @return: Dictionary of values
        """
        result = {}
        for prod in self.browse(cr, uid, ids, context=context):
            result[prod.id] = prod.date_planned
            for line in prod.real_operation_ids:
                result[prod.id] = max(line.date_planned_end, result[prod.id])
        return result

    def action_production_end(self, cr, uid, ids):
        """ Finishes work order if production order is done.
        @return: Super method
        """

        obj = self.browse(cr, uid, ids)[0]
        wf_service = netsvc.LocalService("workflow")
        for operation_line in obj.real_operation_ids:

            wf_service.trg_validate(uid, 'mrp.production.operation.real', operation_line.id, 'button_done', cr)

        return super(mrp_production,self).action_production_end(cr, uid, ids)
    
    def action_ready(self, cr, uid, ids):
        self.write(cr, uid, ids, {'state': 'ready'})
        return True
    
    def action_in_production(self, cr, uid, ids):
        """ Changes state to In Production and writes starting date.
        @return: True 
        """        
        obj = self.browse(cr, uid, ids)[0]
        wf_service = netsvc.LocalService("workflow")
        for operation_line in obj.real_operation_ids:
            wf_service.trg_validate(uid, 'mrp.production.operation.real', operation_line.id, 'button_start_working', cr)
        return super(mrp_production,self).action_in_production(cr, uid, ids)
    
    def action_cancel(self, cr, uid, ids):
        """ Cancels work order if production order is canceled.
        @return: Super method
        """
        obj = self.browse(cr, uid, ids)[0]
        wf_service = netsvc.LocalService("workflow")
        for operation_line in obj.real_operation_ids:
            if operation_line.state in ('deliverpick', 'sent'):
                wf_service.trg_validate(uid, 'mrp.production.operation.real', operation_line.id, 'button_abort', cr)
            else:
                wf_service.trg_validate(uid, 'mrp.production.operation.real', operation_line.id, 'button_cancel', cr)
        return super(mrp_production,self).action_cancel(cr,uid,ids)

    def _compute_planned_operations(self, cr, uid, ids, context=None, mini=False):
        """ Computes planned and finished dates for work order.
        @return: Calculated date
        """
        dt_end = datetime.now()
        if context is None:
            context = {}
        for po in self.browse(cr, uid, ids, context=context):
            dt_end = datetime.strptime(po.date_planned, '%Y-%m-%d %H:%M:%S')
            if not po.date_start:
                self.write(cr, uid, [po.id], {
                    'date_start': po.date_planned
                }, context=context, update=False)
            old = None
            for opi in range(len(po.real_operation_ids)):
                op  = po.real_operation_ids[opi]
                if (old is None) or (op.sequence>old):
                    dt = dt_end
                if context.get('__last_update'):
                    del context['__last_update']
                if (op.date_planned < dt.strftime('%Y-%m-%d %H:%M:%S')) or mini:
                    self.pool.get('mrp.production.operation.real').write(cr, uid, [op.id],  {
                        'date_planned': dt.strftime('%Y-%m-%d %H:%M:%S')
                    #}, context=context, update=False)
                    }, context=context)
                    i = self.pool.get('resource.calendar').interval_get(
                        cr,
                        uid,
                        op.workcenter_id.calendar_id and op.workcenter_id.calendar_id.id or False,
                        dt,
                        op.hour or 0.0
                    )
                    if i:
                        dt_end = max(dt_end, i[-1][1])
                else:
                    dt_end = datetime.strptime(op.date_planned_end, '%Y-%m-%d %H:%M:%S')

                old = op.sequence or 0
            super(mrp_production, self).write(cr, uid, [po.id], {
                'date_finished': dt_end
            })
        return dt_end

    def _move_pass(self, cr, uid, ids, context=None):
        """ Calculates start date for stock moves finding interval from resource calendar.
        @return: True 
        """
        for po in self.browse(cr, uid, ids, context=context):
            if po.allow_reorder:
                continue
            todo = po.move_lines
            dt = po.date_start
            while todo:
                l = todo.pop(0)
                if l.state in ('done','cancel','draft'):
                    continue
                todo += l.move_dest_id_lines
                if l.production_id and (l.production_id.date_finished > dt):
                    if l.production_id.state not in ('done','cancel'):
                        for wc in l.production_id.workcenter_lines:
                            i = self.pool.get('resource.calendar').interval_min_get(
                                cr,
                                uid,
                                wc.workcenter_id.calendar_id.id or False,
                                dt, wc.hour or 0.0
                            )
                            dt = i[0][0]
                        if l.production_id.date_start > dt:
                            self.write(cr, uid, [l.production_id.id], {'date_start':dt.strftime('%Y-%m-%d %H:%M:%S')}, mini=True)
        return True

    def _move_futur(self, cr, uid, ids, context=None):
        """ Calculates start date for stock moves.
        @return: True 
        """
        for po in self.browse(cr, uid, ids, context=context):
            if po.allow_reorder:
                continue
            for line in po.move_created_ids:
                l = line
                while l.move_dest_id:
                    l = l.move_dest_id
                    if l.state in ('done','cancel','draft'):
                        break
                    if l.production_id.state in ('done','cancel'):
                        break
                    if l.production_id and (l.production_id.date_start < po.date_finished):
                        self.write(cr, uid, [l.production_id.id], {'date_start': po.date_finished})
                        break

    def write(self, cr, uid, ids, vals, context=None, update=True, mini=True):
        stock_move_obj = self.pool.get('stock.move')
        stock_picking_obj = self.pool.get('stock.picking')
        direction = {}
        if vals.get('date_start', False):
            for po in self.browse(cr, uid, ids, context=context):
                direction[po.id] = cmp(po.date_start, vals.get('date_start', False))

        result = super(mrp_production, self).write(cr, uid, ids, vals, context=context)
        if (vals.get('real_operation_ids', False) or vals.get('date_start', False)) and update:
            self._compute_planned_operations(cr, uid, ids, context=context, mini=mini)
        for d in direction:
            if direction[d] == 1:
                # the production order has been moved to the passed
                self._move_pass(cr, uid, [d], context=context)
                pass
            elif direction[d] == -1:
                self._move_futur(cr, uid, [d], context=context)
                # the production order has been moved to the future
                pass
  
        return result

    def action_confirm(self, cr, uid, ids):
        components = []
        finals = []
        result = super(mrp_production, self).action_confirm(cr, uid, ids)
        production = self.pool.get('mrp.production').browse(cr, uid, ids)[0]
        workorder_objs = self.pool.get('mrp.production.operation.real')
        stock_move_objs = self.pool.get('stock.move')
        pick_objs = self.pool.get('stock.picking')
        if production.production_operation_ids:
            count = 0
            for operation in production.production_operation_ids:
                count += 1
                components = []
                finals = []
                code = production.name + '-' + str(count)
                if operation.location_id.usage == 'internal':
                    wo_id = workorder_objs.create(cr, uid, {
                        'sequence': operation.sequence,
                        'code': code,
                        'name': operation.name,
                        'operation_id': operation.operation_id.id,
                        'location_id': operation.location_id.id,
                        'workcenter_id': operation.workcenter_id.id,
                        'production_id': production.id,
                        'capacity_per_cycle': operation.capacity_per_cycle,
                        'time_efficiency': operation.time_efficiency,
                        'time_cycle': operation.time_cycle,
                        'time_start': operation.time_start,
                        'time_stop': operation.time_stop,
                        'costs_hour': operation.costs_hour,
                        'costs_cycle': operation.costs_cycle,
                        'cycle': operation.cycle,
                        'hour': operation.hour,
                        'mrp_manu_op': operation.id,
                    })
                    for component_move in production.move_lines:
                        if component_move.operation_id.id == operation.id:
                            components.append(component_move.id)                            
                    for final_move in production.move_created_ids:
                        if final_move.operation_id.id == operation.id:
                            finals.append(final_move.id)
                
                    workorder_objs.write(cr, uid, [wo_id], {'move_lines': [(6, 0, components)], 'move_created_ids': [(6, 0, finals)], 'state': 'draft'})
                else:
                    send = []
                    receive = []
                    wo_id = workorder_objs.create(cr, uid, {
                        'sequence': operation.sequence,
                        'code': code,
                        'name': operation.name,
                        'operation_id': operation.operation_id.id,
                        'location_id': operation.location_id.id,
                        'workcenter_id': operation.workcenter_id.id,
                        'production_id': production.id,
                        'mrp_manu_op': operation.id,
                    })
                    for picks in pick_objs.search(cr, uid, [('origin', 'like', production.name + '/' + operation.name), ('type', '=', 'out')]):
                        send.append(picks)
                    for picks in pick_objs.search(cr, uid, [('origin', 'like', production.name + '/' + operation.name), ('type', '=', 'in')]):
                        receive.append(picks)
                    workorder_objs.write(cr, uid, [wo_id], {'send_picking_ids': [(6, 0, send)], 'receive_picking_ids': [(6, 0, receive)], 'state': 'deliverpick'})
            self._compute_planned_operations(cr, uid, ids, context=None, mini=False)
        else:
            return False
        
        return result
mrp_production()

#class mrp_operations_operation_code(osv.osv):
#    _name="mrp_operations.operation.code"
#    _columns={
#        'name': fields.char('Operation Name',size=64, required=True),
#        'code': fields.char('Code', size=16, required=True),
#        'start_stop': fields.selection([('start','Start'),('pause','Pause'),('resume','Resume'),('cancel','Cancelled'),('done','Done')], 'Status', required=True),
#    }
#mrp_operations_operation_code()
#
#class mrp_operations_operation(osv.osv):
#    _name="mrp_operations.operation"
#
#    def _order_date_search_production(self, cr, uid, ids, context=None):
#        """ Finds operations for a production order.
#        @return: List of ids
#        """
#        operation_ids = self.pool.get('mrp_operations.operation').search(cr, uid, [('production_id','=',ids[0])], context=context)
#        return operation_ids
#
#    def _get_order_date(self, cr, uid, ids, field_name, arg, context=None):
#        """ Calculates planned date for an operation.
#        @return: Dictionary of values
#        """
#        res={}
#        operation_obj = self.browse(cr, uid, ids, context=context)
#        for operation in operation_obj:
#                res[operation.id] = operation.production_id.date_planned
#        return res
#
#    def calc_delay(self, cr, uid, vals):
#        """ Calculates delay of work order.
#        @return: Delay
#        """
#        code_lst = []
#        time_lst = []
#
#        code_ids = self.pool.get('mrp_operations.operation.code').search(cr, uid, [('id','=',vals['code_id'])])
#        code = self.pool.get('mrp_operations.operation.code').browse(cr, uid, code_ids)[0]
#
#        oper_ids = self.search(cr,uid,[('production_id','=',vals['production_id']),('workcenter_id','=',vals['workcenter_id'])])
#        oper_objs = self.browse(cr,uid,oper_ids)
#
#        for oper in oper_objs:
#            code_lst.append(oper.code_id.start_stop)
#            time_lst.append(oper.date_start)
#
#        code_lst.append(code.start_stop)
#        time_lst.append(vals['date_start'])
#        diff = 0
#        for i in range(0,len(code_lst)):
#            if code_lst[i] == 'pause' or code_lst[i] == 'done' or code_lst[i] == 'cancel':
#                if not i: continue
#                if code_lst[i-1] not in ('resume','start'):
#                   continue
#                a = datetime.strptime(time_lst[i-1],'%Y-%m-%d %H:%M:%S')
#                b = datetime.strptime(time_lst[i],'%Y-%m-%d %H:%M:%S')
#                diff += (b-a).days * 24
#                diff += (b-a).seconds / float(60*60)
#        return diff
#
#    def check_operation(self, cr, uid, vals):
#        """ Finds which operation is called ie. start, pause, done, cancel.
#        @param vals: Dictionary of values.
#        @return: True or False
#        """
#        code_ids=self.pool.get('mrp_operations.operation.code').search(cr,uid,[('id','=',vals['code_id'])])
#        code=self.pool.get('mrp_operations.operation.code').browse(cr,uid,code_ids)[0]
#        code_lst = []
#        oper_ids=self.search(cr,uid,[('production_id','=',vals['production_id']),('workcenter_id','=',vals['workcenter_id'])])
#        oper_objs=self.browse(cr,uid,oper_ids)
#
#        if not oper_objs:
#            if code.start_stop!='start':
#                raise osv.except_osv(_('Sorry!'),_('Operation is not started yet !'))
#                return False
#        else:
#            for oper in oper_objs:
#                 code_lst.append(oper.code_id.start_stop)
#            if code.start_stop=='start':
#                    if 'start' in code_lst:
#                        raise osv.except_osv(_('Sorry!'),_('Operation has already started !' 'You  can either Pause /Finish/Cancel the operation'))
#                        return False
#            if code.start_stop=='pause':
#                    if  code_lst[len(code_lst)-1]!='resume' and code_lst[len(code_lst)-1]!='start':
#                        raise osv.except_osv(_('Error!'),_('You cannot Pause the Operation other then Start/Resume state !'))
#                        return False
#            if code.start_stop=='resume':
#                if code_lst[len(code_lst)-1]!='pause':
#                   raise osv.except_osv(_('Error!'),_(' You cannot Resume the operation other then Pause state !'))
#                   return False
#
#            if code.start_stop=='done':
#               if code_lst[len(code_lst)-1]!='start' and code_lst[len(code_lst)-1]!='resume':
#                  raise osv.except_osv(_('Sorry!'),_('You cannot finish the operation without Starting/Resuming it !'))
#                  return False
#               if 'cancel' in code_lst:
#                  raise osv.except_osv(_('Sorry!'),_('Operation is Already Cancelled  !'))
#                  return False
#            if code.start_stop=='cancel':
#               if  not 'start' in code_lst :
#                   raise osv.except_osv(_('Error!'),_('There is no Operation to be cancelled !'))
#                   return False
#               if 'done' in code_lst:
#                  raise osv.except_osv(_('Error!'),_('Operation is already finished !'))
#                  return False
#        return True
#
#    def write(self, cr, uid, ids, vals, context=None):
#        oper_objs = self.browse(cr, uid, ids, context=context)[0]
#        vals['production_id']=oper_objs.production_id.id
#        vals['workcenter_id']=oper_objs.workcenter_id.id
#
#        if 'code_id' in vals:
#            self.check_operation(cr, uid, vals)
#
#        if 'date_start' in vals:
#            vals['date_start']=vals['date_start']
#            vals['code_id']=oper_objs.code_id.id
#            delay=self.calc_delay(cr, uid, vals)
#            wc_op_id=self.pool.get('mrp.production.operation.real').search(cr,uid,[('workcenter_id','=',vals['workcenter_id']),('production_id','=',vals['production_id'])])
#            self.pool.get('mrp.production.operation.real').write(cr,uid,wc_op_id,{'delay':delay})
#
#        return super(mrp_operations_operation, self).write(cr, uid, ids, vals, context=context)
#
#    def create(self, cr, uid, vals, context=None):
#        wf_service = netsvc.LocalService('workflow')
#        code_ids=self.pool.get('mrp_operations.operation.code').search(cr,uid,[('id','=',vals['code_id'])])
#        code=self.pool.get('mrp_operations.operation.code').browse(cr, uid, code_ids, context=context)[0]
#        wc_op_id=self.pool.get('mrp.production.operation.real').search(cr,uid,[('workcenter_id','=',vals['workcenter_id']),('production_id','=',vals['production_id'])])
#        if code.start_stop in ('start','done','pause','cancel','resume'):
#            if not wc_op_id:
#                production_obj=self.pool.get('mrp.production').browse(cr, uid, vals['production_id'], context=context)
#                wc_op_id.append(self.pool.get('mrp.production.operation.real').create(cr,uid,{'production_id':vals['production_id'],'name':production_obj.product_id.name,'workcenter_id':vals['workcenter_id']}))
#            if code.start_stop=='start':
#                self.pool.get('mrp.production.operation.real').action_start_working(cr,uid,wc_op_id)
#                wf_service.trg_validate(uid, 'mrp.production.operation.real', wc_op_id[0], 'button_start_working', cr)
#                
#
#            if code.start_stop=='done':
#                self.pool.get('mrp.production.operation.real').action_done(cr,uid,wc_op_id)
#                wf_service.trg_validate(uid, 'mrp.production.operation.real', wc_op_id[0], 'button_done', cr)
#                self.pool.get('mrp.production').write(cr,uid,vals['production_id'],{'date_finished':datetime.now().strftime('%Y-%m-%d %H:%M:%S')})
#
#            if code.start_stop=='pause':
#                self.pool.get('mrp.production.operation.real').action_pause(cr,uid,wc_op_id)
#                wf_service.trg_validate(uid, 'mrp.production.operation.real', wc_op_id[0], 'button_pause', cr)
#
#            if code.start_stop=='resume':
#                self.pool.get('mrp.production.operation.real').action_resume(cr,uid,wc_op_id)
#                wf_service.trg_validate(uid, 'mrp.production.operation.real', wc_op_id[0], 'button_resume', cr)
#
#            if code.start_stop=='cancel':
#                self.pool.get('mrp.production.operation.real').action_cancel(cr,uid,wc_op_id)
#                wf_service.trg_validate(uid, 'mrp.production.operation.real', wc_op_id[0], 'button_cancel', cr)
#
#        if not self.check_operation(cr, uid, vals):
#            return
#        delay=self.calc_delay(cr, uid, vals)
#        line_vals = {}
#        line_vals['delay'] = delay
#        if vals.get('date_start',False):
#            if code.start_stop == 'done':
#                line_vals['date_finished'] = vals['date_start']
#            elif code.start_stop == 'start':    
#                line_vals['date_start'] = vals['date_start']
#
#        self.pool.get('mrp.production.operation.real').write(cr, uid, wc_op_id, line_vals, context=context)
#
#        return super(mrp_operations_operation, self).create(cr, uid, vals, context=context)
#
#    _columns={
#        'production_id':fields.many2one('mrp.production','Production',required=True),
#        'workcenter_id':fields.many2one('mrp.workcenter','Work Center',required=True),
#        'code_id':fields.many2one('mrp_operations.operation.code','Code',required=True),
#        'date_start': fields.datetime('Start Date'),
#        'date_finished': fields.datetime('End Date'),
#        'order_date': fields.function(_get_order_date,method=True,string='Order Date',type='date',store={'mrp.production':(_order_date_search_production,['date_planned'], 10)}),
#        }
#    _defaults={
#        'date_start': lambda *a:datetime.now().strftime('%Y-%m-%d %H:%M:%S')
#    }
#
#mrp_operations_operation()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

