# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
import time

class partial_produce(osv.osv_memory):
    _name = "partial.produce"
    _description = "Partial Production"
    
    _columns = {
        'partial_real_operation_id': fields.many2one('mrp.production.operation.real', 'Real Operation'),
        'partial_product_id': fields.many2one('product.product', 'Consumed Product', readonly=True),
        'partial_product_qty': fields.float('Qty'),
        'partial_semi_prod_move': fields.many2one('stock.move', 'Stock move'),
        'partial_semi_prod_loc': fields.many2one('stock.location', 'Location'),
        'partial_consumed_ids': fields.one2many('parcial.consumed.aux', 'wizard_id', 'Consumed Moves'),
        'partial_finished_ids': fields.one2many('parcial.finished.aux', 'wizard_id', 'Finished Moves'),
    }  
    
    def default_get(self, cr, uid, fields, context=None):
        if context is None:
            context = {}
        objs = self.pool.get(context['active_model']).browse(cr , uid, context['active_ids'])
        real_operation_obj = self.pool.get('mrp.production.operation.real')
        result = []
        result2 = []
        res = {}
        if 'value' not in context.keys():
            for obj in objs:
                for consumed in obj.move_lines:
                    if consumed.state == 'assigned':
                        values = {
                             'stock_move_id': consumed.id,
                             'product_id':consumed.product_id.id,
                             'product_qty': consumed.product_qty,
                             'bom_prop': consumed.bom_prop,
                             'product_uom': consumed.product_id.uom_id.id,
                             'location_id': consumed.location_id.id,
                        }
                        result.append(values)
                for finished in obj.move_created_ids:
                    if (finished.product_id.default_location.usage == 'production') or (finished.product_id.id == obj.product.id):
                        res ={
                            'partial_real_operation_id': context.get('real_operation_id'),
                            'partial_product_id': finished.product_id.id,
                            'partial_product_qty': finished.product_qty,
                            'partial_semi_prod_move': finished.id,
                            'partial_semi_prod_loc': finished.location_id.id,
                        }
                    else:
                        res ={
                            'partial_real_operation_id': context.get('real_operation_id'),
                            'partial_product_id': finished.product_id.id,
                            'partial_product_qty': finished.product_qty,
                        }
                        values2 = {
                            'stock_move_id': finished.id,
                            'product_id':finished.product_id.id,
                            'product_qty': finished.product_qty,
                            'bom_prop': finished.bom_prop,
                            'product_uom': finished.product_id.uom_id.id,
                            'location_id': finished.location_id.id,
                        }
                        result2.append(values2)
            res.update({
                        'partial_consumed_ids': result,
                        'partial_finished_ids': result2,
            })
        else:
            res = context['value']
        return res
    
    def partial_production(self, cr, uid, ids, context = None):        
        move_obj = self.pool.get('stock.move')
        operation_real_obj = self.pool.get('mrp.production.operation.real')
        mrp_production_obj = self.pool.get('mrp.production')
        if context is None:
            context = {}

        for obj in self.browse(cr, uid, ids):
            for consumed in obj.partial_consumed_ids:
                move_obj.action_consume(cr, uid, [consumed.stock_move_id.id] ,consumed.product_qty,consumed.location_id.id,context=context)
            for finished in obj.partial_finished_ids:
                if finished.product_qty <= finished.stock_move_id.product_qty:
                    move_obj.action_consume(cr, uid, [finished.stock_move_id.id] ,finished.product_qty,finished.location_id.id,context=context)
                    if finished.stock_move_id.move_dest_id:
                        move_obj.write(cr, uid, finished.stock_move_id.move_dest_id.id, {'state': 'waiting'})
                else:
                    raise osv.except_osv(_('Error!'),_('Cannot produce more than scheduled in this operation'))
		  
            if obj.partial_semi_prod_move:
                if obj.partial_product_qty <= obj.partial_semi_prod_move.product_qty:
                    if obj.partial_product_qty <= obj.partial_semi_prod_move.move_dest_id.product_qty or obj.partial_product_id.id == obj.partial_semi_prod_move.real_operation_id.product.id:
                        move_obj.action_consume(cr, uid, [obj.partial_semi_prod_move.id] , obj.partial_product_qty, obj.partial_semi_prod_loc.id,context=context)
                        if not obj.partial_product_id.id == obj.partial_semi_prod_move.real_operation_id.product.id:
                            move_obj.write(cr, uid, obj.partial_semi_prod_move.move_dest_id.id, {'state': 'waiting'})
                            self.mrp_split(cr, uid, ids, obj, obj.partial_semi_prod_move, context)     
                    else:
                        raise osv.except_osv(_('Error!'),_('Cannot produce more than scheduled in the next operation'))
		      
                else:
                    raise osv.except_osv(_('Error!'),_('Cannot produce more than scheduled in this operation'))        
		  
            real_operation = operation_real_obj.browse(cr,uid,obj.partial_real_operation_id.id)
            real_operation_ids = operation_real_obj.search(cr, uid,[('production_id', '=', real_operation.production_id.id),
                                                                    ('sequence','>', real_operation.sequence)])  
            if not real_operation_ids:
                production = mrp_production_obj.browse(cr,uid,real_operation.production_id.id)
                qty = 0
                if not production.real_product_qty:
                    qty = obj.partial_product_qty
                else:
                    qty = obj.partial_product_qty + production.real_product_qty
                mrp_production_obj.write(cr, uid, [production.id], {'real_product_qty':qty})                
 
        return {'type': 'ir.actions.act_window_close'}
    
    def mrp_split(self, cr, uid, ids, obj, part_move, context=None):
        move_obj = self.pool.get('stock.move')
        picking_obj = self.pool.get('stock.picking')
        part_dest = None
        assigned_found = False
        qty_runout = False
        qty = 0
        if part_move.move_dest_id.real_operation_ids:
            part_dest = part_move.move_dest_id.real_operation_ids[0]
        else:
            part_dest = part_move.move_dest_id.picking_id
        if not part_dest:
            return False
        
        
        for move in part_dest.move_lines:
            if (move.product_id.id == part_move.product_id.id) and (move.state == 'assigned'):
                assigned_found = True
        if not assigned_found:
            move = move_obj.action_split(cr, uid, [part_move.move_dest_id.id], obj.partial_product_qty, obj.partial_product_qty, False, False, context=context)
            move_obj.write(cr, uid, move, {'state': 'assigned'})
        for move in part_dest.move_lines:
            if (move.product_id.id == part_move.product_id.id) and (move.state == 'assigned'):
                qty = move.product_qty + obj.partial_product_qty
                move_obj.write(cr, uid, move.id, {'product_qty': qty})
            elif (move.product_id.id == part_move.product_id.id) and (move.state in ('waiting')):
                qty = move.product_qty - obj.partial_product_qty
                if not qty:
                    qty_runout = True
                else:
                    move_obj.write(cr, uid, move.id, {'product_qty': qty})
        if qty_runout and assigned_found:
            move_obj.write(cr, uid, [part_move.move_dest_id.id], {'state': 'draft'})
            move_obj.unlink(cr, uid, [part_move.move_dest_id.id])
        return True
    
    def button_calculate (self, cr, uid, ids, context= None):
        res = {}
        result = []
        result2 = []
        for obj in self.browse(cr, uid, ids):
            for consumed in obj.partial_consumed_ids:
                values = {
                    'product_qty': obj.partial_product_qty * consumed.bom_prop,
                    'bom_prop': consumed.bom_prop,
                    'stock_move_id': consumed.stock_move_id.id,
                    'product_id':consumed.product_id.id,
                    'product_uom': consumed.product_id.uom_id.id,
                    'location_id': consumed.location_id.id,
                }
                result.append(values)
            for finished in obj.partial_finished_ids:
                values2 = {
                    'product_qty': obj.partial_product_qty * finished.bom_prop,
                    'bom_prop': finished.bom_prop,
                    'stock_move_id': finished.stock_move_id.id,
                    'product_id':finished.product_id.id,
                    'product_uom': finished.product_id.uom_id.id,
                    'location_id': finished.location_id.id,
                }
                result2.append(values2)
            res = {
                'partial_real_operation_id': obj.partial_real_operation_id.id,
                'partial_product_qty': obj.partial_product_qty,
                'partial_product_id': obj.partial_product_id.id,
                'partial_semi_prod_move': obj.partial_semi_prod_move.id,
                'partial_semi_prod_loc': obj.partial_semi_prod_loc.id,
                'partial_consumed_ids': result,
                'partial_finished_ids': result2,
            }
        context.update({'value': res})
        return {'type': 'ir.actions.act_window', 'res_model': 'partial.produce', 'view_type': 'form', 'view_mode': 'form', 'target': 'new', 'context':context}
    
partial_produce()

class parcial_finished_aux(osv.osv_memory):
    _name = "parcial.finished.aux"
    
    _columns = {
            'stock_move_id':fields.many2one('stock.move', 'Stock move', required=True, select=True),
            'product_id':fields.many2one('product.product', 'Product', required=True, select=True),
            'product_qty': fields.float('Qty', required=True),
            'bom_prop': fields.float('BOM Proportion'),
            'product_uom': fields.many2one('product.uom', 'UOM', required=True),
            'location_id': fields.many2one('stock.location', 'Location', required=True),
            'wizard_id': fields.many2one('partial.produce', 'wizard'),
            }
    
parcial_finished_aux()

class parcial_consumed_aux(osv.osv_memory):
    _name = "parcial.consumed.aux"
    
    _columns = {
            'stock_move_id':fields.many2one('stock.move', 'Stock move', required=True, select=True),
            'product_id':fields.many2one('product.product', 'Product', required=True, select=True),
            'product_qty': fields.float('Qty', required=True),
            'bom_prop': fields.float('BOM Proportion'),
            'product_uom': fields.many2one('product.uom', 'UOM', required=True),
            'location_id': fields.many2one('stock.location', 'Location', required=True),
            'wizard_id': fields.many2one('partial.produce', 'wizard'),
            }
    
parcial_consumed_aux()


