# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import osv, fields
import tools
import math
from tools.translate import _
import time
from datetime import datetime
from dateutil.relativedelta import relativedelta
import netsvc
import decimal_precision as dp

class account_analytic_account(osv.osv):
    _name = "account.analytic.account"
    _inherit = "account.analytic.account"
    
#    def _acc_result(cr,uid,ids):
#        dif_acc = self.browse(cr,uid,ids)
#        dif = dif_acc.debit - dif_acc.total_estimated
#        return dif
    
    _columns = {
                'total_estimated':fields.float('Estimated'),
                'total_estimated_puc':fields.float('Estimated PLP'),
                'total_estimated_standard':fields.float('Estimated Standard'),
                'acc_type':fields.char('Account type',size = 2),
                'acc_result':fields.float('Account result')
                }
    
    def recalculate_result(self, cr, uid, acc_id):
        if acc_id:
            acc = self.browse(cr, uid, acc_id)
            
            resu = float(acc.balance) + float(acc.total_estimated)
            
            self.write(cr, uid, acc_id, {'acc_result': resu})
            
            if not acc.acc_type == 'MF':            
                self.recalculate_result(cr, uid, acc.parent_id.id)
        return True
    
    def calculate_estimated(self, cr, uid, parnt_id, amount, amount_puc, amount_standard):
        
        if parnt_id:
            acc = self.browse(cr, uid, parnt_id)
            
            total = acc.total_estimated
            total2 = acc.total_estimated_puc
            total3 = acc.total_estimated_standard
            if total is None:
                total = 0.0
            if total2 is None:
                total2 = 0.0
            if total3 is None:
                total3 = 0.0
            tot = total + amount
            tot2 = total2 + amount_puc
            tot3 = total3 + amount_standard
            resu = acc.debit - tot
            self.write(cr, uid, parnt_id, {
                                           'total_estimated':tot,
                                           'total_estimated_puc':tot2,
                                           'total_estimated_standard': tot3,
                                           'acc_result':resu
                                           })
            if not acc.acc_type == 'MF':            
                self.calculate_estimated(cr, uid, acc.parent_id.id, amount, amount_puc, amount_standard)
        
        return True
    
    _defaults = {
                 'total_estimated': 0.00,    
                 'total_estimated_puc': 0.00,
                 'total_estimated_standard': 0.00,
                 }
account_analytic_account()


class mrp_production_operation_real(osv.osv):

    _inherit = "mrp.production.operation.real"
    _name = "mrp.production.operation.real"
    
    def action_done(self, cr, uid, ids):
        
        """ Sets state to done, writes finish date and calculates delay.
        @return: True
        """
        mrp_production_operation_obj = self.pool.get('mrp.production.operation')
        mrp_production_obj = self.pool.get('mrp.production')
        move_obj = self.pool.get('stock.move')
        
        tot_time = 0.0
        date_now = time.strftime('%Y-%m-%d %H:%M:%S')
        obj_line = self.browse(cr, uid, ids[0])
        
        if not obj_line.move_lines2:   
            raise osv.except_osv(_('Error'), _('You can not finish before produce'))
        
        #
        ### Creo el albaran de salida
        last_operation = False
        real_operation_ids = self.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                    ('sequence', '>',obj_line.sequence)])
        if not real_operation_ids:
            last_operation = True
            production = mrp_production_obj.browse(cr,uid,obj_line.production_id.id)
            
            production_operation_ids = mrp_production_operation_obj.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                                                     ('sequence', '=',obj_line.sequence)])
            operation = mrp_production_operation_obj.browse(cr,uid,production_operation_ids[0])
            picking_id = mrp_production_obj.create_picking(cr, uid, 'internal', 'direct', production, operation, 'output')
            self.pool.get('stock.picking').write(cr, uid, [picking_id], {'production_id': production.id,
                                                                         'auto_picking': True})
            
            move_new_id = move_obj.create(cr, uid, {
                    'name': 'PROD:' + production.name + '/' + operation.name,
                    'date': production.date_planned,
                    'picking_id': picking_id,
                    'product_id': production.product_id.id,
                    'product_qty': production.real_product_qty,
                    'product_uom': production.product_uom.id,
                    'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                    'product_uos': production.product_uos and production.product_uos.id or False,
                    'location_id': operation.location_id.id,
                    'location_dest_id': production.product_id.default_location.id,
                    'state': 'confirmed',
                    'company_id': production.company_id.id,
                    'operation_id': operation.id,
                    'real_operation_id': obj_line.id,
                    'production_id': production.id
                })
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr) 
        
        date_start = datetime.strptime(obj_line.date_start, '%Y-%m-%d %H:%M:%S')
        
        date_finished = datetime.strptime(date_now, '%Y-%m-%d %H:%M:%S')
        tot_time += (date_finished-date_start).days * 24
        tot_time += (date_finished-date_start).seconds / float(60*60)
        
        tot_time -= obj_line.paused_time       
        
        if obj_line.date_restart:
                date_start = datetime.strptime(obj_line.date_restart,'%Y-%m-%d %H:%M:%S')
        
        delay = 0.0
        
        date_finished = datetime.strptime(date_now,'%Y-%m-%d %H:%M:%S')
        delay += (date_finished-date_start).days * 24
        delay += (date_finished-date_start).seconds / float(60*60)              
            
            
        time_obj = self.pool.get('new.input.time')           
        
        time_obj.input_real_worktime(cr, uid, ids, uid, delay, obj_line.mrp_manu_op.id, date_now, 'Work in '+str(obj_line.workcenter_id.name))
        
        self.write(cr, uid, ids, {
                                  'state': 'done',
                                  'date_finished': date_now,
                                  'delay': tot_time
                                  })
        self.modify_production_order_state(cr, uid, ids, 'done')
        
                #Añadido link a contabilidad analitica para costes de materiales
        
        if obj_line.production_id:
            if not obj_line.production_id.company_id:
                raise osv.except_osv(_('Error'), _('You have to assign a company to this work order!'))
            else:
                self.pool.get('mrp.analytic.wizard').mats_final_cost(cr, uid, ids[0], obj_line.production_id.company_id.id)
        
        if last_operation:
            real_operation = self.browse(cr, uid, ids[0])
            self._endoperation_generate_products_to_return(cr, uid, real_operation.production_id.id)
        
        return True
    
mrp_production_operation_real()
#class mrp_production_operation_real(osv.osv):
#    _name = 'mrp.production.operation.real'
#    _inherit = 'mrp.production.operation.real'
#    
#    def action_done(self, cr, uid, ids):
#        
#    
#mrp_production_real()