# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import osv, fields
from datetime import datetime
from dateutil.relativedelta import relativedelta
from tools.translate import _
import netsvc

class mrp_wc_cost(osv.osv):
    _name = "mrp.wc.cost"
    _inherit = "mrp.wc.cost"
    
    def create(self, cr, uid, data, context=None):
        if context is None:
            context = {}  
        us_obj = self.pool.get('production.opert.users')
        account_analytic_obj = self.pool.get('account.analytic.account')
        input_time_wc_obj = self.pool.get('input.time.wc')
        aux_prod = self.pool.get('mrp.production')
        aux_prod_op = self.pool.get('mrp.production.operation')
  
        new_id = super(mrp_wc_cost, self).create(cr, uid, data, context)     
        wc_cost = self.browse(cr,uid,new_id)
        if wc_cost.create_structure == True:     
            # Imputación en Contabilidad Analítica
            ope_acc_code = 'OF'+ str(wc_cost.production.name.lstrip('OF/'))+'-OP'+str(wc_cost.operation.sequence)
            account_search = account_analytic_obj.search(cr, uid, [('code','=',ope_acc_code)])
            if not account_search:
                raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(ope_acc_code))
            
            ope_account_id=False
            
            for aux_var in account_search:
                ope_account_id = aux_var
                
            if ope_account_id:
                resu = input_time_wc_obj.analityc_inpute(cr, uid ,wc_cost.date,wc_cost.descript,wc_cost.wc.id,wc_cost.time, wc_cost.operation.id, wc_cost.production.id)
        
            #Times
            op = aux_prod_op.browse(cr , uid, wc_cost.operation.id)
            op_time_add = op.tot_ops + wc_cost.time
            tot_time_add = op.tot + wc_cost.time
            aux_prod_op.write(cr, uid, [op.id], {'tot_mac':op_time_add , 'tot':tot_time_add  })  

        return new_id
    
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}  
            
        production_obj = self.pool.get('mrp.production')
        operation_obj = self.pool.get('mrp.production.operation')
        analytic_account_obj = self.pool.get('account.analytic.account')
        analytic_account_line_obj = self.pool.get('account.analytic.line')
        wc_obj = self.pool.get('mrp.workcenter')
        product_obj = self.pool.get('product.product')
        res_company_obj = self.pool.get('res.company')
        account_analytic_jour_obj = self.pool.get('account.analytic.journal')
        wc_obj = self.pool.get('mrp.workcenter')
            
        if ids:
            for wc_cost in self.browse(cr,uid,ids):
                # Descuento Tiempos de producción
                if wc_cost.production:
                    production = production_obj.browse(cr, uid, wc_cost.production.id)
                    values = {'tot': production.tot - wc_cost.time,
                              'tot_mac': production.tot_mac - wc_cost.time
                              }
                    production_obj.write(cr, uid, [production.id], values)  
                # Descuento Tiempos de la operación
                if wc_cost.operation:
                    operation = operation_obj.browse(cr , uid, wc_cost.operation.id)
                    op_time_add = operation.tot_ops - wc_cost.time
                    tot_time_add = operation.tot - wc_cost.time
                    operation_obj.write(cr, uid, [operation.id], {'tot_mac':op_time_add , 'tot':tot_time_add  })  
                # Borro la cuenta analítica
                parent_code = 'OF'+ str(wc_cost.production.name.lstrip('OF/')) + '-OP' + str(wc_cost.operation.sequence) + '-MA'
                parent_account = analytic_account_obj.search(cr, uid, [('code','=',parent_code)])[0]
                if not parent_account:
                    raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(parent_code))
   
                usr_search = wc_obj.search(cr, uid,[('id', '=', wc_cost.wc.id)])
                wc = wc_obj.browse(cr , uid, usr_search[0])   
                if not wc.product_id:
                    raise osv.except_osv(_('You must assign a product to all machines'), _(wc.name))    
                account_ids = analytic_account_obj.search(cr, uid, [('name','=',parent_code + " " + wc.name),
                                                                    ('code','=',parent_code),
                                                                    ('currency_id','=',1),
                                                                    ('company_id','=',1),
                                                                    ('state','=','open'),
                                                                    ('parent_id','=',parent_account),
                                                                    ('acc_type','=','MA')])
                analytic_account_obj.unlink(cr,uid,account_ids)
                
                
                # Borro el apunte analítico              
                aux_product = product_obj.browse(cr , uid, wc.product_id.id) 
                op_cost_amount = aux_product.standard_price * wc_cost.time
                
                ope_account_id = False
                ope_acc_code = 'OF'+ str(wc_cost.production.name.lstrip('OF/'))  + '-OP' + str(wc_cost.operation.sequence) + '-MA'
                account_search = analytic_account_obj.search(cr, uid, [('code','=',ope_acc_code)])
                if not account_search:
                    raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(ope_acc_code))
                    
                for aux_var in account_search:
                    ope_account_id = aux_var
                man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])        
                man_journal = account_analytic_jour_obj.browse(cr, uid, man_journal_search[0])

 
                analytic_line_ids = analytic_account_line_obj.search(cr, uid, [('amount','=',op_cost_amount*(-1)),
                                                                               ('name','=',wc_cost.descript),
                                                                               ('date','=',wc_cost.date),
                                                                               ('company_id','=',wc.factory_id.company_id.id),
                                                                               ('general_account_id','=',wc.factory_id.company_id.workcenter_account.id),
                                                                               ('account_id','=',ope_account_id),
                                                                               ('journal_id','=',man_journal.id)])
                analytic_account_line_obj.unlink(cr,uid,analytic_line_ids)

        super(mrp_wc_cost,self).unlink(cr, uid, ids, context=context)

        return True
    
    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
            
        production_obj = self.pool.get('mrp.production')
        operation_obj = self.pool.get('mrp.production.operation')
        analytic_account_obj = self.pool.get('account.analytic.account')
        analytic_account_line_obj = self.pool.get('account.analytic.line')
        product_obj = self.pool.get('product.product')
        res_company_obj = self.pool.get('res.company')
        account_analytic_jour_obj = self.pool.get('account.analytic.journal')
            
        if ids:
            for wc_cost in self.browse(cr,uid,ids):
                if vals.has_key('time'):
                    new_time = vals.get('time')
                    if new_time == wc_cost.time:
                        super(mrp_wc_cost, self).write(cr, uid, [wc_cost.id], vals, context=context)
                    else:
                        # Descuento Tiempos de producción
                        if wc_cost.production:
                            production = production_obj.browse(cr, uid, wc_cost.production.id)
                            values = {'tot': production.tot - wc_cost.time,
                                      'tot_mac': production.tot_mac - wc_cost.time
                                      }
                            production_obj.write(cr, uid, [production.id], values)  
                        # Descuento Tiempos de la operación
                        if wc_cost.operation:
                            operation = operation_obj.browse(cr , uid, wc_cost.operation.id)
                            op_time_add = operation.tot_ops - wc_cost.time
                            tot_time_add = operation.tot - wc_cost.time
                            operation_obj.write(cr, uid, [operation.id], {'tot_mac':op_time_add , 'tot':tot_time_add  }) 
                        # Busco el Apunte Analítico con el importe viejo
                        usr_search = wc_obj.search(cr, uid,[('id', '=', wc_cost.wc.id)])
                        wc = wc_obj.browse(cr , uid, usr_search[0])
                        if not wc.product_id:
                            raise osv.except_osv(_('You must assign a product to all machines'), _(wc.name))                
                        aux_product = product_obj.browse(cr , uid, wc.product_id.id) 
                        op_cost_amount = aux_product.standard_price * wc_cost.time
                        
                        ope_account_id = False
                        ope_acc_code = 'OF'+ str(wc_cost.production.name.lstrip('OF/'))  + '-OP' + str(wc_cost.operation.sequence) + '-MA'
                        account_search = analytic_account_obj.search(cr, uid, [('code','=',ope_acc_code)])
                        if not account_search:
                            raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(ope_acc_code))
                            
                        for aux_var in account_search:
                            ope_account_id = aux_var
                        man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])        
                        man_journal = account_analytic_jour_obj.browse(cr, uid, man_journal_search[0])
        
         
                        analytic_line_ids = analytic_account_line_obj.search(cr, uid, [('amount','=',op_cost_amount*(-1)),
                                                                                       ('name','=',wc_cost.descript),
                                                                                       ('date','=',wc_cost.date),
                                                                                       ('company_id','=',wc.factory_id.company_id.id),
                                                                                       ('general_account_id','=',wc.factory_id.company_id.workcenter_account.id),
                                                                                       ('account_id','=',ope_account_id),
                                                                                       ('journal_id','=',man_journal.id)])

                        # Llamo al metodo super
                        super(mrp_wc_cost, self).write(cr, uid, [wc_cost.id], vals, context=context)
                        wc_costs2 = self.browse(cr,uid,wc_cost.id)
                        
                        # Imputo Tiempos de producción
                        if wc_costs2.production:
                            production = production_obj.browse(cr, uid, wc_costs2.production.id)
                            values = {'tot': production.tot + wc_costs2.time,
                                      'tot_mac': production.tot_mac + wc_costs2.time
                                      }
                            production_obj.write(cr, uid, [production.id], values)  
                        # Imputo Tiempos de la operación
                        if wc_costs2.operation:
                            operation = operation_obj.browse(cr , uid, wc_costs2.operation.id)
                            op_time_add = operation.tot_ops + wc_costs2.time
                            tot_time_add = operation.tot + wc_costs2.time
                            operation_obj.write(cr, uid, [operation.id], {'tot_mac':op_time_add , 'tot':tot_time_add  }) 
                        # Modifico el Importe Analítico con el Importe Nuevo  
                        op_cost_amount = aux_product.standard_price * wc_costs2.time
                        analytic_account_line_obj.write(cr, uid, [analytic_line_ids[0]], {'amount': op_cost_amount*(-1)}) 
                        analytic_account_obj.recalculate_result(cr, uid, ope_account_id)
        
        return True
    
mrp_wc_cost()