# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import osv, fields
from datetime import datetime
from dateutil.relativedelta import relativedelta
from tools.translate import _
import netsvc

class mrp_production(osv.osv):
    _name = "mrp.production"
    _inherit = "mrp.production"
    
    def create(self, cr, uid, data, context=None):
        data.update({'with_analytic_control': True})
        return super(mrp_production, self).create(cr, uid, data, context)

    def action_button_confirm(self, cr, uid, ids, *args):
        res = super(mrp_production,self).action_button_confirm(cr, uid, ids, *args)
        #
        # Creo la estructura analítica automaticamente
        self.create_analytic_structure2(cr, uid, ids, context=None)

        return res
 
    def _get_default_account2(self, cr, uid, context=None):
        # TODO: Change this to use code instead for translational option
        # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')])
        analytic_account = self.pool.get('account.analytic.account').search(cr, uid,[('name', '=', 'Manufacturing')])
        if analytic_account:
            return analytic_account[0]
        else:
            return False
    
    def create_analytic_structure2(self, cr, uid, ids, context=None): 
        if context is None:
            context = {}

        # TODO: Add translational option
        # parent_op_acc_name = _('Manufacturing')
        parent_op_acc_name = 'Manufacturing'
        mrp_production_obj = self.pool.get('mrp.production')  
        prod_brow = mrp_production_obj.browse(cr, uid,ids) 
        operations_obj = self.pool.get('mrp.production.operation')
        account_analytic_obj = self.pool.get('account.analytic.account')
        account_analytic_jour_obj = self.pool.get('account.analytic.journal')
        account_analytic_line_obj = self.pool.get('account.analytic.line')
        res_company_obj = self.pool.get('res.company')
        result = []
        main_account = False
        
        manufacturing_account_id = self._get_default_account2(cr, uid, context)
        if manufacturing_account_id:
            main_account = account_analytic_obj.browse(cr,uid, manufacturing_account_id)
        
        for prod in prod_brow:
            if not (prod.company_id.material_account or prod.company_id.workcenter_account or prod.company_id.operator_account):
                raise osv.except_osv(_('Unable to perform action'), _('MRP accounts NOT configured for this company.'))     
            else:
                w_company_id = prod.company_id.id     
                
        company = res_company_obj.browse(cr,uid,w_company_id)   
        
        #Comprobación de que la orden esta iniciada
        for prod in prod_brow:
            if prod.state == 'draft':
                raise osv.except_osv(_('Unable to perform action'), _('You cannot create analytic accounting before starting the production order.'))      
        
        #Comprobación y asignación del diario analitico
        
        # TODO: Change this to use code instead for translational option
        # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')])   
        man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])        
        if not man_journal_search:               
            values={
                 'name': parent_op_acc_name,
                 'code': 'MRP',
                 'type': 'general',
                 'company_id': company.id,
                 }
            man_journal_id = account_analytic_jour_obj.create(cr, uid, values)
            man_journal = account_analytic_jour_obj.browse(cr, uid, man_journal_id)
        else:
            man_journal = account_analytic_jour_obj.browse(cr, uid, man_journal_search[0])
        
        if not man_journal:
            raise osv.except_osv(_('Unable to perform action'), _('The system is not able to find the main account analytic journal.'))
        #Comprobación y creación de cuenta analitica general de fabricación
        
        if main_account:
            parent_op_acc = main_account
        else:
            parent_acc_search = account_analytic_obj.search(cr, uid,[('name', '=', parent_op_acc_name)])
                
            if parent_acc_search:
                parent_op_acc = account_analytic_obj.browse(cr, uid, parent_acc_search[0])
            else:
                values = {
                     'name': parent_op_acc_name,
                     'code': parent_op_acc_name,
                     'currency_id': company.currency_id.id,
                     'company_id': company.id,
                     'state': 'open',
                     'acc_type': 'MF',
                     'total_estimated': 0.0,
                     'total_estimated_puc': 0.0,
                     'total_estimated_standard': 0.0,
                          }
                parent_op_acc_create = account_analytic_obj.create(cr, uid, values)
                parent_op_acc = account_analytic_obj.browse(cr, uid,parent_op_acc_create)
                result.append(values)    
            
        #Comprobación y creación de cuenta analitica de orden de fabricación
        op_acc_code = False 
        production_id = False
        progress = 90 / len(prod_brow)
        for prod in prod_brow:
            production_id = prod.id
            mrp_production = mrp_production_obj.browse(cr,uid,production_id)
            op_acc_code = 'OF' + str(mrp_production.name.lstrip('OF/'))
            acc_search = account_analytic_obj.search(cr, uid,[('code', '=', op_acc_code)])
            
            if  acc_search:
                   raise osv.except_osv(_('Unable to perform action'), _('Analytic account for this Manufacturing order already exists.'))             
            else:  
                values={
                 'name': op_acc_code + " " +prod.name +" -> "+prod.product_id.name,
                 'code': op_acc_code,
                 'currency_id': company.currency_id.id,
                 'company_id': company.id,
                 'state': 'open',
                 'parent_id': parent_op_acc.id,
                 'total_estimated': 0.0,
                 'total_estimated_puc': 0.0,
                 'total_estimated_standard': 0.0,
                 'acc_type': 'OF',
                 'mrp_production_id': prod.id
                }
                man_account = account_analytic_obj.create(cr, uid, values)
                values['name'] = '|            '+values['name']                
                result.append(values)
                operations_obj_search = operations_obj.search(cr, uid, [('production_id','=',prod.id)])
                operations = operations_obj.browse(cr, uid, operations_obj_search)
                
                #Cuenta analitica de costes fijos
                    
                prod_maq_name_acc_code = op_acc_code + '-FIX'    
                cost_fixed = prod.fixed_costs
                est = 0
                for fin in cost_fixed:
                    est += fin.estimated   
                values={
                    'name': prod_maq_name_acc_code + " " + _('Fixed') ,
                    'code': prod_maq_name_acc_code ,
                    'currency_id': company.currency_id.id,
                    'company_id': company.id,
                    'state': 'open',
                    'parent_id': man_account,
                    'total_estimated': est,
                    'acc_type': 'FIX',
                }   
                man_fix_account = account_analytic_obj.create(cr, uid, values)
                values['name'] = '|                                              '+values['name']
                values['parent_id'] = man_fix_account
                result.append(values)
                account_analytic_obj.recalculate_result(cr, uid, man_fix_account) 
                account_analytic_obj.calculate_estimated(cr, uid, man_account, est, 0, 0)
                    
                    #añadir costes reales de costes fijos
                
                for fin in cost_fixed:     
                    values = {
                        'amount': fin.amount*(-1),
                        'name': fin.name,
                        'date': datetime.today(),
                        'company_in': company.id,
                        'account_id': man_fix_account,
                        'general_account_id': company.material_account.id,
                        'journal_id': man_journal.id,
                    }
                    new_id = account_analytic_line_obj.create(cr, uid, values)          
                    account_analytic_obj.recalculate_result(cr, uid, man_fix_account)  
                
                #Creacion de las cuentas analiticas de operaciones de fabricacion

                for prod_op in operations:
                    prod_op_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) 
                    values = {
                        'name': prod_op_name_acc_code + " " + prod_op.name,
                        'code': prod_op_name_acc_code,
                        'currency_id': company.currency_id.id,
                        'company_id': company.id,
                        'state': 'open',
                        'parent_id': man_account,
                        'total_estimated': 0.0,
                        'total_estimated_puc': 0.0,
                        'total_estimated_standard': 0.0,
                        'acc_type': 'OP'
                        }
                    man_op_account = account_analytic_obj.create(cr, uid, values)
                    values['name'] = '|                        '+values['name']
                    result.append(values)
                    
                    #Cuenta analitica de maquina
                    
                    prod_maq_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-MA' 
                    maq_obj = self.pool.get('mrp.workcenter')
                    maq = maq_obj.browse(cr, uid, prod_op.workcenter_id.id)
                    if not maq.product_id:
                        raise osv.except_osv(_('You must assign a product to all machines'), _(maq.name))                
                    
                    aux_product = self.pool.get('product.product').browse(cr , uid, maq.product_id.id) 
                    total_estimated = prod_op.hour * aux_product.standard_price
                    values = {
                        'name': prod_maq_name_acc_code + " " + _('Machine') + " " + prod_op.workcenter_id.name,
                        'code': prod_maq_name_acc_code ,
                        'currency_id': company.currency_id.id,
                        'company_id': company.id,
                        'state': 'open',
                        'parent_id': man_op_account,
                        'total_estimated': total_estimated,
                        'acc_type': 'MA'
                    }
                    account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                    man_maq_account = account_analytic_obj.create(cr, uid, values)
                    values['name'] = '|                                              '+values['name']
                    values['parent_id'] = man_maq_account
                    result.append(values)
                    account_analytic_obj.recalculate_result(cr, uid, man_maq_account)
                    
                    #Cuenta analitica de materiales
                    
                    prod_mat_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-MAT' 
                    total_estimated = 0
                    for aux_mat in prod_op.components_used_ids:
                        total_estimated += aux_mat.product_id.standard_price * aux_mat.product_qty 
                    values = {
                        'name': prod_mat_name_acc_code + " " + _('Materials') ,
                        'code': prod_mat_name_acc_code ,
                        'currency_id': company.currency_id.id,
                        'company_id': company.id,
                        'state': 'open',
                        'parent_id': man_op_account,
                        'total_estimated': total_estimated,
                        'acc_type': 'MT'
                    }
                    account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                    man_mat_account = account_analytic_obj.create(cr, uid, values)
                    values['name'] = '|                                              '+values['name']
                    values['parent_id'] = man_mat_account
                    result.append(values)
                    account_analytic_obj.recalculate_result(cr, uid, man_mat_account)
                    
                    #Lineas analiticas de los materiales en las operaciones realizadas hasta ahora
                    real_obj = self.pool.get('mrp.production.operation.real')
                    real_search = real_obj.search(cr, uid, [('state','=','done'),('mrp_manu_op','=',prod_op.id)])
                    for id in real_search:
                        self.mats_final_cost(cr, uid, id, company.id)
                    
                    #Crear cuenta analitica de operadores por estimación
                    
                    manu_users_obj = self.pool.get('mrp.production.users')
                    manu_tot_users = manu_users_obj.search(cr, uid, [('op','=',prod_op.id)])
                    usrs = manu_users_obj.browse(cr, uid, manu_tot_users)
                    for manufact_worker in usrs:
                            manu_users_search = manu_users_obj.search(cr, uid, [('name','=',manufact_worker.name.id),('op','=',prod_op.id)])
                            manus = manu_users_obj.browse(cr, uid, manu_users_search)
                            
                            user_employer = self.pool.get('hr.employee').browse(cr, uid, self.pool.get('hr.employee').search(cr, uid, [('user_id','=',manufact_worker.name.id)]))
                            if user_employer:
                                if not user_employer[0].product_id:
                                    raise osv.except_osv(_('Unable to perform action'), _('%s haven\'t got product assigned!') %(user_employer[0].name))
                                user_product = self.pool.get('product.product').browse(cr, uid, user_employer[0].product_id.id)
                                total_estimated=0
                                
                                for aux_user in manus:
                                    total_estimated = total_estimated + (manufact_worker.percent * user_product.standard_price * prod_op.hour)
                                   
                                prod_wrk_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-WK'+ str(manufact_worker.name.id) 
                                prod_wrk_acc = account_analytic_obj.search(cr, uid, [('code','=', prod_wrk_name_acc_code)])
                                
                                #compruebo que existe y adjudico el estimado o no existe y lo creo
                                
                                if prod_wrk_acc:
                                    wrk_acc = account_analytic_obj.browse(cr, uid, prod_wrk_acc)[0]
                                    wrk_acc.total_estimated = total_estimated
                                else:
                                    nombre  = manufact_worker.name.name
                                    values={
                                        'name': prod_wrk_name_acc_code + " " + _('Worker')+ " " + str(nombre).encode('ascii', 'replace'),
                                        'code': prod_wrk_name_acc_code ,
                                        'currency_id': company.currency_id.id,
                                        'company_id': company.id,
                                        'state': 'open',
                                        'parent_id': man_op_account,
                                        'total_estimated': total_estimated,
                                        'acc_type': 'WK'
                                    }
                                    account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                                    man_wrk_account = account_analytic_obj.create(cr, uid, values)
                                    values['name'] = '|                                              '+values['name']
                                    values['parent_id'] = man_wrk_account
                                    result.append(values)
                                    account_analytic_obj.recalculate_result(cr, uid, man_wrk_account)

                    #Crear cuenta analitica de operadores por inputación
                    
                    usr_obj = self.pool.get('mrp.ops.cost')
                    usr_search = usr_obj.search(cr,uid, [('operation','=',prod_op.id)])
                    usrs = usr_obj.browse(cr, uid, usr_search)
                    for manufact_worker in usrs:                                                                      
                            nombre  = manufact_worker.user.name
                            prod_wrk_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-WK'+ str(manufact_worker.user.id) 
                            prod_wrk_acc = account_analytic_obj.search(cr, uid, [('code','=', prod_wrk_name_acc_code)])
                            
                            #compruebo que existe la cuenta analitica de este operario sino la creo
                            
                            if not prod_wrk_acc:
                                values = {
                                    'name': prod_wrk_name_acc_code + " " + _('Operator') + " " + str(nombre).encode('ascii', 'replace'),
                                    'code': prod_wrk_name_acc_code ,
                                    'currency_id': company.currency_id.id,
                                    'company_id': company.id,
                                    'state':'open',
                                    'parent_id': man_op_account,
                                    'total_estimated': 0,
                                    'total_estimated_puc': 0,
                                    'total_estimated_standard': 0,
                                    'acc_type': 'WK'
                                }
                                account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                                man_wrk_account = account_analytic_obj.create(cr, uid, values)
                                values['name'] = '|                                              '+values['name']
                                values['parent_id'] = man_wrk_account
                                result.append(values)
                                account_analytic_obj.recalculate_result(cr, uid, man_wrk_account)
        
        #Creacion de lineas analiticas de los datos existentes
        
        ops_op_obj = self.pool.get('mrp.ops.cost')
        ops_wc_obj = self.pool.get('mrp.wc.cost')
        
        #Costes de maquinas
        
        ops_wc_obj_all = ops_wc_obj.browse(cr, uid, ops_wc_obj.search(cr, uid, [('production','=',production_id)]))
        for wc_cost in ops_wc_obj_all:
            wc_cost_amount = wc_cost.wc.costs_hour * wc_cost.time
            wc_acc_code = op_acc_code + '-OP' + str(wc_cost.operation.sequence) + '-MA'
            wc_account_id = False
            for aux_result in result:
                if str(aux_result['code']) == (wc_acc_code):
                    wc_account_id =  aux_result['parent_id']
                    
            if not wc_account_id:
                raise osv.except_osv(_('Programing Error'), _('The analytic account of the machine is not created. Code: %s') %(wc_acc_code))
            
            # TODO: Change this to use code instead for translational option
            # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')]) 
            man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])
            values = {
                      'amount': wc_cost_amount*(-1),
                      'name': wc_cost.descript,
                      'date': wc_cost.date,
                      'company_in': company.id,
                      'account_id': wc_account_id,
                      'general_account_id': company.workcenter_account.id,
                      'journal_id': man_journal_search[0],
            }
            new_id = account_analytic_line_obj.create(cr, uid, values)          
            account_analytic_obj.recalculate_result(cr, uid, wc_account_id)
            
        #Costes de operarios    
         
        ops_op_obj_all = ops_op_obj.browse(cr, uid, ops_op_obj.search(cr, uid, [('production','=',production_id)]))
        for op_cost in ops_op_obj_all:
            usr_obj = self.pool.get('hr.employee')
            usr_search = usr_obj.search(cr, uid,[('user_id', '=', op_cost.user.id)])
            usr = usr_obj.browse(cr , uid, usr_search[0])
            if not usr.product_id:
                raise osv.except_osv(_('You must assign a product to all employers'), _(usr.name))                
            aux_product = self.pool.get('product.product').browse(cr , uid, usr.product_id.id)
            op_cost_amount = aux_product.standard_price * op_cost.time
            ope_acc_code = op_acc_code + '-OP' + str(op_cost.operation.sequence) + '-WK'+ str(op_cost.user  .id)
            ope_account_id = False
            for aux_result in result:
                if str(aux_result['code']) == (ope_acc_code):
                    ope_account_id =  aux_result['parent_id']
            if not ope_account_id:
                raise osv.except_osv(_('Programing Error'), _('The analytic account of the operator is not created. Code: %s') %(ope_acc_code))
            
            # TODO: Change this to use code instead for translational option
            # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')]) 
            man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])
            values = {
                      'amount':op_cost_amount*(-1),
                      'name':op_cost.descript,
                      'date':op_cost.date,
                      'company_in': company.id,
                      'account_id': ope_account_id,
                      'general_account_id': company.operator_account.id,
                      'journal_id':man_journal_search[0],
            }
            new_id = account_analytic_line_obj.create(cr, uid, values)          
            account_analytic_obj.recalculate_result(cr, uid, ope_account_id)
            
        for prod in prod_brow:
            if 'mrp_production' in context:
                context.update({'mrp_production': context['mrp_production'] + ', ' + prod.name})
            else:
                context.update({'mrp_production': prod.name})
                
        return True
    
mrp_production()

class mrp_fixed_cost(osv.osv):
    _inherit = 'mrp.fixed.cost'
    _name = 'mrp.fixed.cost'
    _description = 'MRP fixed costs'               
#
#    def change_name(self, cr, uid, ids, name):
#        me = self.browse(cr, uid, ids)[0]
#        if True  :
#                warning = {
#                    'title': _('Warning!'),
#                    'message': _('Not allowed to change the name of fixed costs!')
#                        }
#                return { 'warning': warning}
#        else:
#            return {'value':{'created':True}}

    def unlink(self, cr, uid, ids, context=None):
        # TODO: FIX delete? Spanish in context...
        if 'borrar' in context:
            super(mrp_fixed_cost, self).unlink(cr, uid, ids, context=context)    
        else:
            raise osv.except_osv(_('Unable to perform action'), _('Not allowed to delete the fixed costs, set its quantity to 0!'))
        return True

    def write(self, cr, uid, ids, vals, context=None):
        fix = self.browse(cr, uid, ids, context)
        if vals['name'] <> fix[0].name:
            raise osv.except_osv(_('Unable to perform action'), _('Not allowed to change the name of fixed costs!'))
        res = super(mrp_fixed_cost, self).write(cr, uid, ids, vals, context=context)        
        
        account_analytic_line_obj = self.pool.get('account.analytic.line')
        account_analytic_obj = self.pool.get('account.analytic.account')
        
        for cost in fix:
	    code = 'OF' + str(cost.production_id.name.lstrip('OF/')) + '-FIX'
            fix_acc_search = account_analytic_obj.search(cr, uid, [('code','=','OF'+str(cost.production_id.id)+ "FIX" )])
            if fix_acc_search:
                line_search = account_analytic_line_obj.search(cr, uid, [('name','=',cost.name),('account_id','=',fix_acc_search[0])])
                if line_search:
                    account_analytic_line_obj.write(cr, uid, line_search[0],{'amount': (cost.amount*(-1))})
                    account_analytic_obj.recalculate_result(cr, uid, fix_acc_search[0])
                else:
                    raise osv.except_osv(_('Programming error'), _('Error in reading the analytic account line'))
            else:
                raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(code))
        return res
    
    def create(self, cr, uid, vals, context=None):
        mrp_production_obj = self.pool.get('mrp.production')
        mrp_production = mrp_production_obj.browse(cr,uid,vals['production_id'])
        fix_created = super(mrp_fixed_cost, self).create(cr, uid, vals, context=context)        
        account_analytic_obj = self.pool.get('account.analytic.account')
        account_analytic_jour_obj = self.pool.get('account.analytic.journal')
        code = 'OF' + str(mrp_production.name.lstrip('OF/'))
        acc_search = account_analytic_obj.search(cr, uid,[('code','=',code)])
        if not acc_search:
            raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(code))
            
        acc_id = False
        if acc_search:
            acc_id = acc_search[0]
        # TODO: Change this to use code instead for translational option
        # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')])
        man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])        
        
        if acc_id:
           
           account_analytic_line_obj = self.pool.get('account.analytic.line')
           op_acc_code = False
           prod_maq_name_acc_code = False
           code = 'OF' + str(mrp_production.name.lstrip('OF/')) + '-FIX'
           man_fix_account = account_analytic_obj.search(cr, uid,[('code','=',code)])[0]
           
           if not man_fix_account:         
               #Crear cuenta analitica de costes fijos                   
               est = 0                                                
               values={
                    'name': prod_maq_name_acc_code + " " + _('Fixed'),
                    'code': prod_maq_name_acc_code,
                    'currency_id': prod.company_id.currency_id.id,
                    'company_id': prod.company_id.id,
                    'state': 'open',
                    'parent_id': acc_id,
                    'total_estimated': est,
                    'total_estimated_puc': 0,
                    'total_estimated_standard' : 0,
                    'acc_type': 'FIX'
                  }
                            
                            
               man_fix_account = account_analytic_obj.create(cr, uid, values)
               account_analytic_obj.recalculate_result(cr, uid, man_fix_account) 
                            
           
                            #añadir costes reales de costes fijos                        
                                
           values = {
               'amount': vals['amount']*(-1),
               'name': vals['name'],
               'date': datetime.today(),
               'company_in': prod.company_id.id,
               'account_id': man_fix_account,
               'general_account_id': prod.company_id.material_account.id, 
               'journal_id': man_journal_search[0],
               }
           new_id = account_analytic_line_obj.create(cr, uid, values)          
           account_analytic_obj.recalculate_result(cr, uid, man_fix_account)        
                    
        return True
    
mrp_fixed_cost()