# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import time
from datetime import datetime

from osv import osv, fields
import tools
from tools.translate import _


class mrp_analytic_wizard(osv.osv_memory):    
    _name = "mrp.analytic.wizard"
    _description = "Wizard to create account analityc schema"
       
    _columns = {
          'company_id': fields.many2one('res.company','Company', required=True, readonly=True),
          'main_account': fields.many2one('account.analytic.account','Main Analytic Account'),
    }
        
    
    def get_default_account(self, cr, uid, context=None):
        # TODO: Change this to use code instead for translational option
        # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')])
        analytic_account = self.pool.get('account.analytic.account').search(cr, uid,[('name', '=', 'Manufacturing')])
        if analytic_account:
            return analytic_account[0]
        else:
            return False
    
    def default_get(self, cr, uid, fields_list, context=None):
        values = {} 
        if context == None:
            context = {}
        for order in self.pool.get('mrp.production').browse(cr, uid, context['active_ids']):
            values = {
                'company_id': order.company_id.id,
                'main_account': self.get_default_account(cr, uid, context),
            }
        return values
    
    def mats_final_cost(self, cr, uid, op, company_id):
        company = self.pool.get('res.company').browse(cr, uid, company_id)
        if not company.material_account:
            raise osv.except_osv(_('End Operation Error'), _('You must define one account for material cost'))  
        if not company.workcenter_account:
            raise osv.except_osv(_('End Operation Error'), _('You must define one account for workcenter cost'))
        real_obj = self.pool.get('mrp.production.operation.real')
        real = real_obj.browse(cr, uid, op)
        
        man_op = real.mrp_manu_op
        production = real.production_id
        
        account_analytic_jour_obj = self.pool.get('account.analytic.journal')
        account_analytic_obj = self.pool.get('account.analytic.account')
        account_analytic_line_obj = self.pool.get('account.analytic.line')
        result = []
        
        #Cuenta analitica de materiales
        man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])        
        prod_mat_name_acc_code = 'OF'+ str(production.name.lstrip('OF/')) + '-OP' + str(man_op.sequence) + '-MAT'     
        account_analytic_search = account_analytic_obj.search(cr, uid,[('code', '=', prod_mat_name_acc_code)]) 
        if not account_analytic_search:
            raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(prod_mat_name_acc_code))
        if  account_analytic_search:
            acc_id = account_analytic_search[0]
            mats_amount = 0
            for aux in real.move_lines2:
                if aux.state == 'done':
                    mats_amount += aux.product_id.standard_price * aux.product_qty
            values = {'amount': mats_amount*(-1),
                      'name': aux.product_id.name_template + '  imputation',
                      'date': real.date_finished,
                      'company_id': company.id,
                      'account_id': acc_id,
                      'general_account_id': company.material_account.id,
                      'journal_id': man_journal_search[0],
                      'ref': production.name,
                      'unit_amount': 1,
                      'product_id': aux.product_id.id
                      }
            new_id = account_analytic_line_obj.create(cr, uid, values)          
            account_analytic_obj.recalculate_result(cr, uid, acc_id)  
            
            
        # IMPUTO EL CENTRO DE TRABAJO
        #    
        #busqueda y asignación de cuenta analitica correspondiente
        ope_acc_code = 'OF'+ str(production.name.lstrip('OF/')) + '-OP' + str(man_op.sequence) + '-MA'
        ope_account_id = False
        account_search = account_analytic_obj.search(cr, uid, [('code','=',ope_acc_code)])
        if not account_search:
            raise osv.except_osv(_('ERROR'), _('Analytic Account with code: %s NOT FOUND') %(ope_acc_code))
        for aux_var in account_search:
            ope_account_id = aux_var
        if not ope_account_id:
            raise osv.except_osv(_('End Operation Error'), _('Analytic account: %s NOT FOUND') %(ope_acc_code))                       
        man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])        
        man_journal = account_analytic_jour_obj.browse(cr, uid, man_journal_search[0])
        w_amount = real.delay * real.workcenter_id.costs_hour
        w_amount = w_amount * (-1)
        if not real.workcenter_id.product_id:
            raise osv.except_osv(_('End Operation Error'), _('You must define one product for workcenter cost'))
            
        values = {'amount': w_amount,
                  'name': real.workcenter_id.name + ' workcenter imputation',
                  'date': real.date_finished,
                  'company_id': company.id,
                  'account_id': ope_account_id,
                  'general_account_id': company.workcenter_account.id,
                  'journal_id': man_journal.id,
                  'ref': production.name,
                  'unit_amount': 1,
                  'product_id': real.workcenter_id.product_id.id
                  }
        new_id = account_analytic_line_obj.create(cr, uid, values)     
        account_analytic_obj.recalculate_result(cr, uid, ope_account_id)       
        
        
        return True  
    
    def create_analytic_structure(self, cr, uid, ids, context=None): 
        if context is None:
            context = {}
        
        wizard_config = self.browse(cr, uid, ids)
        if wizard_config:
            wizard_config = wizard_config[0]
        else:
            return {'type': 'ir.actions.act_window_close'} 
        # TODO: Add translational option
        # parent_op_acc_name = _('Manufacturing')
        parent_op_acc_name = 'Manufacturing'
        mrp_production_obj = self.pool.get('mrp.production')  
        prod_brow = mrp_production_obj.browse(cr, uid,context['active_ids']) 
        operations_obj = self.pool.get('mrp.production.operation')
        account_analytic_obj = self.pool.get('account.analytic.account')
        account_analytic_jour_obj = self.pool.get('account.analytic.journal')
        account_analytic_line_obj = self.pool.get('account.analytic.line')
        result = []
        
        if not (wizard_config.company_id.material_account or  wizard_config.company_id.workcenter_account or wizard_config.company_id.operator_account):
            raise osv.except_osv(_('Unable to perform action'), _('MRP accounts NOT configured for this company.'))             
        
        #Comprobación de que la orden esta iniciada
        for prod in prod_brow:
            if prod.state == 'draft':
                raise osv.except_osv(_('Unable to perform action'), _('You cannot create analytic accounting before starting the production order.'))      
        
        #Comprobación y asignación del diario analitico
        
        # TODO: Change this to use code instead for translational option
        # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')])   
        man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])        
        if not man_journal_search:               
            values={
                 'name': parent_op_acc_name,
                 'code': 'MRP',
                 'type': 'general',
                 'company_id': wizard_config.company_id.id,
                 }
            man_journal_id = account_analytic_jour_obj.create(cr, uid, values)
            man_journal = account_analytic_jour_obj.browse(cr, uid, man_journal_id)
        else:
            man_journal = account_analytic_jour_obj.browse(cr, uid, man_journal_search[0])
        
        if not man_journal:
            raise osv.except_osv(_('Unable to perform action'), _('The system is not able to find the main account analytic journal.'))
        #Comprobación y creación de cuenta analitica general de fabricación
        
        if wizard_config.main_account:
            parent_op_acc = wizard_config.main_account
        else:
            parent_acc_search = account_analytic_obj.search(cr, uid,[('name', '=', parent_op_acc_name)])
                
            if parent_acc_search:
                parent_op_acc = account_analytic_obj.browse(cr, uid, parent_acc_search[0])
            else:
                values = {
                     'name': parent_op_acc_name,
                     'code': parent_op_acc_name,
                     'currency_id': wizard_config.company_id.currency_id.id,
                     'company_id': wizard_config.company_id.id,
                     'state': 'open',
                     'acc_type': 'MF',
                     'total_estimated': 0.0,
                     'total_estimated_puc': 0.0,
                     'total_estimated_standard': 0.0,
                          }
                parent_op_acc_create = account_analytic_obj.create(cr, uid, values)
                parent_op_acc = account_analytic_obj.browse(cr, uid,parent_op_acc_create)
                result.append(values)    
            
        #Comprobación y creación de cuenta analitica de orden de fabricación
        op_acc_code = False 
        production_id = False
        progress = 90 / len(prod_brow)
        for prod in prod_brow:
            production_id = prod.id
            mrp_production = mrp_production_obj.browse(cr,uid,production_id)
            op_acc_code = 'OF' + str(mrp_production.name.lstrip('OF/'))
            acc_search = account_analytic_obj.search(cr, uid,[('code', '=', op_acc_code)])
            
            if  acc_search:
                   raise osv.except_osv(_('Unable to perform action'), _('Analytic account for this Manufacturing order already exists.'))             
            else:  
                values={
                 'name': op_acc_code + " " +prod.name +" -> "+prod.product_id.name,
                 'code': op_acc_code,
                 'currency_id': wizard_config.company_id.currency_id.id,
                 'company_id': wizard_config.company_id.id,
                 'state': 'open',
                 'parent_id': parent_op_acc.id,
                 'total_estimated': 0.0,
                 'total_estimated_puc': 0.0,
                 'total_estimated_standard': 0.0,
                 'acc_type': 'OF',
                 'mrp_production_id': prod.id
                }
                man_account = account_analytic_obj.create(cr, uid, values)
                values['name'] = '|            '+values['name']                
                result.append(values)
                operations_obj_search = operations_obj.search(cr, uid, [('production_id','=',prod.id)])
                operations = operations_obj.browse(cr, uid, operations_obj_search)
                
                #Cuenta analitica de costes fijos
                    
                prod_maq_name_acc_code = op_acc_code + '-FIX'    
                cost_fixed = prod.fixed_costs
                est = 0
                for fin in cost_fixed:
                    est += fin.estimated   
                values={
                    'name': prod_maq_name_acc_code + " " + _('Fixed') ,
                    'code': prod_maq_name_acc_code ,
                    'currency_id': wizard_config.company_id.currency_id.id,
                    'company_id': wizard_config.company_id.id,
                    'state': 'open',
                    'parent_id': man_account,
                    'total_estimated': est,
                    'acc_type': 'FIX',
                }   
                man_fix_account = account_analytic_obj.create(cr, uid, values)
                values['name'] = '|                                              '+values['name']
                values['parent_id'] = man_fix_account
                result.append(values)
                account_analytic_obj.recalculate_result(cr, uid, man_fix_account) 
                account_analytic_obj.calculate_estimated(cr, uid, man_account, est, 0, 0)
                    
                    #añadir costes reales de costes fijos
                
                for fin in cost_fixed:     
                    values = {
                        'amount': fin.amount*(-1),
                        'name': fin.name,
                        'date': datetime.today(),
                        'company_in': wizard_config.company_id.id,
                        'account_id': man_fix_account,
                        'general_account_id': wizard_config.company_id.material_account.id,
                        'journal_id': man_journal.id,
                    }
                    new_id = account_analytic_line_obj.create(cr, uid, values)          
                    account_analytic_obj.recalculate_result(cr, uid, man_fix_account)  
                
                #Creacion de las cuentas analiticas de operaciones de fabricacion

                for prod_op in operations:
                    prod_op_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) 
                    values = {
                        'name': prod_op_name_acc_code + " " + prod_op.name,
                        'code': prod_op_name_acc_code,
                        'currency_id': wizard_config.company_id.currency_id.id,
                        'company_id': wizard_config.company_id.id,
                        'state': 'open',
                        'parent_id': man_account,
                        'total_estimated': 0.0,
                        'total_estimated_puc': 0.0,
                        'total_estimated_standard': 0.0,
                        'acc_type': 'OP'
                        }
                    man_op_account = account_analytic_obj.create(cr, uid, values)
                    values['name'] = '|                        '+values['name']
                    result.append(values)
                    
                    #Cuenta analitica de maquina
                    
                    prod_maq_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-MA' 
                    maq_obj = self.pool.get('mrp.workcenter')
                    maq = maq_obj.browse(cr, uid, prod_op.workcenter_id.id)
                    if not maq.product_id:
                        raise osv.except_osv(_('You must assign a product to all machines'), _(maq.name))                
                    
                    aux_product = self.pool.get('product.product').browse(cr , uid, maq.product_id.id) 
                    total_estimated = prod_op.hour * aux_product.standard_price
                    values = {
                        'name': prod_maq_name_acc_code + " " + _('Machine') + " " + prod_op.workcenter_id.name,
                        'code': prod_maq_name_acc_code ,
                        'currency_id': wizard_config.company_id.currency_id.id,
                        'company_id': wizard_config.company_id.id,
                        'state': 'open',
                        'parent_id': man_op_account,
                        'total_estimated': total_estimated,
                        'acc_type': 'MA'
                    }
                    account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                    man_maq_account = account_analytic_obj.create(cr, uid, values)
                    values['name'] = '|                                              '+values['name']
                    values['parent_id'] = man_maq_account
                    result.append(values)
                    account_analytic_obj.recalculate_result(cr, uid, man_maq_account)
                    
                    #Cuenta analitica de materiales
                    
                    prod_mat_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-MAT' 
                    total_estimated = 0
                    for aux_mat in prod_op.components_used_ids:
                        total_estimated += aux_mat.product_id.standard_price * aux_mat.product_qty 
                    values = {
                        'name': prod_mat_name_acc_code + " " + _('Materials') ,
                        'code': prod_mat_name_acc_code ,
                        'currency_id': wizard_config.company_id.currency_id.id,
                        'company_id': wizard_config.company_id.id,
                        'state': 'open',
                        'parent_id': man_op_account,
                        'total_estimated': total_estimated,
                        'acc_type': 'MT'
                    }
                    account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                    man_mat_account = account_analytic_obj.create(cr, uid, values)
                    values['name'] = '|                                              '+values['name']
                    values['parent_id'] = man_mat_account
                    result.append(values)
                    account_analytic_obj.recalculate_result(cr, uid, man_mat_account)
                    
                    #Lineas analiticas de los materiales en las operaciones realizadas hasta ahora
                    real_obj = self.pool.get('mrp.production.operation.real')
                    real_search = real_obj.search(cr, uid, [('state','=','done'),('mrp_manu_op','=',prod_op.id)])
                    for id in real_search:
                        self.mats_final_cost(cr, uid, id, wizard_config.company_id.id)
                    
                    #Crear cuenta analitica de operadores por estimación
                    
                    manu_users_obj = self.pool.get('mrp.production.users')
                    manu_tot_users = manu_users_obj.search(cr, uid, [('op','=',prod_op.id)])
                    usrs = manu_users_obj.browse(cr, uid, manu_tot_users)
                    for manufact_worker in usrs:
                            manu_users_search = manu_users_obj.search(cr, uid, [('name','=',manufact_worker.name.id),('op','=',prod_op.id)])
                            manus = manu_users_obj.browse(cr, uid, manu_users_search)
                            
                            user_employer = self.pool.get('hr.employee').browse(cr, uid, self.pool.get('hr.employee').search(cr, uid, [('user_id','=',manufact_worker.name.id)]))
                            if user_employer:
                                if not user_employer[0].product_id:
                                    raise osv.except_osv(_('Unable to perform action'), _('%s haven\'t got product assigned!') %(user_employer[0].name))
                                user_product = self.pool.get('product.product').browse(cr, uid, user_employer[0].product_id.id)
                                total_estimated=0
                                
                                for aux_user in manus:
                                    total_estimated = total_estimated + (manufact_worker.percent * user_product.standard_price * prod_op.hour)
                                   
                                prod_wrk_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-WK'+ str(manufact_worker.name.id) 
                                prod_wrk_acc = account_analytic_obj.search(cr, uid, [('code','=', prod_wrk_name_acc_code)])
                                
                                #compruebo que existe y adjudico el estimado o no existe y lo creo
                                
                                if prod_wrk_acc:
                                    wrk_acc = account_analytic_obj.browse(cr, uid, prod_wrk_acc)[0]
                                    wrk_acc.total_estimated = total_estimated
                                else:
                                    nombre  = manufact_worker.name.name
                                    values={
                                        'name': prod_wrk_name_acc_code + " " + _('Worker')+ " " + str(nombre).encode('ascii', 'replace'),
                                        'code': prod_wrk_name_acc_code ,
                                        'currency_id': wizard_config.company_id.currency_id.id,
                                        'company_id': wizard_config.company_id.id,
                                        'state': 'open',
                                        'parent_id': man_op_account,
                                        'total_estimated': total_estimated,
                                        'acc_type': 'WK'
                                    }
                                    account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                                    man_wrk_account = account_analytic_obj.create(cr, uid, values)
                                    values['name'] = '|                                              '+values['name']
                                    values['parent_id'] = man_wrk_account
                                    result.append(values)
                                    account_analytic_obj.recalculate_result(cr, uid, man_wrk_account)

                    #Crear cuenta analitica de operadores por inputación
                    
                    usr_obj = self.pool.get('mrp.ops.cost')
                    usr_search = usr_obj.search(cr,uid, [('operation','=',prod_op.id)])
                    usrs = usr_obj.browse(cr, uid, usr_search)
                    for manufact_worker in usrs:                                                                      
                            nombre  = manufact_worker.user.name
                            prod_wrk_name_acc_code = op_acc_code + '-OP' + str(prod_op.sequence) + '-WK'+ str(manufact_worker.user.id) 
                            prod_wrk_acc = account_analytic_obj.search(cr, uid, [('code','=', prod_wrk_name_acc_code)])
                            
                            #compruebo que existe la cuenta analitica de este operario sino la creo
                            
                            if not prod_wrk_acc:
                                values = {
                                    'name': prod_wrk_name_acc_code + " " + _('Operator') + " " + str(nombre).encode('ascii', 'replace'),
                                    'code': prod_wrk_name_acc_code ,
                                    'currency_id': wizard_config.company_id.currency_id.id,
                                    'company_id': wizard_config.company_id.id,
                                    'state':'open',
                                    'parent_id': man_op_account,
                                    'total_estimated': 0,
                                    'total_estimated_puc': 0,
                                    'total_estimated_standard': 0,
                                    'acc_type': 'WK'
                                }
                                account_analytic_obj.calculate_estimated(cr, uid, man_op_account, total_estimated, 0, 0)
                                man_wrk_account = account_analytic_obj.create(cr, uid, values)
                                values['name'] = '|                                              '+values['name']
                                values['parent_id'] = man_wrk_account
                                result.append(values)
                                account_analytic_obj.recalculate_result(cr, uid, man_wrk_account)
        
        #Creacion de lineas analiticas de los datos existentes
        
        ops_op_obj = self.pool.get('mrp.ops.cost')
        ops_wc_obj = self.pool.get('mrp.wc.cost')
        
        #Costes de maquinas
        
        ops_wc_obj_all = ops_wc_obj.browse(cr, uid, ops_wc_obj.search(cr, uid, [('production','=',production_id)]))
        for wc_cost in ops_wc_obj_all:
            wc_cost_amount = wc_cost.wc.costs_hour * wc_cost.time
            wc_acc_code = op_acc_code + '-OP' + str(wc_cost.operation.sequence) + '-MA'
            wc_account_id = False
            for aux_result in result:
                if str(aux_result['code']) == (wc_acc_code):
                    wc_account_id =  aux_result['parent_id']
                    
            if not wc_account_id:
                raise osv.except_osv(_('Programing Error'), _('The analytic account of the machine is not created. Code: %s') %(wc_acc_code))
            
            # TODO: Change this to use code instead for translational option
            # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')]) 
            man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])
            values = {
                      'amount': wc_cost_amount*(-1),
                      'name': wc_cost.descript,
                      'date': wc_cost.date,
                      'company_in': wizard_config.company_id.id,
                      'account_id': wc_account_id,
                      'general_account_id': wizard_config.company_id.workcenter_account.id,
                      'journal_id': man_journal_search[0],
            }
            new_id = account_analytic_line_obj.create(cr, uid, values)          
            account_analytic_obj.recalculate_result(cr, uid, wc_account_id)
            
        #Costes de operarios    
         
        ops_op_obj_all = ops_op_obj.browse(cr, uid, ops_op_obj.search(cr, uid, [('production','=',production_id)]))
        for op_cost in ops_op_obj_all:
            usr_obj = self.pool.get('hr.employee')
            usr_search = usr_obj.search(cr, uid,[('user_id', '=', op_cost.user.id)])
            usr = usr_obj.browse(cr , uid, usr_search[0])
            if not usr.product_id:
                raise osv.except_osv(_('You must assign a product to all employers'), _(usr.name))                
            aux_product = self.pool.get('product.product').browse(cr , uid, usr.product_id.id)
            op_cost_amount = aux_product.standard_price * op_cost.time
            ope_acc_code = op_acc_code + '-OP' + str(op_cost.operation.sequence) + '-WK'+ str(op_cost.user  .id)
            ope_account_id = False
            for aux_result in result:
                if str(aux_result['code']) == (ope_acc_code):
                    ope_account_id =  aux_result['parent_id']
            if not ope_account_id:
                raise osv.except_osv(_('Programing Error'), _('The analytic account of the operator is not created. Code: %s') %(ope_acc_code))
            
            # TODO: Change this to use code instead for translational option
            # man_journal_search = account_analytic_jour_obj.search(cr, uid,[('code', '=', 'MRP')]) 
            man_journal_search = account_analytic_jour_obj.search(cr, uid,[('name', '=', 'Manufacturing')])
            values = {
                      'amount':op_cost_amount*(-1),
                      'name':op_cost.descript,
                      'date':op_cost.date,
                      'company_in': wizard_config.company_id.id,
                      'account_id': ope_account_id,
                      'general_account_id': wizard_config.company_id.operator_account.id,
                      'journal_id':man_journal_search[0],
            }
            new_id = account_analytic_line_obj.create(cr, uid, values)          
            account_analytic_obj.recalculate_result(cr, uid, ope_account_id)
            
#        res.update({'schema': result})   
#        res.update(values)  
    
#        val = {
#            'schema': 'Contabilidad analitica creada'
#        }
        for prod in prod_brow:
            if 'mrp_production' in context:
                context.update({'mrp_production': context['mrp_production'] + ', ' + prod.name})
            else:
                context.update({'mrp_production': prod.name})
        return {'type': 'ir.actions.act_window', 'res_model': 'mrp.analytic.warning', 'view_type': 'form', 'view_mode': 'form', 'target': 'new', 'context':context}
#        return {'type': 'ir.actions.act_window_close'}                       

mrp_analytic_wizard()

class mrp_analytic_warning(osv.osv_memory):
    _name = 'mrp.analytic.warning'
    
    _columns = {
        'mrp_production': fields.char('Production Nº', size=64, readonly=True),
    }
    
    def default_get(self, cr, uid, fields_list, context=None):
        values = {}
        if 'mrp_production' in context:
            values = {
                'mrp_production': context['mrp_production'],
            }
        return values

mrp_analytic_warning()
