# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import osv, fields
import tools
from tools.translate import _
import time
from datetime import datetime

# duplicated signature
#class production_opert_users(osv.osv):
#    _name = 'production.opert.users'
#    _columns = {
#                'name': fields.char('Name', size=54),
#                'user': fields.many2one('res.users','Operator'),
#                'production': fields.many2one('mrp.production', 'Production'),
#                'op': fields.many2one('mrp.production.operation', 'Operation'),
#                'ops_times': fields.one2many('mrp.ops.cost', 'wuser', 'Production Operations'),
#        }
#production_opert_users()


class mrp_production_users(osv.osv):
    _inherit = 'mrp.production.users'
    _name = 'mrp.production.users'

    def delete_it(self, cr, uid, ids, context=None):  
        obj = self.browse(cr, uid, ids, context=context)
        for aux in obj:
            prod_us_op_search = self.pool.get('mrp.ops.cost').search(cr, uid,[('user', '=', aux.name.id ), ('operation', '=', aux.op.id)])

        if prod_us_op_search:
            raise osv.except_osv(_('The operator can not be deleted, has hours charged.'), _(aux.name.name))
        else:
            self.unlink(cr, uid,ids, context=context)
        return False
mrp_production_users()


class mrp_ops_cost(osv.osv):
    """
    Operators cost
    """
    _name = 'mrp.ops.cost'

    def create(self, cr, uid, data, context=None):
        
        if context is None:
            context = {}
            
        aux_prod = self.pool.get('mrp.production')
        aux_prod_op = self.pool.get('mrp.production.operation')
        op = aux_prod_op.browse(cr , uid, context['operation'])
            
        if 'production' in data:
            op = self.pool.get('mrp.production').browse(cr, uid, data['production'])
            values = {
                      'tot': op.tot + data['time'],
                      'tot_ops': op.tot_mac + data['time']
                      }
            op.write(values),
        
        return super(mrp_ops_cost, self).create(cr, uid, data, context)

    _columns = {
          'user': fields.many2one('res.users','Operator'),
          'production': fields.many2one('mrp.production','Production'),
          'operation': fields.many2one('mrp.production.operation', 'Operation', domain=[('production_id','=','production')]),
          'real_op': fields.many2one('mrp.production.operation.real','Real operation'),
          'time': fields.float('Time'),
          'date': fields.date('Date'),
          'descript': fields.text('Description'),   
          'wuser': fields.many2one('production.opert.users','Operator_relation'),
          'create_structure':fields.boolean('Create Structure'),
     }
    _defaults = {'date': datetime.now().strftime('%Y-%m-%d'),
                 'create_structure': True
                }
    
    def onchange_production(self, cr, uid, ids, production_id, context=None):
        res={}     
        if ids:
            ope_cost = self.browse(cr,uid,ids[0])   
            return {'value': {'production': ope_cost.production.id},
                    'warning': {'title': _('Production Error'),
                                'message': _('You can not modify the production, already imputed')
                                }
                    }
        
        return {'value': res}  
    
    def onchange_operation(self, cr, uid, ids, operation_id, context=None):
        operation_obj = self.pool.get('mrp.production.operation')
        operation_real_obj = self.pool.get('mrp.production.operation.real')
        res={}     
        if ids:
            ope_cost = self.browse(cr,uid,ids[0])   
            return {'value': {'operation': ope_cost.operation.id},
                    'warning': {'title': _('Operation Error'),
                                'message': _('You can not modify the operation, already imputed')
                                }
                    }
        else:
            if operation_id:
                operation = operation_obj.browse(cr,uid,operation_id)
                operation_real_ids = operation_real_obj.search(cr, uid, [('operation_id','=',operation.operation_id.id)])
                if not operation_real_ids:
                    return {'value': {'operation': False},
                            'warning': {'title': _('Operation Error'),
                                        'message': _('Real Operation NOT FOUND for this operation')
                                        }
                            }
                else:
                    operation_real = operation_real_obj.browse(cr,uid,operation_real_ids[0])
                    if operation_real.state != 'startworking':
                        raise osv.except_osv(_('Operation Error'), _('Operation must be started to allocate hours')) 
                    else:
                        res = {'real_op': operation_real.id
                               }
                      
        return {'value': res}      
    
    def onchange_user(self, cr, uid, ids, user_id, context=None):
        usr_obj = self.pool.get('hr.employee')
        user_obj = self.pool.get('res.users')
        res={}     
        if ids:
            ope_cost = self.browse(cr,uid,ids[0])   
            return {'value': {'user': ope_cost.user.id},
                    'warning': {'title': _('User Error'),
                                'message': _('You can not modify the user, already imputed')
                                }
                    }
        else:
            if user_id:
                usr_search = usr_obj.search(cr, uid,[('user_id', '=', user_id)])
                if not usr_search:
                    return {'value': {'user': False},
                            'warning': {'title': _('User Error'),
                                        'message': _('The user IS NOT employ')
                                        }
                            }                
                else:
                    usr = usr_obj.browse(cr , uid, usr_search[0])
                    if not usr.product_id:
                        raise osv.except_osv(_('You must assign a product to the employ'))    
                        return {'value': {'user': False},
                                'warning': {'title': _('User Error'),
                                            'message': _('You must assign a product to the emploey')
                                            }
                                }                            
                    
        return {'value': res}
    
    def onchange_date(self, cr, uid, ids, date, context=None):

        res={}     
        if ids:
            ope_cost = self.browse(cr,uid,ids[0])   
            return {'value': {'date': ope_cost.date},
                    'warning': {'title': _('Date Error'),
                                'message': _('You can not modify the date, already imputed')
                                }
                    }                            
                    
        return {'value': res}
    
    
mrp_ops_cost()


class mrp_wc_cost(osv.osv):
    """
    Work center cost
    """
    _name = 'mrp.wc.cost'

    def create(self, cr, uid, data, context=None):
        if 'production' in data:
            op = self.pool.get('mrp.production').browse(cr, uid, data['production'])
            values = {
                      'tot': op.tot + data['time'],
                      'tot_mac': op.tot_mac + data['time']
                      }
            op.write(values),

        return super(mrp_wc_cost, self).create(cr, uid, data, context)

    _columns = {
          'wc': fields.many2one('mrp.workcenter','Work center'),
          'production': fields.many2one('mrp.production','Production'),
          'operation': fields.many2one('mrp.production.operation', 'Operation'),
          'real_op': fields.many2one('mrp.production.operation.real','Real operation'),
          'time': fields.float('Time'),
          'date': fields.date('Date'),
          'descript': fields.text('Description'),
          'create_structure':fields.boolean('Create Structure'),
     }
    _defaults = {'date': datetime.now().strftime('%Y-%m-%d'),
                 'create_structure': True
                }
    
    def onchange_production(self, cr, uid, ids, production_id, context=None):
        res={}     
        if ids:
            ope_cost = self.browse(cr,uid,ids[0])   
            return {'value': {'production': ope_cost.production.id},
                    'warning': {'title': _('Production Error'),
                                'message': _('You can not modify the production, already imputed')
                                }
                    }
        
        return {'value': res}  
    
    def onchange_operation(self, cr, uid, ids, operation_id, context=None):
        operation_obj = self.pool.get('mrp.production.operation')
        operation_real_obj = self.pool.get('mrp.production.operation.real')
        res={}     
        if ids:
            ope_wc = self.browse(cr,uid,ids[0])   
            return {'value': {'operation': ope_wc.operation.id},
                    'warning': {'title': _('Operation Error'),
                                'message': _('You can not modify the operation, already imputed')
                                }
                    }
        else:
            if operation_id:
                operation = operation_obj.browse(cr,uid,operation_id)
                operation_real_ids = operation_real_obj.search(cr, uid, [('operation_id','=',operation.operation_id.id)])
                if not operation_real_ids:
                    return {'value': {'operation': False},
                            'warning': {'title': _('Operation Error'),
                                        'message': _('Real Operation NOT FOUND for this operation')
                                        }
                            }
                else:
                    operation_real = operation_real_obj.browse(cr,uid,operation_real_ids[0])
                    if operation_real.state != 'startworking':
                        raise osv.except_osv(_('Operation Error'), _('Operation must be started to allocate hours')) 
                    else:
                        res = {'real_op': operation_real.id,
                               'wc': operation.workcenter_id.id
                               }
                      
        return {'value': res}      
    
    def onchange_wc(self, cr, uid, ids, operation_id, wc_id, context=None):
        operation_obj = self.pool.get('mrp.production.operation')
        operation_real_obj = self.pool.get('mrp.production.operation.real')
        res={}     
        if ids:
            ope_wc = self.browse(cr,uid,ids[0])   
            return {'value': {'wc': ope_wc.wc.id},
                    'warning': {'title': _('Work Center Error'),
                                'message': _('You can not modify the Work Center, already imputed')
                                }
                    }
        else:
            if operation_id:
                operation = operation_obj.browse(cr,uid,operation_id)
                res = {'wc': operation.workcenter_id.id
                       }
                      
        return {'value': res}   
    
    def onchange_date(self, cr, uid, ids, date, context=None):

        res={}     
        if ids:
            ope_wc = self.browse(cr,uid,ids[0])   
            return {'value': {'date': ope_wc.date},
                    'warning': {'title': _('Date Error'),
                                'message': _('You can not modify the date, already imputed')
                                }
                    }                            
                    
        return {'value': res}
    
mrp_wc_cost()


class production_opert_users(osv.osv):
    _name = 'production.opert.users'
    _columns = {
                'name': fields.char('Name', size=54), 
                'user': fields.many2one('res.users','Operator'),
                'production': fields.many2one('mrp.production', 'Production'),
                'op': fields.many2one('mrp.production.operation', 'Operation'),
                'ops_times': fields.one2many('mrp.ops.cost', 'wuser', 'Production Operations'),
        }
production_opert_users()


class manufacturing_user_op(osv.osv):
    _inherit = 'mrp.production.operation'
    _name = 'mrp.production.operation'
    _columns = {
                'tot': fields.float('Total hr'),
                'tot_ops': fields.float('Total hr operators'),
                'tot_mac': fields.float('Total hr machines'),
                }
    _defaults = {
        'tot': 0,
        'tot_ops': 0,
        'tot_mac': 0,
     }
manufacturing_user_op()


class mrp_production(osv.osv):
    _name = "mrp.production"
    _inherit = "mrp.production"
    _columns = {
                'users_cost':fields.one2many('production.opert.users', 'production', 'Users'),
                'ops_times': fields.one2many('mrp.ops.cost', 'production', 'Users times'),
                'wc_times': fields.one2many('mrp.wc.cost', 'production', 'Work centers'),
                'tot': fields.float('Total'),
                'tot_ops': fields.float('Total Operators'),
                'tot_mac': fields.float('Total Machines'),
                'tot_cost': fields.float('Total cost'),
                'ops_cost': fields.float('Operators cost'),
                'mac_cost': fields.float('Machines cost'),
                }
    _defaults = {
        'tot': 0,
        'tot_ops': 0,
        'tot_mac': 0,
        'tot_cost': 0,
        'ops_cost': 0,
        'mac_cost': 0,
                }
    
    def create(self, cr, uid, data, context=None):
        data.update({'with_manufacturing_cost': True})
        return super(mrp_production, self).create(cr,uid,data,context)
    
mrp_production()


class mrp_production_operation_real(osv.osv):
    _inherit = "mrp.production.operation.real"
    _name = "mrp.production.operation.real"
    _order = "sequence, date_planned"

    _columns = {
#ALLOCATION OF HOUR
       'operators_all_hours':fields.one2many('mrp.ops.cost', 'real_op', 'Operators'),
       'wc_all_hours':fields.one2many('mrp.wc.cost', 'real_op', 'Machines'),
       }

    def action_pause(self, cr, uid, ids):
            """ Sets state to pause.
            @return: True
            """

            delay = 0.0
            date_now = time.strftime('%Y-%m-%d %H:%M:%S')
            obj_line = self.browse(cr, uid, ids[0])

            date_start = datetime.strptime(obj_line.date_start, '%Y-%m-%d %H:%M:%S')

            if obj_line.date_restart:
                date_start = datetime.strptime(obj_line.date_restart, '%Y-%m-%d %H:%M:%S')

            date_finished = datetime.strptime(date_now, '%Y-%m-%d %H:%M:%S')
            delay += (date_finished-date_start).days * 24
            delay += (date_finished-date_start).seconds / float(60*60)

            time_obj = self.pool.get('new.input.time')

            time_obj.input_real_worktime(cr, uid, ids, uid, delay, obj_line.mrp_manu_op.id, date_now,
                                         _('Work in ') + (obj_line.workcenter_id.name or ''))

            self.write(cr, uid, ids, {'state': 'pause', 'date_pause': date_now})

            return True

    def action_done(self, cr, uid, ids):
        """ Sets state to done, writes finish date and calculates delay.
        @return: True
        """
        mrp_production_operation_obj = self.pool.get('mrp.production.operation')
        mrp_production_obj = self.pool.get('mrp.production')
        move_obj = self.pool.get('stock.move')        
        
        tot_time = 0.0
        date_now = time.strftime('%Y-%m-%d %H:%M:%S')
        obj_line = self.browse(cr, uid, ids[0])
        
        #
        ### Creo el albaran de salida
        last_operation = False
        real_operation_ids = self.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                    ('sequence', '>',obj_line.sequence)])
        if not real_operation_ids:
            last_operation = True
            production = mrp_production_obj.browse(cr,uid,obj_line.production_id.id)
            
            production_operation_ids = mrp_production_operation_obj.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                                                     ('sequence', '=',obj_line.sequence)])
            operation = mrp_production_operation_obj.browse(cr,uid,production_operation_ids[0])
            picking_id = mrp_production_obj.create_picking(cr, uid, 'internal', 'direct', production, operation, 'output')
            self.pool.get('stock.picking').write(cr, uid, [picking_id], {'production_id': production.id,
                                                                         'auto_picking': True})
            
            move_new_id = move_obj.create(cr, uid, {
                    'name': 'PROD:' + production.name + '/' + operation.name,
                    'date': production.date_planned,
                    'picking_id': picking_id,
                    'product_id': production.product_id.id,
                    'product_qty': production.real_product_qty,
                    'product_uom': production.product_uom.id,
                    'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                    'product_uos': production.product_uos and production.product_uos.id or False,
                    'location_id': operation.location_id.id,
                    'location_dest_id': production.product_id.default_location.id,
                    'state': 'confirmed',
                    'company_id': production.company_id.id,
                    'operation_id': operation.id,
                    'real_operation_id': obj_line.id,
                    'production_id': production.id
                })
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
        
        date_start = datetime.strptime(obj_line.date_start, '%Y-%m-%d %H:%M:%S')
        date_finished = datetime.strptime(date_now, '%Y-%m-%d %H:%M:%S')
        tot_time += (date_finished - date_start).days * 24
        tot_time += (date_finished - date_start).seconds / float(60 * 60)

        tot_time -= obj_line.paused_time

        if obj_line.date_restart:
                date_start = datetime.strptime(obj_line.date_restart, '%Y-%m-%d %H:%M:%S')

        delay = 0.0
        date_finished = datetime.strptime(date_now, '%Y-%m-%d %H:%M:%S')
        delay += (date_finished-date_start).days * 24
        delay += (date_finished-date_start).seconds / float(60*60)

        time_obj = self.pool.get('new.input.time')

        time_obj.input_real_worktime(cr, uid, ids, uid, delay, obj_line.mrp_manu_op.id, date_now,
                                     _('Work in ') + (obj_line.workcenter_id.name or ''))

        self.write(cr, uid, ids, {'state':'done', 'date_finished': date_now,'delay':tot_time})
        self.modify_production_order_state(cr,uid,ids,'done')
        
        if last_operation:
            real_operation = self.browse(cr, uid, ids[0])
            self._endoperation_generate_products_to_return(cr, uid, real_operation.production_id.id)
        
        return True
    
mrp_production_operation_real()
