# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import time
from datetime import datetime

from osv import osv, fields
import tools
from tools.translate import _


class final_cost(osv.osv_memory):
    _name = "final.cost"
    _description = "Production costs"

    _columns = {
                'tot_cost': fields.float('Total cost'),
                'ops_cost': fields.float('Operators cost'),
                'mac_cost': fields.float('Machines cost'),
                'ec_ops': fields.float('Operators estimated cost'),
                'ec_mac': fields.float('Machines estimated cost'),
                'ec_tot': fields.float('Total estimated cost'),
                'dif': fields.float('Balance of final costs'),
                }

    def default_get(self, cr, uid, fields, context=None):

        if context is None:
            context = {}

        res = super(final_cost, self).default_get(cr, uid, fields, context=context)
        aux = self.pool.get('mrp.production').browse(cr, uid, context['active_ids'][0])

        #Calculo costes reales 

        ops_times = aux.ops_times
        wc_times = aux.wc_times

        ops_tot = 0
        wc_tot = 0
        tot = 0

        for t_ops in ops_times:
            usr_obj = self.pool.get('hr.employee')
            usr_search = usr_obj.search(cr, uid,[('user_id', '=', t_ops.user.id)])
            if len(usr_search) != 1:
                raise osv.except_osv(_('User is not employee'), _(t_ops.user.name))
            usr = usr_obj.browse(cr , uid, usr_search[0])
            if not usr.product_id:
                raise osv.except_osv(_('You must assign a product to all employers'), _(usr.name))
            aux_product = self.pool.get('product.product').browse(cr , uid, usr.product_id.id)
            ops_tot = ops_tot + (t_ops.time*aux_product.standard_price)

        for t_wc in wc_times:
            aux_wc = self.pool.get('mrp.workcenter').browse(cr , uid, t_wc.wc.id) 
            if not aux_wc.product_id:
                raise osv.except_osv(_('You must assign a product to all work centers'), _(aux_wc.name))
            wc_tot = wc_tot + (t_wc.time * aux_wc.costs_hour)   

        tot = ops_tot + wc_tot

        #Calculo costes estimados

        prod_opert = aux.production_operation_ids
        op_users = aux.user_ops

        wc_ec = 0
        ops_ec = 0
        tot_ec = 0

        for ops in prod_opert:
            aux_wc = self.pool.get('mrp.workcenter').browse(cr, uid, ops.workcenter_id.id) 
            wc_ec = wc_ec + (aux_wc.costs_hour * ops.hour)

        for op_usr in op_users:
            usr_obj = self.pool.get('hr.employee')
            usr_search = usr_obj.search(cr, uid,[('user_id', '=', op_usr.name.id)])
            usr = usr_obj.browse(cr, uid, usr_search[0]) 
            aux_product = self.pool.get('product.product').browse(cr, uid, usr.product_id.id) 
            ops_ec = ops_ec + (((op_usr.percent * op_usr.op.hour)/100)*aux_product.standard_price) 

        tot_ec = wc_ec + ops_ec

        dif = tot - tot_ec

        res.update({'tot_cost': tot,  'ops_cost':ops_tot, 'mac_cost':wc_tot, 'ec_mac':wc_ec, 'ec_ops':ops_ec, 'ec_tot':tot_ec, 'dif':dif}),
        return res

final_cost()
