# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

import time
from datetime import datetime

from osv import osv, fields
import tools
from tools.translate import _


class wc_costing(osv.osv_memory):
    _name = "wc.costing"
    _description = "Operation Workcenter"
    _columns = {   
          'wc_id':fields.many2one('mrp.workcenter','Work center'),
          'wc_name':fields.char('Work Center', size=256),
          'time':fields.float('Total time'), 
          'operation':fields.many2one('mrp.production.operation','Operation'),
          'operation_name':fields.char('Operation', size = 256),
    }

    def default_get(self, cr, uid, fields, context=None):

        if context is None:
            context = {}
        res = super(wc_costing, self).default_get(cr, uid, fields, context=context)

        if 'aux_op_id' in context:
            act_id = context['aux_op_id']
        else:
            act_id = context['active_ids'][0]
        aux = self.pool.get('mrp.production.operation').browse(cr , uid, act_id)
        op_name= aux.name
        op_id = aux.id
        prod_id = aux.production_id.id
        production_obj = self.pool.get('mrp.production').browse(cr , uid, prod_id)

#        if production_obj.state == 'draft':
#             raise osv.except_osv(_('Unable to perform action'), _('You cannot impute hours before starting the production order.'))
#
        wc = aux.workcenter_id.id
        aux_wc = self.pool.get('mrp.workcenter').browse(cr , uid, wc)
        wc_name = aux_wc.name

        wc_time = 0

        wc_cost_obj = self.pool.get('mrp.wc.cost')
        costs_ids = wc_cost_obj.search(cr, uid,[('operation', '=', op_id )])
        costs = wc_cost_obj.browse(cr , uid, costs_ids)
        for inputed_time in costs:
            wc_time = wc_time + inputed_time.time

        values = {
                'wc_id': wc,
                'wc_name': wc_name,
                'time': wc_time,
                'operation': op_id,
                'operation_name': op_name
                                  }
        res.update(values)
        return res

wc_costing()


class input_time_wc(osv.osv_memory):
    _name = "input.time.wc"
    _description = "Input time"
    _columns = {
                'wc': fields.many2one('mrp.workcenter','Work center'),
                'wtime': fields.float('time'),
                'descript': fields.text('Description'),
                'work_date': fields.date('Work date'),
                'wc_name': fields.char('Work center', size=256)
                }
    _defaults = {
        'work_date': datetime.now().strftime('%Y-%m-%d'),
                }
 
    def default_get(self, cr, uid, fields, context=None):
        res = super(input_time_wc, self).default_get(cr, uid, fields, context=context)
        values = {
          'wc': context['wc_id'],
          'wc_name': context['wc_name'],
            }
        res.update(values)
        return res

    def input_worktime(self, cr, uid, fields, context=None):
        if context is None:
            context = {}

        if context['wtime'] <= 0:
            raise osv.except_osv(_('The time must be greater than 0'), _(context['wtime']))

        aux = self.pool.get('mrp.production.operation')
        op = aux.browse(cr, uid, context['operation'])

        aux_prod = self.pool.get('mrp.production')
        prod = aux_prod.browse(cr, uid, op.production_id.id)

        ops_wc_obj = self.pool.get('mrp.wc.cost')

        real_op_obj = self.pool.get('mrp.production.operation.real')
        real_op_search = real_op_obj.search(cr, uid, [('mrp_manu_op', '=', op.id )])
        real_id = real_op_obj.browse(cr, uid, real_op_search[0])

#        if real_id.state == 'draft':
#            raise osv.except_osv(_('Unable to perform action'), _('Operation must be started to allocate hours')) 
        values = {
          'wc': context['wc'],
          'operation': context['operation'],
          'time': context['wtime'],
          'date': context['work_date'],
          'descript': context['descript'],
          'production': prod.id,
          'real_op': real_id.id
            }

        res = ops_wc_obj.create(cr, uid, values)

        return {'type': 'ir.actions.act_window_close'}
input_time_wc()


class new_input_time_wc(osv.osv_memory):
    _name = "new.input.time.wc"
    _description = "Input time"
    _columns = {
                'operation':fields.many2one('mrp.production.operation','Operation'),
                'wc':fields.many2one('mrp.workcenter','Work center'),
                'wtime':fields.float('time'),
                'descript':fields.text('Description'),
                'work_date':fields.date('Work date'),
                'wc_name':fields.char('Work center', size=256),
                'production':fields.many2one('mrp.production'),
                }
    _defaults = {
        'work_date': datetime.now().strftime('%Y-%m-%d'),
                }

    def default_get(self, cr, uid, fields, context=None):
        res = super(new_input_time_wc, self).default_get(cr, uid, fields, context=context)

        obj = self.pool.get('mrp.production').browse(cr , uid, context['active_ids'][0])

        if obj.state == 'draft':
            raise osv.except_osv(_('Unable to perform action'), _('You cannot impute hours before starting the production order.')) 

        values = {
          'production':context['active_ids'][0],
            }
        res.update(values)
        return res

    def input_worktime(self, cr, uid, fields, context=None):

        if context is None:
            context = {}

        if context['wtime']<=0:
            raise osv.except_osv(_('The time must be greater than 0'), _(context['wtime']))
        aux = self.pool.get('mrp.production.operation')
        op = aux.browse(cr , uid, context['operation'])
        
        aux_prod = self.pool.get('mrp.production')        
        prod = aux_prod.browse(cr , uid, op.production_id.id)
        
        ops_wc_obj = self.pool.get('mrp.wc.cost')
        wc = op.workcenter_id
        
        real_op_obj = self.pool.get('mrp.production.operation.real')
        real_op_search = real_op_obj.search(cr, uid,[('mrp_manu_op', '=', op.id )])        
        real_id = real_op_obj.browse(cr, uid, real_op_search[0])

#        if real_id.state == 'draft':
#            raise osv.except_osv(_('Unable to perform action'), _('Operation must be started to allocate hours')) 
        
        values = {
          'wc': wc.id,
          'operation':context['operation'],
          'time': context['wtime'],
          'date':context['work_date'],
          'descript':context['descript'],
          'production':prod.id,
          'real_op' : real_id.id
            }
        
        res = ops_wc_obj.create(cr, uid, values)
        

        
        return {'type': 'ir.actions.act_window_close'}
new_input_time_wc()