# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import osv, fields
import tools
import math
from tools.translate import _
import decimal_precision as dp

class bom_fixed_cost(osv.osv):
    _name = 'bom.fixed.cost'
    _description = 'Bom fixed costs'
    _columns = {
                'name': fields.char('Description', size = 256),
                'amount': fields.float('Amount'),
                'estimated': fields.float('Estimated amount'),
                'bom_id': fields.many2one('mrp.bom', 'Bom')
                }
bom_fixed_cost()

class mrp_bom_operations_product(osv.osv):
    _name = 'mrp.bom.operations.product'
    _description = 'materials used to produce'

    def _calculate_time(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        operation = self.pool.get('mrp.bom.operations.product').browse(cr, uid, ids)[0]
        if operation:
            if operation.semi_prod_qty:
                cycle = (operation.semi_prod_qty/operation.capacity_per_cycle)
            else:
                cycle = (operation.bom_id.product_qty/operation.capacity_per_cycle)
            res[operation.id] = {
                    'cycle': cycle,
                    'hour': float(((operation.time_cycle * cycle) + operation.time_start + operation.time_stop) / operation.time_efficiency)
            }
        return res
    
#    def _compute_qty(self, cr, uid, ids, field_name, arg, context=None):
#        id = ids[0]
#        bom_objs = self.pool.get('mrp.bom')
#        operation = self.pool.get('mrp.bom.operations.product').browse(cr, uid, id)
#        bom = bom_objs.browse(cr, uid, bom_objs.search(cr, uid, [('mrp_bom_operations_product_ids', 'in', id)]))[0]
#        if operation.type == 'variable' and operation.semi_prod_qty > 0:
#            return {id: 1 / float(bom.product_qty / operation.semi_prod_qty)}
#        else:
#            return {id: float(operation.semi_prod_qty)}
    
    def create(self, cr, uid, data, context=None):
        if 'name' not in data:
            op = self.pool.get('mrp.operation').browse(cr, uid, data['operation_id'])
            wc = self.pool.get('mrp.workcenter').browse(cr, uid, data['workcenter_id'])
            data['name'] = '%s -> %s' % (op.name, wc.name)
            
        new_id = super(mrp_bom_operations_product, self).create(cr, uid, data, context)
        
        line = self.browse(cr,uid,new_id)
        if line.workcenter_id:
            if line.workcenter_id.user_ids:
                w_num = 0
                for employe in line.workcenter_id.user_ids:
                    w_num = w_num + 1
                self.write(cr,uid,[line.id],{'number_employees': w_num}) 
        
        return new_id

    def _calc_costs_hour(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.workcenter_id:
                    if line.workcenter_id.product_id:
                        res[line.id] = line.workcenter_id.product_id.standard_price
                    else:
                        res[line.id] = 0      
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_costs_cycle(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                w_cost = 0
                if line.workcenter_id:
                    if line.workcenter_id.product_id:
                        w_cost = line.workcenter_id.product_id.standard_price 
                if w_cost > 0:
                    res[line.id] = w_cost * line.time_cycle
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_workcenter_cost(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.semi_prod_qty:
                    cycle = (line.semi_prod_qty/line.capacity_per_cycle)
                else:
                    cycle = (line.bom_id.product_qty/line.capacity_per_cycle)                
                
                if line.workcenter_id:
                    if line.workcenter_id.location_id.usage == 'supplier':
                        res[line.id] = cycle * line.time_cycle
                    else:
                        w_cost = 0
                        if line.workcenter_id.product_id:
                            w_cost = line.workcenter_id.product_id.standard_price 
                        if w_cost > 0:
                            res[line.id] = cycle * w_cost * line.time_cycle
                        else:
                            res[line.id] = 0
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_average_cost_employee(self, cr, uid, ids, name, args, context=None):
        hr_employee_obj = self.pool.get('hr.employee')
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.workcenter_id:
                    if line.workcenter_id.location_id.usage == 'supplier':
                        res[line.id] = 0
                    else:             
                        if line.workcenter_id.user_ids:
                            w_num_emp = 0
                            w_cost = 0
                            for user in line.workcenter_id.user_ids:
                                hr_employee_ids = hr_employee_obj.search(cr, uid, [('user_id','=',user.id)])
                                if hr_employee_ids:
                                    hr_employee = hr_employee_obj.browse(cr,uid,hr_employee_ids [0])
                                    if hr_employee.product_id:
                                        w_num_emp = w_num_emp + 1
                                        w_cost = w_cost + hr_employee.product_id.standard_price
                                        
                            if w_num_emp > 0 and w_cost > 0:
                                res[line.id] = w_cost / w_num_emp
                            else:
                                res[line.id] = 0     
                        else:
                            res[line.id] = 0
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_operators_cost(self, cr, uid, ids, name, args, context=None):
        hr_employee_obj = self.pool.get('hr.employee')
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                w_average_cost = 0
                if line.workcenter_id:
                    if line.workcenter_id.user_ids:
                        w_num_emp = 0
                        w_cost = 0
                        for user in line.workcenter_id.user_ids:
                            hr_employee_ids = hr_employee_obj.search(cr, uid, [('user_id','=',user.id)])
                            if hr_employee_ids:
                                hr_employee = hr_employee_obj.browse(cr,uid,hr_employee_ids [0])
                                if hr_employee.product_id:
                                    w_num_emp = w_num_emp + 1
                                    w_cost = w_cost + hr_employee.product_id.standard_price
                                    
                        if w_num_emp > 0 and w_cost > 0:
                            w_average_cost = w_cost / w_num_emp

                if w_average_cost > 0:
                    res[line.id] = w_average_cost * line.number_employees
                       
        return res
        
    _columns = { 
        'name': fields.char('Name', size = 64, required= True),    
        'sequence': fields.integer('Sequence', required= True),
        'descript': fields.text('Description', size= 64),    
        'product_id': fields.many2one('product.product', 'Product'),
        'semi_prod_qty': fields.float('Product Qty'),
        'semi_prod_uom': fields.many2one('product.uom', 'Product UOM'),
        'semi_prod_qty_fact': fields.float('Quantity Factor'),
        'operation_id': fields.many2one('mrp.operation', 'Operation', required=True),
        'workcenter_id': fields.many2one('mrp.workcenter', 'Work Center', required=True),
        'location_id': fields.many2one('stock.location', 'Operation Location', required= True, domain=[('usage','in',('internal','supplier'))]),
        'capacity_per_cycle': fields.float('Capacity per Cycle'),
        'time_efficiency' : fields.float('Efficiency factor', required=True),
        'time_cycle': fields.float('Time for 1 cycle (hour)', help="Time in hours for doing one cycle of the material."),
        'time_start': fields.float('Time before prod.', help="Time in hours for the setup for the material."),
        'time_stop': fields.float('Time after prod.', help="Time in hours for the cleaning."),
        'costs_hour': fields.function(_calc_costs_hour, method=True, type='float', string='Cost per hour',store=True, help="Specify Cost of Work Center per hour when using this material."),
        'costs_cycle': fields.function(_calc_costs_cycle, method=True, type='float', string='Cost per cycle',store=True, help="Specify Cost of Work Center per cycle when using this material."),
        'cycle': fields.function(_calculate_time, string='Nº Cycles', method=True, type='float',
            store={'mrp.bom.operations.product': (lambda self, cr, uid, ids, c={}: ids, None, 20)}, multi="All"),
        'hour': fields.function(_calculate_time, string='Nº Hour', method=True, type='float',
            store={'mrp.bom.operations.product': (lambda self, cr, uid, ids, c={}: ids, None, 20)}, multi="All"),
        'workcenter_cost': fields.function(_calc_workcenter_cost, method=True, type='float', string='Workcenter Cost',store=True),
        'final_products_ids': fields.one2many('mrp.bom.final.product', 'mrp_bom_operations_product_id', 'Final products'),        
        'components_used_ids': fields.one2many('mrp.bom', 'mrp_bom_operations_product_id', 'Components used'), 
        'bom_id': fields.many2one('mrp.bom', 'Parent BOM', required=True),
        'type': fields.selection([('fixed','Fixed'),('variable','Variable')],'Type'),   
        'modified':fields.boolean('Modified'),
        'by_quantity':fields.boolean('By Quantity'),
        'by_unit':fields.boolean('By Unit'),
        'merma_percentage': fields.float('% Merma'),
        # Numero de Empleados
        'number_employees': fields.float('Number Employees'),
        # Coste medio empleados
        'average_cost_employee': fields.function(_calc_average_cost_employee, method=True, type='float', string='Average Cost Employee',store=True),
        # Coste Empleados
        'operators_cost': fields.function(_calc_operators_cost, method=True, type='float', string='Operators Cost',store=True),
    }

    _defaults = {
        'sequence': lambda *a: 1,
        'time_efficiency': lambda *a: 1,
        'cycle': lambda *a: 1.0,
        'hour': lambda *a: 0.0,
        'type': lambda *a: 'fixed',
        'by_quantity': lambda *a: True
    }

    def default_get(self, cr, uid, fields, context=None):
        if context is None:
            context = {}
        res = super(mrp_bom_operations_product, self).default_get(cr, uid, fields, context)
        if 'routing_id' in context.keys():
            id = self.pool.get('mrp.bom').search(cr, uid, [('name', '=', context['name']), ('routing_id', '=', context['routing_id'])])[0]
            res.update({'bom_id': id})
        return res

    def wc_onchange(self, cr, uid, ids, workcenter_id, operation_id):
        value = {}
        if workcenter_id:
            wc = self.pool.get('mrp.workcenter').browse(cr, uid, workcenter_id)
            op = self.pool.get('mrp.operation').browse(cr, uid, operation_id)
            value['location_id'] = wc.location_id.id
            value['capacity_per_cycle'] = wc.capacity_per_cycle
            value['time_efficiency'] = wc.time_efficiency
            value['time_cycle'] = wc.time_cycle
            value['time_start'] = wc.time_start
            value['time_stop'] = wc.time_stop
            value['costs_hour'] = wc.costs_hour
            value['costs_cycle'] = wc.costs_cycle
            value['name'] = '%s -> %s' % (op.name, wc.name)
        return {'value': value}

    def prod_onchange(self, cr, uid, ids, product_id):
        value = {}
        warning = {}
        if product_id:
            prod = self.pool.get('product.product').browse(cr, uid, product_id)
            if prod.default_location.usage != 'production':
                if isinstance(ids, int):
                    ids = [ids]
                    value.update({
                        'product_id': self.browse(cr, uid, ids[0]).product_id.id,
                    })
                warning = {
                    'title': _("Wrong Product"),
                    'message': _("This is not a valid product. Please, select a product with 'production' location type."),
                }
            else:
                value.update({
                    'semi_prod_qty_fact': 0,
                    'semi_prod_qty': 0,
                    'modified': True,
                    'semi_prod_uom': prod.product_tmpl_id.uom_id.id or False,
                })
        return {'value': value, 'warning': warning}
    
    def onchange_semi_prod_qty(self, cr, uid, ids, operation_id, bom_id, sequence, semi_prod_qty):
        bom_obj = self.pool.get('mrp.bom')
        res={} 
        if not bom_id:
            raise osv.except_osv(_('Error'),_('You must save the list of materials'))
        else:
            if semi_prod_qty == 0:
                res = {'semi_prod_qty_fact': 0}
            else:
                bom = bom_obj.browse(cr,uid,bom_id)
                if sequence == 1:
                    factor = semi_prod_qty / bom.product_qty
                    res = {'semi_prod_qty_fact': factor,
                           'modified': True}       
                else:                 
                    if bom.mrp_bom_operations_product_ids:
                        my_sequence = 0
                        qty = 0
                        for operation in bom.mrp_bom_operations_product_ids:
                            if my_sequence == 0:
                                my_sequence = operation.sequence
                                qty = operation.semi_prod_qty
                            else:
                                if operation.sequence > my_sequence and operation.sequence < sequence:
                                    my_sequence = operation.sequence
                                    qty = operation.semi_prod_qty
                        if sequence == 0:
                            raise osv.except_osv(_('Error'), _('Sequences Operations are wrong(3)'))
                        else:
                            factor = semi_prod_qty / qty
                            res = {'semi_prod_qty_fact': factor,
                                   'modified': True}                           

        return {'value': res}  
    
    def onchange_semi_prod_qty_fact(self, cr, uid, ids, operation_id, bom_id, sequence, semi_prod_qty_fact):
        bom_obj = self.pool.get('mrp.bom')
        res={} 
        if not bom_id:
            raise osv.except_osv(_('Error'),_('You must save the list of materials'))
        else:
            if semi_prod_qty_fact == 0:
                res = {'semi_prod_qty': 0}
            else:
                bom = bom_obj.browse(cr,uid,bom_id)
                if sequence == 1:
                    qty = bom.product_qty * semi_prod_qty_fact
                    res = {'semi_prod_qty': qty,
                           'modified': True}       
                else:                 
                    if bom.mrp_bom_operations_product_ids:
                        my_sequence = 0
                        qty = 0
                        for operation in bom.mrp_bom_operations_product_ids:
                            if my_sequence == 0:
                                my_sequence = operation.sequence
                                qty = operation.semi_prod_qty
                            else:
                                if operation.sequence > my_sequence and operation.sequence < sequence:
                                    my_sequence = operation.sequence
                                    qty = operation.semi_prod_qty
                        if sequence == 0:
                            raise osv.except_osv(_('Error'), _('Sequences Operations are wrong(3)'))
                        else:
                            my_qty = qty * semi_prod_qty_fact
                            res = {'semi_prod_qty': my_qty,
                                   'modified': True}                           

        return {'value': res}  
    
    def onchange_by_quantity_unit(self, cr, uid, ids, by_quantity,by_unit):
        res={} 
        if by_quantity == 1 and by_unit == 1:
            raise osv.except_osv(_('Error'),_('You can not dial per unit and amount at a time'))

        return {'value': res}  
    
    def cycle_onchange(self, cr, uid, ids, time_cycle, costs_hour):
        value = {}
        costh = costs_hour * time_cycle
        value['costs_cycle'] = costh
        return {'value':value}
        
mrp_bom_operations_product()


class mrp_bom_final_product(osv.osv):
    _name = 'mrp.bom.final.product'
    _description = 'Bill of Material'

    _columns = {
        'product_id': fields.many2one('product.product', 'Product', required=True),
        'product_qty': fields.float('Product Qty', required=True),
        'product_uom': fields.many2one('product.uom', 'Product UOM', required=True, help="UoM (Unit of Measure) is the unit of measurement for the inventory control"),
        'mrp_bom_operations_product_id': fields.many2one('mrp.bom.operations.product','Produced On'),
        'location_id': fields.related('product_id', 'default_location', type='many2one', relation='stock.location', string='Default Location', required=True),
        'bom_id': fields.many2one('mrp.bom', 'Final Product'),
    }

    _defaults = {
        'product_qty': lambda *a: 1,
        'product_uom': lambda *a: 1,
    }

    def default_get(self, cr, uid, fields, context=None):
        if context is None:
            context = {}
        res = super(mrp_bom_final_product, self).default_get(cr, uid, fields, context)
        if 'routing_id' in context.keys():
            id = self.pool.get('mrp.bom').search(cr, uid, [('name', '=', context['name']), ('routing_id', '=', context['routing_id'])])
            res.update({'bom_id': id})
        if 'bom_id' in context.keys():
            res.update({'bom_id': context['bom_id']})
        return res

    def prod_onchange(self, cr, uid, ids, product_id):
        value = {}
        if product_id:
            prod = self.pool.get('product.product').browse(cr, uid, product_id)
            value['location_id'] = prod.default_location.id
            value['product_uom'] = prod.product_tmpl_id.uom_id.id or False
        return {'value': value}

mrp_bom_final_product()

def rounding(f, r):
    if not r:
        return f
    return round(f / r) * r


class mrp_bom(osv.osv):
    _description = 'Bill of Material'
    _inherit = 'mrp.bom'

    def write(self, cr, uid, ids, vals, context=None):
        operation_obj = self.pool.get('mrp.bom.operations.product')
        final_product_obj = self.pool.get('mrp.bom.final.product')
        location_obj = self.pool.get('stock.location')
        product_obj = self.pool.get('product.product')
        
        res = super(mrp_bom, self).write(cr, uid, ids, vals, context=context)
        
        #
        ### TRATO LAS OPERACIONES DE FABRICACION PARA CALCULAR CANTIDADES DE PRODUCTO Y FACTOR DE CANTIDAD
        if ids:
            if not isinstance(ids,list):
                ids=[ids]
            for bom in self.browse(cr,uid,ids):
                if bom.mrp_bom_operations_product_ids:
                    for operation in bom.mrp_bom_operations_product_ids:
                           
                        if operation.semi_prod_qty_fact == 0:
                            if operation.sequence == 1:
                                operation_obj.write(cr,uid,[operation.id],{'semi_prod_qty': bom.product_qty,
                                                                           'semi_prod_qty_fact': 1}) 
                            else:
                                operations_ids = operation_obj.search(cr, uid, [('bom_id','=', operation.bom_id.id),
                                                                                ('sequence','<',operation.sequence)])
                                if not operations_ids:
                                    raise osv.except_osv(_('Error'), _('Sequences Operations are wrong(1)'))
                                else:
                                    sequence = 0
                                    qty = 0
                                    for operation2 in operation_obj.browse(cr,uid,operations_ids):
                                        if sequence == 0:
                                            sequence = operation2.sequence
                                            qty = operation2.semi_prod_qty
                                        else:
                                            if operation2.sequence > sequence:
                                                sequence = operation2.sequence
                                                qty = operation2.semi_prod_qty
                                    if sequence == 0:
                                        raise osv.except_osv(_('Error'), _('Sequences Operations are wrong(2)'))
                                    else:
                                        operation_obj.write(cr,uid,[operation.id],{'semi_prod_qty': qty,
                                                                                   'semi_prod_qty_fact': 1})
                        else:
                            if operation.modified == True:
                                operations2_ids = operation_obj.search(cr, uid, [('bom_id','=', operation.bom_id.id),
                                                                                 ('sequence','>',operation.sequence)])
                                if operations2_ids:
                                    for operation2_id in operations2_ids:
                                        operation_obj.write(cr,uid,[operation2_id],{'semi_prod_qty': operation.semi_prod_qty,
                                                                                    'semi_prod_qty_fact': 1}) 
                                operation_obj.write(cr,uid,[operation.id],{'modified': False}) 
                                
                        if operation.merma_percentage:
                            if operation.merma_percentage > 0:
                                location_ids = location_obj.search(cr, uid, [('name','=', 'Scrapped'),
                                                                             ('scrap_location','=', True),
                                                                             ('active','=', True)])
                                if not location_ids:
                                    raise osv.except_osv(_('Error'),_('No encontrada la localizacion DESECHADO'))
                                qty = (operation.semi_prod_qty * operation.merma_percentage) / 100 
                                if operation.product_id:
                                    my_product = operation.product_id.id  
                                else:
                                    my_product = operation.bom_id.product_id.id
                                product = product_obj.browse(cr,uid,my_product)
                                if not operation.final_products_ids:
                                    line = {'mrp_bom_operations_product_id': operation.id,
                                            'bom_id': operation.bom_id.id,
                                            'product_id' : my_product,
                                            'product_qty': qty,
                                            'product_uom': product.uom_id.id,
                                            'location_id': location_ids[0]
                                            }
                                    final_product_id = final_product_obj.create(cr, uid, line)
                                    final_product_obj.write(cr,uid,[final_product_id],{'location_id': location_ids[0]})
                                else:
                                    found = 0
                                    for final_product in operation.final_products_ids:
                                        if final_product.product_id.id == my_product:
                                            found = 1
                                            final_product_obj.write(cr,uid,[final_product.id],{'product_qty': qty}) 
                                    if found == 0:       
                                        if not location_ids:
                                            raise osv.except_osv(_('Error'),_('No encontrada la localizacion DESECHADO'))
                                        line = {'mrp_bom_operations_product_id': operation.id,
                                                'bom_id': operation.bom_id.id,
                                                'product_id' : my_product,
                                                'product_qty': qty,
                                                'product_uom': product.uom_id.id,
                                                'location_id': location_ids[0]
                                                }
                                        final_product_id = final_product_obj.create(cr, uid, line)
                                        final_product_obj.write(cr,uid,[final_product_id],{'location_id': location_ids[0]})
                                                   
        return res

    def create(self, cr, uid, data, context=None):
        operation_obj = self.pool.get('mrp.bom.operations.product')
        final_product_obj = self.pool.get('mrp.bom.final.product')
        location_obj = self.pool.get('stock.location')
        new_id = super(mrp_bom, self).create(cr,uid,data,context)
        bom = self.browse(cr,uid,new_id)
        #
        ### TRATO LAS OPERACIONES DE FABRICACION PARA CALCULAR CANTIDADES DE PRODUCTO Y FACTOR DE CANTIDAD
        bom = self.browse(cr,uid,new_id)
        if bom.mrp_bom_operations_product_ids:
            for operation in bom.mrp_bom_operations_product_ids:
                if operation.semi_prod_qty_fact == 0:
                    if operation.sequence == 1:
                        operation_obj.write(cr,uid,[operation.id],{'semi_prod_qty': bom.product_qty,
                                                                   'semi_prod_qty_fact': 1}) 
                    else:
                        operations_ids = operation_obj.search(cr, uid, [('bom_id','=', operation.bom_id.id),
                                                                        ('sequence','<',operation.sequence)])
                        if not operations_ids:
                            raise osv.except_osv(_('Error'), _('Sequences Operations are wrong(1)'))
                        else:
                            sequence = 0
                            qty = 0
                            for operation2 in operation_obj.browse(cr,uid,operations_ids):
                                if sequence == 0:
                                    sequence = operation2.sequence
                                    qty = operation2.semi_prod_qty
                                else:
                                    if operation2.sequence > sequence:
                                        sequence = operation2.sequence
                                        qty = operation2.semi_prod_qty
                            if sequence == 0:
                                raise osv.except_osv(_('Error'), _('Sequences Operations are wrong(2)'))
                            else:
                                operation_obj.write(cr,uid,[operation.id],{'semi_prod_qty': qty,
                                                                           'semi_prod_qty_fact': 1})
                                
                                
                if operation.merma_percentage:
                    if operation.merma_percentage > 0:
                        location_id = location_obj.search(cr, uid, [('name','=', 'Scrapped')], limit=1)[0]
                        qty = (operation.semi_prod_qty * operation.merma_percentage) / 100
                        if not operation.final_products_ids:
                            if not location_id:
                                raise osv.except_osv(_('Error'),_('Location Scrapped not found'))
                            line = {
                                    'mrp_bom_operations_product_id': operation.id,
                                    'bom_id': operation.bom_id.id,
                                    'product_id': operation.product_id.id,
                                    'product_qty': qty,
                                    'product_uom': operation.product_id.uom_id.id,
                                    'location_id': location_id
                                    }
                            final_product_id = final_product_obj.create(cr, uid, line)
                        else:
                            found = 0
                            for final_product in operation.final_products_ids:
                                if final_product.product_id.id == operation.product_id.id:
                                    found = 1
                                    final_product_obj.write(cr,uid,[final_product.id],{'product_qty': qty}) 
                            if found == 0:       
                                if not location_ids:
                                    raise osv.except_osv(_('Error'),_('No encontrada la localizacion DESECHADO'))
                                line = {
                                        'mrp_bom_operations_product_id': operation.id,
                                        'bom_id': operation.bom_id.id,
                                        'product_id' : operation.product_id.id,
                                        'product_qty': qty,
                                        'product_uom': operation.product_id.uom_id.id,
                                        'location_id': location_id
                                        }
                                final_product_id = final_product_obj.create(cr, uid, line)
                                
        return new_id
    
    def copy(self, cr, uid, id, default=None, context=None):
        if default is None:
            default = {}       
        
        bom_super = super(mrp_bom, self).copy(cr, uid, id, default, context)
        
        original_bom = self.browse(cr, uid, id)
        #copy_bom_id = self.search(cr, uid, [('name','=',(original_bom.name+' Copy'))])[0]
        #copy_bom  = self.browse(cr, uid, copy_bom_id)
        copy_bom  = self.browse(cr, uid, bom_super)
        for bom_aux in copy_bom.bom_lines:
            aux_bom_op = self.pool.get('mrp.bom.operations.product').browse(cr, uid, bom_aux.mrp_bom_operations_product_id)[0]
            for aux_op in bom_aux.mrp_bom_operations_product_ids:
                if aux_op.name == aux_bom_op.name:
                    bom_aux.write({'mrp_bom_operations_product_id':aux_op.id})
        return bom_super	
    
    
    def _check_product(self, cr, uid, ids, context=None):
        all_prod = []
        boms = self.browse(cr, uid, ids, context=context)
        def check_bom(boms):
            res = True
            for bom in boms:
                if bom.product_id.id in all_prod:
                    res = res and False
                if not bom.bom_id:
                    all_prod.append(bom.product_id.id)
                lines = bom.bom_lines
                if lines:
                    res = res and check_bom([bom_id for bom_id in lines if bom_id not in boms])
            return res
        
        return check_bom(boms)
    
    def _calc_cost_price(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = line.product_id.standard_price * line.product_qty
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_standard_price(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = line.product_id.standard_cost * line.product_qty
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_estimated_puc(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = line.product_id.last_purchase_price * line.product_qty
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_list_price(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = line.product_id.list_price * line.product_qty
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_mat_cost_price(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = 0
                else:
                    if not line.bom_lines:
                        res[line.id] = 0
                    else:
                        w_imp = 0
                        for bom_line in line.bom_lines:
                            w_imp = w_imp + bom_line.cost_price
                        res[line.id] = w_imp
                
        return res
    
    def _calc_mat_standard_price(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = 0
                else:
                    if not line.bom_lines:
                        res[line.id] = 0
                    else:
                        w_imp = 0
                        for bom_line in line.bom_lines:
                            w_imp = w_imp + bom_line.standard_price
                        res[line.id] = w_imp
                
        return res
    
    def _calc_mat_estimated_puc(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = 0
                else:
                    if not line.bom_lines:
                        res[line.id] = 0
                    else:
                        w_imp = 0
                        for bom_line in line.bom_lines:
                            w_imp = w_imp + bom_line.estimated_puc
                        res[line.id] = w_imp
                
        return res
    
    def _calc_mat_list_price(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.bom_id:
                    res[line.id] = 0
                else:
                    if not line.bom_lines:
                        res[line.id] = 0
                    else:
                        w_imp = 0
                        for bom_line in line.bom_lines:
                            w_imp = w_imp + bom_line.list_price
                        res[line.id] = w_imp
                
        return res
    
    def _calc_mat_workcenter_cost(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.mrp_bom_operations_product_ids:
                    w_imp = 0
                    for operation_product in line.mrp_bom_operations_product_ids:
                        w_imp = w_imp + operation_product.workcenter_cost
                    res[line.id] = w_imp      
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_mat_operators_cost(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.mrp_bom_operations_product_ids:
                    w_imp = 0
                    for operation_product in line.mrp_bom_operations_product_ids:
                        w_imp = w_imp + operation_product.operators_cost
                    res[line.id] = w_imp      
                else:
                    res[line.id] = 0
                
        return res
    
    _columns = {
                'mrp_bom_operations_product_ids': fields.one2many('mrp.bom.operations.product', 'bom_id', 'Operations'),
                'mrp_bom_operations_product_id': fields.many2one('mrp.bom.operations.product','Consumed On'),
                'final_product_line_ids': fields.one2many('mrp.bom.final.product', 'bom_id', 'Final Product Lines'),
                'routing_id_name': fields.char('Routing',size=64),   
                'fixed_costs':fields.one2many('bom.fixed.cost','bom_id','Fixed costs'),
                'consuming_amount':fields.boolean('Ctad Fija'),
                # Precio de compra para cabecera de lista de materiales
                'mat_cost_price': fields.function(_calc_mat_cost_price, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Cost Price',store=True),
                # Precio de compra para cabecera de lista de materiales
                'mat_standard_price': fields.function(_calc_mat_standard_price, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Standard cost',store=True),
                # Precio de compra para cabecera de lista de materiales
                'mat_estimated_puc': fields.function(_calc_mat_estimated_puc, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Estimated PLP',store=True),
                # Precio de venta para cabecera de lista de materiales
                'mat_list_price': fields.function(_calc_mat_list_price, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Sale Price',store=True),
                # Precio de venta para cabecera de lista de materiales
                'mat_list_price': fields.function(_calc_mat_list_price, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Sale Price',store=True),
                # Precio Coste Maquina para cabecera de lista de materiales
                'mat_workcenter_cost': fields.function(_calc_mat_workcenter_cost, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Workcenters Cost',store=True),
                # Precio Coste Operadores para cabecera de lista de materiales
                'mat_operators_cost': fields.function(_calc_mat_operators_cost, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Operators Cost',store=True),                  
                # Precio de compra para componente
                'cost_price': fields.function(_calc_cost_price, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Cost Price',store=True),
                # Precio standard
                'standard_price': fields.function(_calc_standard_price, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Standard cost',store=True),
                # Estimado PUC
                'estimated_puc': fields.function(_calc_estimated_puc, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Estimated PLP',store=True),
                # Precio de venta para componente
                'list_price': fields.function(_calc_list_price, method=True, digits_compute=dp.get_precision('Purchase Price'), string='Sale Price',store=True),
                }
    
    _constraints = [
        (_check_product, 'BoM line product should not be same as BoM product.', ['product_id']),
        ]
    
    def default_get(self, cr, uid, fields, context=None):
        if context is None:
            context = {}
        res = super(mrp_bom, self).default_get(cr, uid, fields, context)
        if 'routing_id' in context.keys():
            id = self.search(cr, uid, [('name', '=', context['name']), ('routing_id', '=', context['routing_id'])])
            res.update({'bom_id': id})
        if 'bom_id' in context.keys():
            res.update({'bom_id': context['bom_id']})
        return res

    def prod_onchange(self, cr, uid, ids, product_id):
        value = {}
        if product_id:
            prod = self.pool.get('product.product').browse(cr, uid, product_id)
#            value['location_id'] = prod.default_location.id
            value['name'] = prod.name
            value['product_uom'] = prod.product_tmpl_id.uom_id.id or False
        return {'value': value}

    def onchange_routingField(self, cr, uid, ids, routing_id):
        operations_product_obj = self.pool.get('mrp.bom.operations.product')
        if not self.browse(cr, uid, ids):
          raise osv.except_osv(_('User error'), _('Save the record first !'))
        mrp_bom_id = self.browse(cr, uid, ids)[0]
        workcenter_lines_obj = self.pool.get('mrp.production.workcenter.line')
        workcenter_line_ids = workcenter_lines_obj.search(cr, uid, [('id', '=', routing_id)])
        workcenter_line_objs = []

        if len(workcenter_line_ids):
            workcenter_line_objs = workcenter_lines_obj.browse(cr, uid, workcenter_line_ids)

        for operation_product_id in mrp_bom_id.mrp_bom_operations_product_ids:
            operations_product_obj.unlink(cr, uid, [operation_product_id.id])

        values = {
                'sequence': 0,
                'descript': '',
                'operation_id': None,
                'location_id': None,
                'workcenter_id': None,
                'bom_id': mrp_bom_id.id,
                'capacity_per_cycle': 0,
                'time_efficiency': 1,
                'time_cycle': 0,
                'time_start': 0,
                'time_stop': 0,
                'costs_hour': 0,
                'costs_cycle': 0,
        }

        for workcenter_line in workcenter_line_objs:
            values['sequence'] = workcenter_line.sequence
            values['descript'] = workcenter_line.descript
            values['operation_id'] = workcenter_line.operation_id.id
            values['location_id'] = workcenter_line.workcenter_id.location_id.id
            values['workcenter_id'] = workcenter_line.workcenter_id.id
            values['capacity_per_cycle'] = workcenter_line.workcenter_id.capacity_per_cycle
            values['time_efficiency'] = workcenter_line.workcenter_id.time_efficiency
            values['time_cycle'] = workcenter_line.workcenter_id.time_cycle
            values['time_start'] = workcenter_line.workcenter_id.time_start
            values['time_stop'] = workcenter_line.workcenter_id.time_stop
            values['costs_hour'] = workcenter_line.workcenter_id.costs_hour
            values['costs_cycle'] = workcenter_line.workcenter_id.costs_cycle
            values['name'] = '%s -> %s' % (workcenter_line.operation_id.name, workcenter_line.workcenter_id.name)
            res = operations_product_obj.create(cr, uid, values)

        return {}

    def _bom_explode(self, cr, uid, bom, factor, properties=[], addthis=False, level=0):
        """ Finds Products and Work Centers for related BoM for manufacturing order.
        @param bom: BoM of particular product.
        @param factor: Factor of product UoM.
        @param properties: A List of properties Ids.
        @param addthis: If BoM found then True else False.
        @param level: Depth level to find BoM lines starts from 10.
        @return: result: List of dictionaries containing product details.
                 result2: List of dictionaries containing work center details.
        """
        factor = factor / (bom.product_efficiency or 1.0)
        factor = rounding(factor, bom.product_rounding)
        if factor < bom.product_rounding:
            factor = bom.product_rounding
        result = []
        phantom = False
        bom_type = bom.type
        if bom.type == 'phantom' and bom.bom_id: # Not Master Phantom / No control over phanton into phantom 
            phantom = True

        if not phantom:
            if bom.routing_id:
                for op_use in bom.mrp_bom_operations_product_ids:
                    products = []
                    finals = []
                    product_lst = []
                    for product in op_use.components_used_ids:
                        product_lst.append(product)
                    for product in op_use.components_used_ids:
                        if product.type == 'phantom' and product.bom_id:  # product is a not master phantom BOM
                            bom_lst = self.search(cr,uid,[('bom_id','=', product.id)])  # search phantom components BOM
                            phantom = True
                            for bom_id in bom_lst:  # Exchange Phantom product components
                                bom_reg = self.browse(cr, uid, bom_id)
                                if bom_reg.product_uom.rounding == 1.0:
                                    qty = math.ceil(product.product_qty)
                                else:
                                    qty = bom_reg.product_qty
                                if bom_reg.type == 'variable':
                                    qty = qty * factor
                                products.append({
                                        'name': bom_reg.product_id.name,
                                        'product_id': bom_reg.product_id.id,
                                        'product_qty': qty,
                                        'product_uom': bom_reg.product_uom.id,
                                        'product_uos_qty': bom_reg.product_uos and bom.product_uos_qty * factor or False,
                                        'product_uos': bom_reg.product_uos and bom.product_uos.id or False,
                                        'consuming_amount': bom_reg.consuming_amount,
                                        'by_quantity': bom_reg.mrp_bom_operations_product_id.by_quantity,
                                        'by_unit': bom_reg.mrp_bom_operations_product_id.by_unit,
                                        })
                        else:
                            if product.product_uom.rounding == 1.0:
                                qty = math.ceil(product.product_qty)
                            else:
                                qty = product.product_qty
                            if op_use.type == 'variable':
                                qty = qty * factor
                            products.append({
                                'name': product.product_id.name,
                                'product_id': product.product_id.id,
                                'product_qty': qty,
                                'product_uom': product.product_uom.id,
                                'product_uos_qty': product.product_uos and bom.product_uos_qty * factor or False,
                                'product_uos': product.product_uos and bom.product_uos.id or False,
                                'consuming_amount': product.consuming_amount,
                                'by_quantity': product.mrp_bom_operations_product_id.by_quantity,
                                'by_unit': product.mrp_bom_operations_product_id.by_unit,
                            })
                        for final in op_use.final_products_ids:
                            if final.product_uom.rounding == 1.0:
                                qty = math.ceil(final.product_qty)
                            else:
                                qty = final.product_qty
                            if op_use.type == 'variable':
                                qty = qty * factor

                            finals.append({
                                'name': final.product_id.name,
                                'product_id': final.product_id.id,
                                'product_qty': qty,
                                'product_uom': final.product_uom.id,
                                'location_id': final.location_id.id,
                            'by_quantity': final.mrp_bom_operations_product_id.by_quantity,
                            'by_unit': final.mrp_bom_operations_product_id.by_unit,
                            })
                    result.append({
                        'name': tools.ustr(op_use.name),
                        'workcenter_id': op_use.workcenter_id.id,
                        'operation_id': op_use.operation_id.id,
                        'location_id': op_use.location_id.id,
                        'type': op_use.type,
                        'product_id': op_use.product_id.id,
                        'semi_prod_uom': op_use.semi_prod_uom.id,
                        'semi_prod_qty_fact': op_use.semi_prod_qty_fact or 0,
                        'sequence': level + (op_use.sequence or 0),
                        'capacity_per_cycle': op_use.capacity_per_cycle,
                        'time_efficiency': op_use.time_efficiency,
                        'time_cycle': op_use.time_cycle,
                        'time_start': op_use.time_start,
                        'time_stop': op_use.time_stop,
                        'costs_hour': op_use.costs_hour,
                        'costs_cycle': op_use.costs_cycle,
                        'hour': op_use.hour,
                        'cycle': op_use.cycle,
                        'by_quantity': op_use.by_quantity,
                        'by_unit': op_use.by_unit,
                        'semi_prod_qty': op_use.semi_prod_qty,
                        'merma_percentage': op_use.merma_percentage,
                        'products': products,
                        'finals': finals,
                    })
            for bom2 in bom.bom_lines:
                res = self._bom_explode(cr, uid, bom2, factor, properties, addthis=True, level=level+10)
                if res != []:
                    result = result + res
        return result
    
    def onchange_product_id(self, cr, uid, ids, product_id, name, context=None):
        product_obj = self.pool.get('product.product')
        res = super(mrp_bom,self).onchange_product_id(cr,uid,ids,product_id,name,context)
        if res.get('value'):
            value = res.get('value')
            product = product_obj.browse(cr,uid,product_id)
            value.update({'cost_price': product.standard_price,
                          'standard_price': product.standard_cost,
                          'estimated_puc': product.last_purchase_price,
                          'list_price': product.list_price
                          })
            res.update({'value': value})

        return res
    
    def onchange_product_qty(self, cr, uid, ids, product_id, product_qty, context=None):
        product_obj = self.pool.get('product.product')
        res={}
        if product_id and product_qty:
            product = product_obj.browse(cr,uid,product_id)
            res = {'cost_price': product.standard_price * product_qty,
                   'standard_price': product.standard_cost * product_qty,
                   'estimated_puc': product.last_purchase_price * product_qty,
                   'list_price': product.list_price * product_qty
                   }

        return {'value': res}  
    
    def onchange_bom_lines(self, cr, uid, ids, bom_lines, product_qty, context=None):
        res={}
        w_cost_price = 0
        w_standard_price = 0
        w_list_price = 0
        w_estimated_puc = 0
        for bom_line in bom_lines:
            bomline = bom_line[2]
            if bomline:
                if bomline.get('cost_price'):
                    w_cost_price = w_cost_price + bomline.get('cost_price')
                if bomline.get('standard_price'):
                    w_standard_price = w_standard_price + bomline.get('standard_price')
                if bomline.get('list_price'):
                    w_list_price = w_list_price + bomline.get('list_price')
                if bomline.get('estimated_puc'):
                    w_estimated_puc = w_estimated_puc + bomline.get('estimated_puc')
                    
        res = {'mat_cost_price': w_cost_price,
               'mat_standard_price': w_standard_price,
               'mat_estimated_puc': w_estimated_puc,
               'mat_list_price': w_list_price,
               }

        return {'value': res}  

mrp_bom()
