# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import fields, osv
from tools.translate import _


class mrp_workcenter(osv.osv):
    _description = 'Work Center Inheritance'
    _inherit = 'mrp.workcenter'
    
    def _calc_costs_hour(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.product_id:
                    res[line.id] = line.product_id.standard_price        
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_costs_cycle(self, cr, uid, ids, name, args, context=None):
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                w_cost = 0
                if line.product_id:
                    w_cost = line.product_id.standard_price        
                if w_cost > 0:
                    res[line.id] = w_cost * line.time_cycle
                else:
                    res[line.id] = 0
                
        return res
    
    def _calc_average_cost_employee(self, cr, uid, ids, name, args, context=None):
        hr_employee_obj = self.pool.get('hr.employee')
        res = {}
        if ids:
            for line in self.browse(cr,uid,ids, context=context):
                if line.user_ids:
                    w_num_emp = 0
                    w_cost = 0
                    for user in line.user_ids:
                        hr_employee_ids = hr_employee_obj.search(cr, uid, [('user_id','=',user.id)])
                        if hr_employee_ids:
                            hr_employee = hr_employee_obj.browse(cr,uid,hr_employee_ids [0])
                            if hr_employee.product_id:
                                w_num_emp = w_num_emp + 1
                                w_cost = w_cost + hr_employee.product_id.standard_price
                                
                    if w_num_emp > 0 and w_cost > 0:
                        res[line.id] = w_cost / w_num_emp
                    else:
                        res[line.id] = 0     
                else:
                    res[line.id] = 0
                
        return res

    _columns = {'costs_hour': fields.function(_calc_costs_hour, method=True, type='float', string='Cost per hour',store=True, help="Specify Cost of Work Center per hour."),
                'costs_cycle': fields.function(_calc_costs_cycle, method=True, type='float', string='Cost per cycle',store=True, help="Specify Cost of Work Center per cycle."),
                'average_cost_employee': fields.function(_calc_average_cost_employee, method=True, type='float', string='Average Cost Employee',store=True, ),
                'location_id': fields.many2one('stock.location', 'Location', required= True, domain=[('usage','in',('internal','supplier'))]),
                'purchase_date': fields.date('Purchase Date', size = 32, help = "The machine's purchase date"),
                'warranty_date': fields.date('Warranty Date', size = 32, help = "Warranty limit date of the Work Center"),
                'maintain_notes': fields.text('Maintain Notes', readonly = False),
                'alter_workcenter_ids': fields.many2many('mrp.workcenter','alter_workcenter','parent_workcenter_id','child_workcenter_id','Alternative Work center'),
                'mrp_op_ids': fields.many2many('mrp.operation', 'mrp_operations_workcenter', 'workcenter_id', 'operation_id','Operations', domain=[('type','=','mfo')]),
                'maintain_op_id': fields.many2one('mrp.operation', 'Maintain Operation', size = 32, domain=[('type','=','mto')]),
                'factory_id': fields.many2one('mrp.factory', 'Factory', required= True),
                'operations_product_ids': fields.one2many('mrp.bom.operations.product', 'workcenter_id', 'Work Center', required= True),
                'user_ids': fields.many2many('res.users', 'mrp_wc_user_rel', 'workcenter_id', 'user_id'),
            }

    def _check_location_product(self, cr, uid, ids, context=None):
        for wc in self.browse(cr, uid, ids, context=context):
            product_id = wc.product_id and wc.product_id.id
            location_usage = wc.location_id and wc.location_id.usage
            if location_usage == 'supplier' and not product_id:
                return False
        return True

    _constraints = [
        (_check_location_product, 'If location usage is of type supplier then product cant be null.', ['location_id','product_id']),
        ]

    def cycle_onchange(self, cr, uid, ids, time_cycle, costs_hour):
        value = {}
        costh = costs_hour * time_cycle
        value['costs_cycle'] = costh
        return {'value':value}

    def default_get(self, cr, uid, fields_list, context=None):
        if context == None:
            context = {}
        res = super(mrp_workcenter, self).default_get(cr, uid, fields_list, context)
        if 'factory' in context.keys():
            if context['factory']:
                ids = self.pool.get('mrp.factory').search(cr, uid, [('name', '=', context['factory'])])
                res.update({'factory_id': ids[0]})
        return res

mrp_workcenter()