# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import osv, fields
import decimal_precision as dp
from tools.translate import _
import netsvc


class stock_picking(osv.osv):
    _inherit = 'stock.picking'
    _columns = {
            'internal_type': fields.selection([
                                               ('input', 'Production Input'),
                                               ('output', 'Production Output')
                                              ], 'Type'),
    }
stock_picking()


class StockMove(osv.osv):
    _inherit = 'stock.move'
    _columns = {
        'operation_id': fields.many2one('mrp.production.operation', 'Production Operation'),
        'production_id2': fields.many2one('mrp.production', 'Production'),
        'bom_prop': fields.float('BOM Proportion'),
        'type': fields.related('picking_id', 'type', type="selection", selection=[('out', 'Sending Goods'), ('in', 'Getting Goods'), ('internal', 'Internal')], store=True, readonly=True)
    }

    _defaults = {
        'bom_prop': lambda *a: 1.0,
    }

    #funcion basada en action_consume de stock.move

    def action_produce(self, cr, uid, ids, quantity, location_id=False, context=None):
        if context is None:
            context = {}
        if quantity <= 0:
            raise osv.except_osv(_('Warning!'), _('Please provide Proper Quantity !'))
        res = []
        for move in self.browse(cr, uid, ids, context=context):
            move_qty = move.product_qty
            if move_qty <= 0:
                raise osv.except_osv(_('Error!'), _('Can not produce a move with negative or zero quantity !'))
            quantity_rest = move.product_qty
            quantity_rest -= quantity
            uos_qty_rest = quantity_rest / move_qty * move.product_uos_qty
            if quantity_rest <= 0:
                quantity_rest = 0
                uos_qty_rest = 0
                quantity = move.product_qty

            uos_qty = quantity / move_qty * move.product_uos_qty

            if quantity_rest > 0:
                default_val = {
                    'product_qty': quantity,
                    'product_uos_qty': uos_qty,
                    'state': move.state,
                    'location_id': location_id or move.location_id.id,
                }
                if (not move.prodlot_id.id) and (move.product_id.track_production and location_id):
                    # IF product has checked track for production lot, move lines will be split by 1
                    res += self.action_split(cr, uid, [move.id], quantity, split_by_qty=1, context=context)
                else:
                    current_move = self.copy(cr, uid, move.id, default_val)
                    res += [current_move]
                update_val = {}
                update_val['product_qty'] = quantity_rest
                update_val['product_uos_qty'] = uos_qty_rest
                self.write(cr, uid, [move.id], update_val)

            else:
                quantity_rest = quantity
                uos_qty_rest = uos_qty
                if (not move.prodlot_id.id) and (move.product_id.track_production and location_id):
                    res += self.action_split(cr, uid, [move.id], quantity_rest, split_by_qty=1, context=context)
                else:
                    res += [move.id]
                    update_val = {
                        'product_qty': quantity_rest,
                        'product_uos_qty': uos_qty_rest,
                        'location_id': location_id or move.location_id.id
                    }
                    self.write(cr, uid, [move.id], update_val)

            product_obj = self.pool.get('product.product')
            for new_move in self.browse(cr, uid, res, context=context):
                for (id, name) in product_obj.name_get(cr, uid, [new_move.product_id.id]):
                    message = _('Product ') + " '" + name + "' "+ _("is consumed with") + " '" + str(new_move.product_qty) + "' "+ _("quantity.")
                    self.log(cr, uid, new_move.id, message)
        self.action_done(cr, uid, res)

        return res
    
    # ESTA FUNCION VIENE DEL MODULO MRP
    def action_consume(self, cr, uid, ids, product_qty, location_id=False, context=None): 
        """ Consumed product with specific quatity from specific source location.
        @param product_qty: Consumed product quantity
        @param location_id: Source location
        @return: Consumed lines
        """       
        res = []
        move_obj = self.pool.get('stock.move')
        production_obj = self.pool.get('mrp.production')
        wf_service = netsvc.LocalService("workflow")
        for move in self.browse(cr, uid, ids):
            new_moves = move_obj.action_consume2(cr, uid, [move.id], product_qty, location_id, context=context)
            production_ids = production_obj.search(cr, uid, [('move_lines', 'in', [move.id])])
            for prod in production_obj.browse(cr, uid, production_ids, context=context):
                if prod.state == 'confirmed':
                    production_obj.force_production(cr, uid, [prod.id])
                wf_service.trg_validate(uid, 'mrp.production', prod.id, 'button_produce', cr)
            for new_move in new_moves:
                if new_move == move.id:
                    #This move is already there in move lines of production order
                    continue
                production_obj.write(cr, uid, production_ids, {'move_lines': [(4, new_move)]})
                res.append(new_move)
        return res
    
    # Esta funcion viene de stock y se llamva action_consume
    def action_consume2(self, cr, uid, ids, quantity, location_id=False, context=None):
        """ Consumed product with specific quatity from specific source location
        @param cr: the database cursor
        @param uid: the user id
        @param ids: ids of stock move object to be consumed
        @param quantity : specify consume quantity
        @param location_id : specify source location
        @param context: context arguments
        @return: Consumed lines
        """
        if context is None:
            context = {}
        if quantity <= 0:
            raise osv.except_osv(_('Warning!'), _('Please provide Proper Quantity !'))
        res = []
        product_obj = self.pool.get('product.product')
        for move in self.browse(cr, uid, ids, context=context):
            move_qty = move.product_qty
            if move_qty <= 0:
                raise osv.except_osv(_('Error!'), _('Can not consume a move with negative or zero quantity !'))
            quantity_rest = move.product_qty
            quantity_rest -= quantity
            uos_qty_rest = quantity_rest / move_qty * move.product_uos_qty
            if quantity_rest <= 0:
                quantity_rest = 0
                uos_qty_rest = 0
                quantity = move.product_qty

            uos_qty = quantity / move_qty * move.product_uos_qty

            if quantity_rest > 0:
                default_val = {
                    'product_qty': quantity,
                    'product_uos_qty': uos_qty,
                    'state': move.state,
                    'location_id': location_id or move.location_id.id,
                }
                if (not move.prodlot_id.id) and (move.product_id.track_production and location_id):
                    # IF product has checked track for production lot, move lines will be split by 1
                    res += self.action_split(cr, uid, [move.id], quantity, split_by_qty=quantity, context=context)
                else:
                    current_move = self.copy(cr, uid, move.id, default_val)
                    res += [current_move]
                update_val = {}
                update_val['product_qty'] = quantity_rest
                update_val['product_uos_qty'] = uos_qty_rest
                self.write(cr, uid, [move.id], update_val)

            else:
                quantity_rest = quantity
                uos_qty_rest =  uos_qty
                if (not move.prodlot_id.id) and (move.product_id.track_production and location_id):
                    res += self.action_split(cr, uid, [move.id], quantity_rest, split_by_qty=quantity_rest, context=context)
                else:
                    res += [move.id]
                    update_val = {
                        'product_qty' : quantity_rest,
                        'product_uos_qty' : uos_qty_rest,
                        'location_id': location_id or move.location_id.id
                    }
                    self.write(cr, uid, [move.id], update_val)

            for new_move in self.browse(cr, uid, res, context=context):
                for (id, name) in product_obj.name_get(cr, uid, [new_move.product_id.id]):
                    message = _('Product ') + " '" + name + "' "+ _("is consumed with") + " '" + str(new_move.product_qty) + "' "+ _("quantity.")
                    self.log(cr, uid, new_move.id, message)
        self.action_done(cr, uid, res)

        return res

StockMove()
