# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
import time

class delete_ops(osv.osv_memory):
    _name = "delete.ops"
    _description = "Delete"
    _columns = {
                'op_us': fields.many2one('mrp.production.users','User to delete'),
                'user_name': fields.char('Are you sure you want to decouple worker from operation  :::>  ',size=64)
                }

    def default_get(self, cr, uid, fields, context=None):

        if context is None:
            context = {}

        res = super(delete_ops, self).default_get(cr, uid, fields, context=context)

        obj = self.pool.get('mrp.production.users').browse(cr , uid, context['op_us'])
        values = {
                  'op_us': obj.id,
                  'user_name': obj.name.name
                  }
        res.update(values)
        return res

    def delete_user(self, cr, uid, fields, context=None):
        obj = self.pool.get('mrp.production.users').browse(cr, uid, context['op_us'])
        obj.delete_it()
        return {'type': 'ir.actions.act_window_close'}

delete_ops()


class delete_choice(osv.osv_memory):

    _name = "delete.choice"
    _description = "Choice user to delete"
    _columns = {
                'op_us': fields.many2one('mrp.production.users', 'User to delete'),
                'ava':fields.char('Available ids', size=256),
                }

    def default_get(self, cr, uid, fields, context=None):

        if context is None:
            context = {}

        result = []
        prod_us_op_search = self.pool.get('mrp.production.users').search(cr, uid,[('op', '=', context['op'] )])
        prod_us = self.pool.get('mrp.production.users').browse(cr , uid, prod_us_op_search)
        aux = self.pool.get('mrp.production.operation').browse(cr , uid, context['op'])

        if prod_us:
            for us_obj in prod_us:
                if us_obj.op.id == aux.id:
                    result.append(us_obj.id)

        ava = str(result)

        res = super(delete_choice, self).default_get(cr, uid, fields, context=context)
        values = {
                  'ava':ava,
                  }
        res.update(values)
        return res

delete_choice()

class manufacturing_user(osv.osv_memory):
    _name = "manufacturing.user"
    _description = "Manufacturing new user"

    _columns = {
          'user': fields.many2one('res.users', 'User to add'),
          'percent':fields.float('Estimated %'),
          'est_time':fields.float('Estimated time')
     }
    _default = {
        'percent': 0,
        'est_time': 0
     }

    def add_user(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        objs = self.pool.get('mrp.production').browse(cr , uid, context['active_ids'])
        objs2 = self.pool.get('mrp.production.users')

        con_user = []
        con_ops = []
        for obj in objs:
            for usr_op in obj.user_ops:
                con_user.append((usr_op.name.id, usr_op.op.id))
            obj_ops = self.pool.get('mrp.production.operation')
            ops_ids = obj_ops.search(cr, uid,[('production_id', '=', obj.id )])
            con_ops = obj_ops.browse(cr , uid, ops_ids)

        usr = context['user']
        percent = context['percent']

        if not usr:
            raise osv.except_osv(_('Unable to perform action'), _('You must select Operator'))

        for check in con_ops:
            create = True
            for check_op in con_user:
                if check_op == (usr, check.id):
                    create = False
            if create:
                est_time = check.hour * percent /100
                values = {
                        'name': usr,
                        'op':check.id,
                        'prod':context['active_ids'][0],
                        'percent': percent,
                        'est_time':est_time,
                        }

                res = objs2.create(cr, uid, values)

        return {'type': 'ir.actions.act_window_close'}

manufacturing_user()

class aux_res_users(osv.osv_memory):

    _name = 'aux.res.users'
    _description = "Operation users"

    _columns = {
        'est_time':fields.float('Estimated time'),
        'user_id':fields.many2one('res.users', 'user id'),
        'name':fields.char('Name', size=64),
        'percent':fields.float('% Estimated'),
        'wizard_id': fields.many2one('manufacturing.user.op', 'wizard', required=True),
        'op':fields.many2one('mrp.production.operation','prod user')
     }

aux_res_users()

class manufacturing_user_op(osv.osv_memory):
    _name = "manufacturing.user.op"
    _description = "Manufacturing new user"

    _columns = {
          'user': fields.one2many('aux.res.users', 'wizard_id', 'Users added'),
          'production':fields.many2one('mrp.production', 'Production'),
          'operation':fields.many2one('mrp.production.operation', 'Operation'),
     }

    def default_get(self, cr, uid, fields, context=None):

        if context is None:
            context = {}
        res = super(manufacturing_user_op, self).default_get(cr, uid, fields, context=context)
        aux = self.pool.get('mrp.production.operation').browse(cr , uid, context['active_ids'][0])

        op_id = aux.id
        prod_id = aux.production_id.id

        obj = self.pool.get('mrp.production').browse(cr , uid, prod_id)
        obj2_id= self.pool.get('mrp.production.users').search(cr, uid,[('op', '=', op_id)])
        obj2= self.pool.get('mrp.production.users').browse(cr, uid,obj2_id)

        result = []
        if obj:
            for prod in obj2:
                        values = {
                                  'user_id': prod.name.id,
                                  'name':prod.name.name,
                                  'percent':prod.percent,
                                  'op':op_id,
                                  'est_time':prod.est_time
                                  }
                        result.append(values)
        else:
            raise osv.except_osv(_('there is no production'), _(obj))
        res.update({'user': result , 'production':prod_id, 'operation':op_id})   
        return res

manufacturing_user_op()


class manufacturing_user_op_add(osv.osv_memory):
    _name = "manufacturing.user.op.add"
    _description = "Manufacturing new user"

    _columns = {
          'user': fields.many2one('res.users', 'User to add'),
          'percent': fields.float('% Estimated'),
          'est_time': fields.float('Estimated time'),
          'op':fields.many2one('mrp.production.operation')
     }

    def default_get(self, cr, uid, fields, context=None):

        if context is None:
            context = {}
        res = super(manufacturing_user_op_add, self).default_get(cr, uid, fields, context=context)
        res.update({'op':context['operation']})
        return res

    def est_time_on_change(self, cr, uid, ids, est_time, op, context=None):
        objs = self.pool.get('mrp.production.operation').browse(cr , uid, op)
        tot_time = objs.hour
        etime_list = est_time.rsplit(':')
        etime = float(etime_list[0])+(float(etime_list[1])/60)+(float(etime_list[2])/360)
        percent = (float(etime) * 100) / tot_time
        return {'value': {'percent':percent}}

    def percent_on_change(self, cr, uid, ids, percent,  op, context=None):
        objs = self.pool.get('mrp.production.operation').browse(cr , uid, op)
        tot_time = objs.hour
        est_time_op = (percent / 100) * tot_time
        h = int(est_time_op) 
        est_time_op =(est_time_op -h)* 60
        m= int(est_time_op)
        s = int((est_time_op-m)*60)

        if h < 10:
            h = "0" + str(h)
        if m < 10:
            m = "0" + str(m)
        if s < 10:
            s = "0" + str(s)

        est_time = str(h) + ":" + str(m) + ":" + str(s)

        return {'value': {'est_time': est_time}}

    def add_user_op(self, cr, uid, ids, context=None):

        if context is None:
            context = {}

        production = context['production']

        objs = self.pool.get('mrp.production').browse(cr, uid, production)

        usr = context['user']
        percent = context['percent']

        objs2    = self.pool.get('mrp.production.users')
        objs2_id = objs2.search(cr, uid,[('prod', '=', [production])])
        objs2_s  = objs2.browse(cr , uid, objs2_id)

        con_user = []

        for usr_op in objs2_s:
            con_user.append((usr_op.name.id, usr_op.op.id)) 
        check = context['operation']
        st_est_time = context['est_time']

        est_time_list = st_est_time.rsplit(':')
        est_time = float(est_time_list[0])+(float(est_time_list[1])/60)+(float(est_time_list[2])/360)

        create = True

        for check_op in con_user:
            if check_op == (usr, check):
                    create = False
        if create:
            values = {
                           'name': usr,
                           'op': check,
                           'prod': production,
                           'percent': percent,
                           'est_time': est_time
                           }
            res = objs2.create(cr, uid, values)

        return {'type': 'ir.actions.act_window_close'}

manufacturing_user_op_add()
