# -*- encoding: utf-8 -*-
##############################################################################
#
#    Avanzosc - Avanced Open Source Consulting
#    Copyright (C) 2011 - 2013 Avanzosc <http://www.avanzosc.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################
from osv import osv
from osv import fields
import decimal_precision as dp
from tools.translate import _

class mrp_bom(osv.osv):
    
    _name = 'mrp.bom'
    _inherit = 'mrp.bom'
    
    def _calc_sum_qty(self, cr, uid, ids, name, args, context=None):
        
        res = {}
        if ids:
            for lm in self.browse(cr, uid, ids):
                if lm.generate_bysumqty == True:
                    if lm.bom_lines:
                        sum = 0
                        for line in lm.bom_lines:
                            if lm.product_uom.id == line.product_uom.id:
                                sum = sum + line.product_qty
                        res[lm.id] = sum
                    else:
                        res[lm.id] = 0
                else:
                    res[lm.id] = 0

        return res 
     
    _columns = {'sum_qty': fields.function(_calc_sum_qty, method=True, type='float', digits_compute=dp.get_precision('Product UoM'), string='Sum Qty'),
                'generate_bysumqty':fields.boolean('Generate by sum Qty'),
                'product_uom2': fields.many2one('product.uom', 'Product UOM'),
                }
    _defaults = {'generate_bysumqty': lambda self, cr, uid, c: c.get('generate_bysumqty', False),
                 'product_uom2': lambda self, cr, uid, c: c.get('product_uom2', False),
                 }
    
    
    def onchange_product_id_bysumqty(self, cr, uid, ids, product_id, name, generate_bysumqty, product_uom2, context=None):
        product_obj = self.pool.get('product.product')
        if context is None:
            context = {}
            context['lang'] = self.pool.get('res.users').browse(cr,uid,uid).context_lang
        if product_id and product_uom2:
            product = product_obj.browse(cr, uid, product_id, context=context)
#            if generate_bysumqty == True:
#                if product_uom2 != product.uom_id.id:
#                    raise osv.except_osv(_('Product Error'),_('The product must have the same unit of measurement as the product to produce'))

        
        return super(mrp_bom,self).onchange_product_id(cr, uid, ids, product_id, name, context)
    
    def write(self, cr, uid, ids, vals, context=None):
        operations_product_obj = self.pool.get('mrp.bom.operations.product')
        bom_obj = self.pool.get('mrp.bom')
        res = super(mrp_bom, self).write(cr, uid, ids, vals, context=context)
        #
        ### TRATO LAS OPERACIONES DE FABRICACION PARA CALCULAR CANTIDADES DE PRODUCTO Y FACTOR DE CANTIDAD
        if ids:
            if not isinstance(ids,list):
                ids=[ids]
            for bom in self.browse(cr,uid,ids):
                if bom.generate_bysumqty == True:
                    if bom.mrp_bom_operations_product_ids:
                        for operation in bom.mrp_bom_operations_product_ids:
                            operations_product_ids = operations_product_obj.search(cr, uid, [('bom_id','=', operation.bom_id.id),
                                                                                             ('sequence','<',operation.sequence)])
                            if not operations_product_ids:
                                sum_qty = 0
                                if not operation.components_used_ids:
                                    operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': 0,
                                                                                        'semi_prod_qty_fact': 0})
                                else:
                                    #for component in bom_obj.browse(cr,uid,operation.components_used_ids):
                                    for component in operation.components_used_ids:
                                        sum_qty = sum_qty + component.product_qty
                                    if sum_qty == 0:
                                        fact = 0
                                    else:
                                        fact = sum_qty / bom.product_qty
                                    operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': sum_qty,
                                                                                        'semi_prod_qty_fact': fact})
                            else:
                                w_sequence = 0
                                w_qty_anterior = 0
                                sum_qty = 0
                                for component in operations_product_obj.browse(cr,uid,operations_product_ids):
                                    if w_sequence == 0:
                                        w_sequence = component.sequence
                                        w_qty_anterior = component.semi_prod_qty
                                    else:
                                        if component.sequence > w_sequence:
                                            w_sequence = component.sequence  
                                            w_qty_anterior = component.semi_prod_qty  
                                if not operation.components_used_ids:
                                    operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': w_qty_anterior,
                                                                                        'semi_prod_qty_fact': 1})
                                else:
                                    
                                    #for component in bom_obj.browse(cr,uid,operation.components_used_ids):
                                    for component in operation.components_used_ids:
                                        sum_qty = sum_qty + component.product_qty
                                    sum_qty = sum_qty + w_qty_anterior
                                    if sum_qty == 0:
                                        fact = 0
                                    else:
                                        fact = sum_qty / bom.product_qty
                                    operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': sum_qty,
                                                                                        'semi_prod_qty_fact': fact})    
                                    
        if ids:
            if not isinstance(ids,list):
                ids=[ids]
            for bom in self.browse(cr,uid,ids):
                if bom.bom_lines:
                    for line in bom.bom_lines:
                        self.write(cr,uid,[line.id],{'generate_bysumqty': bom.generate_bysumqty,
                                                     'product_uom2': bom.product_uom.id})    
                        
                                                                                
        return res

    def create(self, cr, uid, data, context=None):
        operations_product_obj = self.pool.get('mrp.bom.operations.product')
        bom_obj = self.pool.get('mrp.bom')
        new_id = super(mrp_bom, self).create(cr, uid, data, context=context)
        
        #
        ### TRATO LAS OPERACIONES DE FABRICACION PARA CALCULAR CANTIDADES DE PRODUCTO Y FACTOR DE CANTIDAD
        bom = self.browse(cr,uid, new_id)
        if bom.generate_bysumqty == True:
            if bom.mrp_bom_operations_product_ids:
                for operation in bom.mrp_bom_operations_product_ids:
                    operations_product_ids = operations_product_obj.search(cr, uid, [('bom_id','=', operation.bom_id.id),
                                                                                     ('sequence','<',operation.sequence)])
                    if not operations_product_ids:
                        sum_qty = 0
                        if not operation.components_used_ids:
                            operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': 0,
                                                                                'semi_prod_qty_fact': 0})
                        else:
                            #for component in bom_obj.browse(cr,uid,operation.components_used_ids):
                            for component in operation.components_used_ids:
                                sum_qty = sum_qty + component.product_qty
                            if sum_qty == 0:
                                fact = 0
                            else:
                                fact = sum_qty / bom.product_qty
                            operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': sum_qty,
                                                                                'semi_prod_qty_fact': fact})
                    else:
                        w_sequence = 0
                        w_qty_anterior = 0
                        sum_qty = 0
                        for component in operations_product_obj.browse(cr,uid,operations_product_ids):
                            if w_sequence == 0:
                                w_sequence = component.sequence
                                w_qty_anterior = component.semi_prod_qty
                            else:
                                if component.sequence > w_sequence:
                                    w_sequence = component.sequence  
                                    w_qty_anterior = component.semi_prod_qty  
                        if not operation.components_used_ids:
                            operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': w_qty_anterior,
                                                                                'semi_prod_qty_fact': 1})
                        else:
                            
                            #for component in bom_obj.browse(cr,uid,operation.components_used_ids):
                            for component in operation.components_used_ids:
                                sum_qty = sum_qty + component.product_qty
                            sum_qty = sum_qty + w_qty_anterior
                            if sum_qty == 0:
                                fact = 0
                            else:
                                fact = sum_qty / bom.product_qty
                            operations_product_obj.write(cr,uid,[operation.id],{'semi_prod_qty': sum_qty,
                                                                                'semi_prod_qty_fact': fact})            
                                
        return new_id
    

mrp_bom()
