# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2011 Ting & Avanzosc (http://www.openerpsite.com) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
import time
import decimal_precision as dp
import netsvc

class produce_bysumqty(osv.osv_memory):
    _name = "produce.bysumqty"
    _description = "Produce By Sum Qty"
    
    _columns = {'operation_real_id':fields.many2one('mrp.production.operation.real', 'Real Operation', readonly=True),
                'product_qty': fields.float('Quantity', digits_compute=dp.get_precision('Product UoM')),
                'consumed_ids': fields.one2many('parcial.consumed.aux', 'wizard_id', 'Consumed Moves', readonly=True),
                }
    
    def do_produce_bysumqty(self, cr, uid, ids, context = None):  
        operation_real_obj = self.pool.get('mrp.production.operation.real') 
        operation_obj = self.pool.get('mrp.production.operation')
        move_obj = self.pool.get('stock.move')
        production_line_obj = self.pool.get('mrp.production.product.line')
        partial_produce_obj = self.pool.get('partial.produce')
        mrp_production_obj = self.pool.get('mrp.production')
        picking_obj = self.pool.get('stock.picking')
        product_obj = self.pool.get('product.product')
        if ids:
            for line in self.browse(cr,uid,ids):
                real_operation = operation_real_obj.browse(cr,uid,line.operation_real_id.id)
                if line.product_qty > 0:
                    w_partial_product_id = 0
                    w_partial_product_qty = 0
                    w_partial_semi_prod_move = 0
                    w_partial_semi_prod_loc = 0
                    w_production_id = 0
                    w_operation_id = 0
                    for finished in real_operation.move_created_ids:
                        if (finished.product_id.default_location.usage == 'production') or (finished.product_id.id == real_operation.product.id):
                            w_partial_product_id = finished.product_id.id
                            if real_operation.production_id.with_generate_bysumqty:
                                if real_operation.production_id.bom_id.generate_bysumqty == True:
                                    w_partial_product_qty = line.product_qty
                                else:
                                    w_partial_product_qty = finished.product_qty
                            else:
                                w_partial_product_qty = finished.product_qty 
                            w_partial_semi_prod_move = finished.id
                            w_partial_semi_prod_loc = finished.location_id.id
                        else:
                            move_obj.action_consume(cr, uid, [finished.id] ,(finished.bom_prop * line.product_qty),finished.location_id.id,context=context)
                            if finished.move_dest_id:
                                move_obj.write(cr, uid, [finished.move_dest_id.id], {'state': 'waiting'})
                                if finished.operation_id:
                                    if finished.operation_id.production_id:
                                        w_operation_id = finished.operation_id.sequence
                                        w_production_id = finished.operation_id.production_id.id
                   
                    if w_partial_semi_prod_move > 0:
                        move_obj.action_consume(cr, uid, [w_partial_semi_prod_move] , w_partial_product_qty, w_partial_semi_prod_loc,context=context)
                        partial_semi_prod_move = move_obj.browse(cr,uid,w_partial_semi_prod_move)
                        if not w_partial_product_id == partial_semi_prod_move.real_operation_id.product.id:
                            move_obj.write(cr, uid, [partial_semi_prod_move.move_dest_id.id], {'state': 'waiting'})
                            self._mrp_split_worker(cr, uid, ids, line, partial_semi_prod_move, w_partial_product_qty, context)                                        
            
        return {'type': 'ir.actions.act_window_close'}
    
    def _mrp_split_worker(self, cr, uid, ids, obj, part_move, w_partial_product_qty,context=None):
        move_obj = self.pool.get('stock.move')
        picking_obj = self.pool.get('stock.picking')
        part_dest = None
        assigned_found = False
        qty_runout = False
        qty = 0
        if part_move.move_dest_id.real_operation_ids:
            part_dest = part_move.move_dest_id.real_operation_ids[0]
        else:
            part_dest = part_move.move_dest_id.picking_id
        if not part_dest:
            return False
        
        
        for move in part_dest.move_lines:
            if (move.product_id.id == part_move.product_id.id) and (move.state == 'assigned'):
                assigned_found = True
        if not assigned_found:
            move = move_obj.action_split(cr, uid, [part_move.move_dest_id.id], w_partial_product_qty, w_partial_product_qty, False, False, context=context)
            move_obj.write(cr, uid, move, {'state': 'assigned'})
        for move in part_dest.move_lines:
            if (move.product_id.id == part_move.product_id.id) and (move.state == 'assigned'):
                qty = move.product_qty + obj.partial_product_qty
                move_obj.write(cr, uid, move.id, {'product_qty': qty})
            elif (move.product_id.id == part_move.product_id.id) and (move.state in ('waiting')):
                qty = move.product_qty - w_partial_product_qty
                if not qty:
                    qty_runout = True
                else:
                    move_obj.write(cr, uid, move.id, {'product_qty': qty})
        if qty_runout and assigned_found:
            move_obj.write(cr, uid, [part_move.move_dest_id.id], {'state': 'draft'})
            move_obj.unlink(cr, uid, [part_move.move_dest_id.id])
        return True
  
 
    def default_get(self, cr, uid, fields, context=None):
        real_operation_obj = self.pool.get('mrp.production.operation.real')
        result = []
        res = {}
        if context is None:
            context = {}  
        finished_product_qty = 0
        real_operation = real_operation_obj.browse(cr,uid,context['operation_real_id'])
        if real_operation.move_lines2:
            for line in real_operation.move_lines2:
                finished_product_qty = finished_product_qty + line.product_qty
                values = {
                     'stock_move_id': line.id,
                     'product_id':line.product_id.id,
                     'product_qty': line.product_qty,
                     'bom_prop': line.bom_prop,
                     'product_uom': line.product_id.uom_id.id,
                     'location_id': line.location_id.id,
                }
                result.append(values)
            
        res = {'consumed_ids': result,
               'operation_real_id': context['operation_real_id'],
               'product_qty': finished_product_qty
               }
        return res
    
produce_bysumqty()
    
class parcial_consumed_aux(osv.osv_memory):
    _name = "parcial.consumed.aux"
    
    _columns = {
            'stock_move_id':fields.many2one('stock.move', 'Stock move', required=True, select=True),
            'product_id':fields.many2one('product.product', 'Product', required=True, select=True),
            'product_qty': fields.float('Qty', required=True),
            'bom_prop': fields.float('BOM Proportion'),
            'product_uom': fields.many2one('product.uom', 'UOM', required=True),
            'location_id': fields.many2one('stock.location', 'Location', required=True),
            'wizard_id': fields.many2one('produce.bysumqty', 'wizard'),
            }
    
parcial_consumed_aux()
