# -*- encoding: utf-8 -*-
##############################################################################
#
#    Avanzosc - Avanced Open Source Consulting
#    Copyright (C) 2011 - 2013 Avanzosc <http://www.avanzosc.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see http://www.gnu.org/licenses/.
#
##############################################################################
from osv import osv
from osv import fields
from tools.translate import _
import decimal_precision as dp
import netsvc
import time
from datetime import datetime
from dateutil.relativedelta import relativedelta

class mrp_production_operation_real(osv.osv):

    _inherit = 'mrp.production.operation.real'

    _columns = {'production_id': fields.many2one('mrp.production', 'Work Order', readonly=True),
                # Lista de Operaciones
                'operation_real_ids':fields.one2many('mrp.production.operation.real','operation_real_id','Operations'),
                # Super Operación
                'operation_real_id':fields.many2one('mrp.production.operation.real', 'Super Operation'),
                # Es una Super Operación
                'super_operation':fields.boolean('Super Operation', readonly=True),
                # Productos a consumir en una super operacion
                'products_to_consume_ids':fields.one2many('super.operation.product.consume','mrp_production_operation_real_id','Products To Consume', domain=[('consumed','=',False)]),
                # Productos a consumir en una super operacion
                'products_consumed_ids':fields.one2many('super.operation.product.consume','mrp_production_operation_real_id','Products Consumed', domain=[('consumed','=',True)]),
                # Productos a consumir en una super operacion
                'products_to_finish_ids':fields.one2many('super.operation.product.finish','mrp_production_operation_real_id','Products To Finish', domain=[('finished','=',False)]),
                # Productos a consumir en una super operacion
                'products_finished_ids':fields.one2many('super.operation.product.finish','mrp_production_operation_real_id','Products Finished', domain=[('finished','=',True)]),                
                }
    
    def button_prepare_start_working(self, cr, uid, ids, context={}):
        if ids:
            for operation in self.browse(cr,uid,ids):
                if operation.operation_real_id: 
                    raise osv.except_osv(_('Error'), _('You can not Init this operation, because this operation is assigned to Super Operation: %s') % (operation.operation_real_id.name))       
                else:
                    if operation.super_operation == False:
                        wf_service = netsvc.LocalService("workflow")
                        wf_service.trg_validate(uid, 'mrp.production.operation.real', operation.id, 'button_start_working', cr) 
                    else:
                        if operation.operation_real_ids:
                            for real in operation.operation_real_ids:
                                wf_service = netsvc.LocalService("workflow")
                                wf_service.trg_validate(uid, 'mrp.production.operation.real', real.id, 'button_start_working', cr)
                            realoperation = self.browse(cr,uid,operation.id)
                            w_found = 0
                            for real in realoperation.operation_real_ids:
                                if real.state != 'startworking':
                                    w_found = 1
                            if w_found == 0:
                                wf_service = netsvc.LocalService("workflow")
                                wf_service.trg_validate(uid, 'mrp.production.operation.real', operation.id, 'button_start_working', cr)
                                
           
                
        return True
    
    def button_prepare_done(self, cr, uid, ids, context={}):
        if ids:
            for operation in self.browse(cr,uid,ids):
                if operation.operation_real_id: 
                    raise osv.except_osv(_('Error'), _('You can not Finish this operation, because this operation is assigned to Super Operation: %s') % (operation.operation_real_id.name))       
                else:
                    if operation.super_operation == False:
                        wf_service = netsvc.LocalService("workflow")
                        wf_service.trg_validate(uid, 'mrp.production.operation.real', operation.id, 'button_done', cr) 
                    else:
                        if operation.operation_real_ids:
                            for real in operation.operation_real_ids:
                                wf_service = netsvc.LocalService("workflow")
                                wf_service.trg_validate(uid, 'mrp.production.operation.real', real.id, 'button_done', cr)
                            realoperation = self.browse(cr,uid,operation.id)
                            w_found = 0
                            for real in realoperation.operation_real_ids:
                                if real.state != 'done':
                                    w_found = 1
                            if w_found == 0:
                                wf_service = netsvc.LocalService("workflow")
                                wf_service.trg_validate(uid, 'mrp.production.operation.real', operation.id, 'button_done', cr)
                                
           
                
        return True

    def button_partial_produce_superoperation(self, cr, uid, ids, context=None):
        value = True
        if context.has_key('real_operation_id'):
            operation = self.browse(cr,uid,context.get('real_operation_id'))
            if operation.operation_real_id:
                raise osv.except_osv(_('Error'), _('You can not Produce in  this operation, because this operation is assigned to Super Operation: %s') % (operation.operation_real_id.name))
            else:
                if operation.super_operation == False:
                    context.update({'active_ids': [operation.id]})
                    context.update({'active_model': 'mrp.production.operation.real'})
                    value = {'view_type': 'form',
                             'view_mode': 'form',
                             'res_model': 'partial.produce',
                             'context': context,
                             'type': 'ir.actions.act_window',
                             'nodestroy': True, 
                             'target':'new',
                             } 
                else:
                    self._partial_produce_superoperation(cr, uid, operation.id,context=context)
        else:
            operation = self.browse(cr,uid,context.get('active_id'))
            if operation.operation_real_id:
                raise osv.except_osv(_('Error'), _('You can not Produce in  this operation, because this operation is assigned to Super Operation: %s') % (operation.operation_real_id.name))
            else:
                context.update({'active_ids': operation.id})
                context.update({'active_model': 'mrp.production.operation.real'})
                value = {'view_type': 'form',
                         'view_mode': 'form',
                         'res_model': 'partial.produce',
                         'context': context,
                         'type': 'ir.actions.act_window',
                         'nodestroy': True,
                         'target':'new',
                         } 
                    
        return value
    
    def _partial_produce_superoperation(self, cr, uid, super_operation_id,context=None):
        partial_produce_obj = self.pool.get('partial.produce')
        move_obj = self.pool.get('stock.move')
        consume_product_obj = self.pool.get('super.operation.product.consume')
        finish_product_obj = self.pool.get('super.operation.product.finish')
        if super_operation_id:
            real_operation = self.browse(cr,uid,super_operation_id)
            if not real_operation.products_to_consume_ids:
                raise osv.except_osv(_('Error!'),_('Already produced in this operation'))
                
            # Trato las operaciones para calcular cantidades de productos
            if real_operation.operation_real_ids:
                for operation in real_operation.operation_real_ids:
                    context.update({'active_ids': [operation.id]})
                    context.update({'active_model': 'mrp.production.operation.real'})
                    res = partial_produce_obj.default_get(cr, uid, fields, context=context)

                    for consumed in res.get('partial_consumed_ids'):
                        move_obj.action_consume(cr, uid, [consumed.get('stock_move_id')] ,consumed.get('product_qty'),consumed.get('location_id'),context=context)
                    for finished in res.get('partial_finished_ids'):
                        stockmove = move_obj.browse(cr,uid,finished.get('stock_move_id'))
                        if finished.get('product_qty') <= stockmove.product_qty:
                            move_obj.action_consume(cr, uid, [stockmove.id] ,finished.get('product_qty'),finished.get('location_id'),context=context)
                            if stockmove.move_dest_id:
                                move_obj.write(cr, uid, stockmove.move_dest_id.id, {'state': 'waiting'})
                        else:
                            raise osv.except_osv(_('Error!'),_('Cannot produce more than scheduled in this operation'))
        
                    if res.has_key('partial_semi_prod_move'):
                        if res.get('partial_semi_prod_move') > 0:
                            partial_semi_prod_move = move_obj.browse(cr,uid,res.get('partial_semi_prod_move'))
                            if res.get('partial_product_qty') <= partial_semi_prod_move.product_qty:
                                if res.get('partial_product_qty') <= partial_semi_prod_move.move_dest_id.product_qty or res.get('partial_product_id') == partial_semi_prod_move.real_operation_id.product.id:
                                    move_obj.action_consume(cr, uid, [partial_semi_prod_move.id] , res.get('partial_product_qty'), res.get('partial_semi_prod_loc'),context=context)
                                    if not res.get('partial_product_id') == partial_semi_prod_move.real_operation_id.product.id:
                                        move_obj.write(cr, uid, partial_semi_prod_move.move_dest_id.id, {'state': 'waiting'})
                                        self.mrp_split(cr, uid, ids, obj, partial_semi_prod_move, context)     
                                else:
                                    raise osv.except_osv(_('Error!'),_('Cannot produce more than scheduled in the next operation'))
                            else:
                                raise osv.except_osv(_('Error!'),_('Cannot produce more than scheduled in this operation'))     
                        
                    real_operation = self.browse(cr,uid,res.get('partial_real_operation_id'))
                    real_operation_ids = self.search(cr, uid,[('production_id', '=', real_operation.production_id.id),
                                                              ('sequence','>', real_operation.sequence)])  
                    if not real_operation_ids:
                        if real_operation.production_id:
                            production = mrp_production_obj.browse(cr,uid,real_operation.production_id.id)
                            qty = 0
                            if not production.real_product_qty:
                                qty = obj.partial_product_qty
                            else:
                                qty = obj.partial_product_qty + production.real_product_qty
                            mrp_production_obj.write(cr, uid, [production.id], {'real_product_qty':qty})                     

                for consume in real_operation.products_to_consume_ids:
                    consume_product_obj.write(cr, uid, consume.id, {'consumed': True})
                    
                for finish in real_operation.products_to_finish_ids:
                    finish_product_obj.write(cr, uid, finish.id, {'finished': True})
                                        
        return True
    
    def action_done(self, cr, uid, ids):
        
        """ Sets state to done, writes finish date and calculates delay.
        @return: True
        """
        mrp_production_operation_obj = self.pool.get('mrp.production.operation')
        mrp_production_obj = self.pool.get('mrp.production')
        move_obj = self.pool.get('stock.move')
        
        tot_time = 0.0
        date_now = time.strftime('%Y-%m-%d %H:%M:%S')
        obj_line = self.browse(cr, uid, ids[0])
        
        if not obj_line.move_lines2 and obj_line.super_operation == False:   
            raise osv.except_osv(_('Error'), _('You can not finish before produce'))
        
        #
        ### Creo el albaran de salida
        last_operation = False
        if obj_line.production_id:
            real_operation_ids = self.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                        ('sequence', '>',obj_line.sequence)])
            if not real_operation_ids:
                last_operation = True
                production = mrp_production_obj.browse(cr,uid,obj_line.production_id.id)
                
                production_operation_ids = mrp_production_operation_obj.search(cr, uid, [('production_id','=',obj_line.production_id.id),
                                                                                         ('sequence', '=',obj_line.sequence)])
                operation = mrp_production_operation_obj.browse(cr,uid,production_operation_ids[0])
                picking_id = mrp_production_obj.create_picking(cr, uid, 'internal', 'direct', production, operation, 'output')
                self.pool.get('stock.picking').write(cr, uid, [picking_id], {'production_id': production.id,
                                                                             'auto_picking': True})
                
                move_new_id = move_obj.create(cr, uid, {
                        'name': 'PROD:' + production.name + '/' + operation.name,
                        'date': production.date_planned,
                        'picking_id': picking_id,
                        'product_id': production.product_id.id,
                        'product_qty': production.real_product_qty,
                        'product_uom': production.product_uom.id,
                        'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                        'product_uos': production.product_uos and production.product_uos.id or False,
                        'location_id': operation.location_id.id,
                        'location_dest_id': production.product_id.default_location.id,
                        'state': 'confirmed',
                        'company_id': production.company_id.id,
                        'operation_id': operation.id,
                        'real_operation_id': obj_line.id,
                        'production_id': production.id
                    })
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr) 
        
        date_start = datetime.strptime(obj_line.date_start,'%Y-%m-%d %H:%M:%S')
        
        date_finished = datetime.strptime(date_now,'%Y-%m-%d %H:%M:%S')
        tot_time += (date_finished-date_start).days * 24
        tot_time += (date_finished-date_start).seconds / float(60*60)
        
        tot_time -=     obj_line.paused_time       
        
        if obj_line.date_restart:
                date_start = datetime.strptime(obj_line.date_restart,'%Y-%m-%d %H:%M:%S')
        
        delay = 0.0
        
        date_finished = datetime.strptime(date_now,'%Y-%m-%d %H:%M:%S')
        delay += (date_finished-date_start).days * 24
        delay += (date_finished-date_start).seconds / float(60*60)              
            
            
        time_obj = self.pool.get('new.input.time')           
        
        if obj_line.super_operation == False:
            time_obj.input_real_worktime(cr, uid, ids, uid, delay, obj_line.mrp_manu_op.id, date_now, 'Work in '+str(obj_line.workcenter_id.name))
        
        self.write(cr, uid, ids, {'state':'done', 'date_finished': date_now,'delay':tot_time})
        self.modify_production_order_state(cr,uid,ids,'done')
        
                #Añadido link a contabilidad analitica para costes de materiales
        
        if obj_line.production_id:
            if not obj_line.production_id.company_id:
                raise osv.except_osv(_('Error'), _('You have to assign a company to this work order!'))
            else:
                self.pool.get('mrp.analytic.wizard').mats_final_cost(cr, uid, ids[0], obj_line.production_id.company_id.id)
        
        if last_operation and obj_line.super_operation == False:
            real_operation = self.browse(cr, uid, ids[0])
            self._endoperation_generate_products_to_return(cr, uid, real_operation.production_id.id)
        
        return True
    
    def check_sent_pick(self, cr, uid, ids):
        if ids:
            for operation in self.browse(cr,uid,ids):
                if operation.super_operation == True:
                    return True
                else:
                    return super(mrp_production_operation_real,self).check_sent_pick(cr, uid, ids)
    
    def check_received_pick(self, cr, uid, ids):
        if ids:
            for operation in self.browse(cr,uid,ids):
                if operation.super_operation == True:
                    return True
                else:
                    return super(mrp_production_operation_real,self).check_received_pick(cr, uid, ids)
                
    def modify_production_order_state(self, cr, uid, ids, action):
        """ Modifies production order state if work order state is changed.
        @param action: Action to perform.
        @return: Nothing
        """
        wf_service = netsvc.LocalService("workflow")
        oper_obj = self.browse(cr, uid, ids)[0]
        prod_obj = oper_obj.production_id
        if action == 'start': 
            if not prod_obj:
                return
            else:
                if prod_obj.state =='confirmed':
                    if prod_obj:
                        self.pool.get('mrp.production').force_production(cr, uid, [prod_obj.id])
                        wf_service.trg_validate(uid, 'mrp.production', prod_obj.id, 'button_produce', cr)
                elif prod_obj.state =='ready':
                    if prod_obj:
                        wf_service.trg_validate(uid, 'mrp.production', prod_obj.id, 'button_produce', cr)
                elif prod_obj.state =='in_production':
                    return
                else:
                    raise osv.except_osv(_('Error!'),_('Production Order Cannot start in [%s] state') % (prod_obj.state,))
        elif action == 'ready':
            if prod_obj:
                wf_service.trg_validate(uid, 'mrp.production', prod_obj.id, 'is_ready', cr)
        else:
            oper_ids = self.search(cr,uid,[('production_id','=',prod_obj.id)])
            obj = self.browse(cr,uid,oper_ids)
            flag = True
            for line in obj:
                if line.state not in ('done', 'received'):
                     flag = False
            if flag:
                if oper_obj.production_id:
                    wf_service.trg_validate(uid, 'mrp.production', oper_obj.production_id.id, 'button_produce_done', cr)
        return
    
    def _create_super_operation_products(self, cr, uid, super_operation_id):
        product_consume_obj = self.pool.get('super.operation.product.consume')
        product_finish_obj = self.pool.get('super.operation.product.finish')
        if super_operation_id:
            real_operation = self.browse(cr,uid,super_operation_id)
            # Borro productos a consumir, consumidos, finalizados, y a finalizar
            if real_operation.products_to_consume_ids:
                for product in real_operation.products_to_consume_ids:
                    product_consume_obj.unlink(cr,uid,product.id)
            if real_operation.products_consumed_ids:
                for product in real_operation.products_consumed_ids:
                    product_consume_obj.unlink(cr,uid,product.id)
            if real_operation.products_to_finish_ids:
                for product in real_operation.products_to_finish_ids:
                    product_finish_obj.unlink(cr,uid,product.id)
            if real_operation.products_finished_ids:
                for product in real_operation.products_finished_ids:
                    product_finish_obj.unlink(cr,uid,product.id)
            # Trato las operaciones para calcular cantidades de productos
            if real_operation.operation_real_ids:
                for operation in real_operation.operation_real_ids:
                    if operation.move_lines:
                        #
                        ### Cantidades a Consumir
                        for move_line in operation.move_lines:
                            if move_line.state != 'assigned':
                                raise osv.except_osv(_('Error'), _('Real Operation Code: %s. The product %s, is not READY') % (operation.code, move_line.product_id.name))
                            else:
                                realoperation = self.browse(cr,uid,super_operation_id)
                                if not realoperation.products_to_consume_ids:
                                    data = {'mrp_production_operation_real_id': super_operation_id,
                                            'product_id': move_line.product_id.id,
                                            'product_qty': move_line.product_qty,
                                            'product_uom': move_line.product_uom.id,
                                            'consumed': False }
                                    product_consume_obj.create(cr,uid,data)
                                else:
                                    w_found = 0
                                    for product in realoperation.products_to_consume_ids:
                                        if product.product_id.id == move_line.product_id.id and product.product_uom.id == move_line.product_uom.id:
                                            w_found = 1
                                            qty = product.product_qty + move_line.product_qty
                                            product_consume_obj.write(cr, uid, product.id, {'product_qty': qty})
                                            break
                                    if w_found == 0:
                                        data = {'mrp_production_operation_real_id': super_operation_id,
                                                'product_id': move_line.product_id.id,
                                                'product_qty': move_line.product_qty,
                                                'product_uom': move_line.product_uom.id,
                                                'consumed': False }
                                        product_consume_obj.create(cr,uid,data)
                        #
                        ### Cantidades a Finalizar
                        for move_line in operation.move_created_ids:
                            realoperation = self.browse(cr,uid,super_operation_id)
                            if not realoperation.products_to_finish_ids:
                                data = {'mrp_production_operation_real_id': super_operation_id,
                                        'product_id': move_line.product_id.id,
                                        'product_qty': move_line.product_qty,
                                        'product_uom': move_line.product_uom.id,
                                        'finished': False }
                                product_finish_obj.create(cr,uid,data)
                            else:
                                w_found = 0
                                for product in realoperation.products_to_finish_ids:
                                    if product.product_id.id == move_line.product_id.id and product.product_uom.id == move_line.product_uom.id:
                                        w_found = 1
                                        qty = product.product_qty + move_line.product_qty
                                        product_finish_obj.write(cr, uid, product.id, {'product_qty': qty})
                                        break
                                if w_found == 0:
                                    data = {'mrp_production_operation_real_id': super_operation_id,
                                            'product_id': move_line.product_id.id,
                                            'product_qty': move_line.product_qty,
                                            'product_uom': move_line.product_uom.id,
                                            'finished': False }
                                    product_finish_obj.create(cr,uid,data)
                                        
        return True
    
    def button_super_operation_calculate_products(self, cr, uid, ids, *args):
        if ids:
            real_operation = self.browse(cr,uid,ids[0])
            if real_operation.state != 'ready':
                raise osv.except_osv(_('Error'), _('The Real Operation is not READY'))
            else:              
                self._create_super_operation_products(cr, uid, ids[0])
        
        return True 
        
      
mrp_production_operation_real()
